/*
 * Copyright 2008  Rivo Laks <rivolaks@hot.ee>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "designer.h"

#include "asteroidgen.h"

#include <QApplication>
#include <QFileDialog>
#include <KComponentData>

int main(int argc, char *argv[])
{
    QApplication app(argc, argv);
    new KComponentData("astrododge");

    Designer d;
    d.show();

    return app.exec();
}


#define generatorValueToUi(val)  ui.val->setValue(mGenerator->val)
#define uiValueToGenerator(val)  mGenerator->val = ui.val->value()

Designer::Designer()
{
    ui.setupUi(this);

    connect(ui.updateButton, SIGNAL(clicked()), this, SLOT(generate()));
    connect(ui.saveButton, SIGNAL(clicked()), this, SLOT(save()));

    mGenerator = new AsteroidGen();
    mGenerator->generate();

    ui.model->setGenerator(mGenerator);

    generatorValueToUi(lodCount);
    generatorValueToUi(seed);
    generatorValueToUi(randomInitial);
    generatorValueToUi(randomIterative);
    generatorValueToUi(randomDamping);
    generatorValueToUi(randomPost);
}

void Designer::generate()
{
    // Sync options from gui to generator
    uiValueToGenerator(lodCount);
    uiValueToGenerator(seed);
    uiValueToGenerator(randomInitial);
    uiValueToGenerator(randomIterative);
    uiValueToGenerator(randomDamping);
    uiValueToGenerator(randomPost);

    // Generate
    mGenerator->generate();

    // Update modelviewer
    ui.model->update();
}

void Designer::save()
{
    QString fileName = QFileDialog::getSaveFileName(this, "Save Model");
    mGenerator->saveModel(fileName);
}

#include "designer.moc"
