/*
 * Copyright 2008  Rivo Laks <rivolaks@hot.ee>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef GAMEOBJECT_H
#define GAMEOBJECT_H

#include <Eigen/Core>
#include <ode/ode.h>

class Model;

using namespace Eigen;


/**
 * @author Rivo Laks <rivolaks@hot.ee>
 */
class GameObject
{
public:
    GameObject(float r);
    virtual ~GameObject();

    virtual void init();
    void createGeomFromMesh(const Vector3f* vertices, int vertexCount,
                            const unsigned int* indices, int indexCount);
    void createSphereGeometry();

    enum Type { T_Unknown = 0, T_Ship, T_Asteroid, T_Bullet };
    virtual Type type() const  { return T_Unknown; }

    virtual void advance(float elapsed);
    virtual void openglTransform() const;

    virtual void addForce(const Vector3f& force);
    virtual void collision(const Vector3f& force)  {}

    inline static void setVectorFromOde(const dReal* odeVector, Vector3f* myVector)
    {
        myVector->x() = odeVector[0];
        myVector->y() = odeVector[1];
        myVector->z() = odeVector[2];
    }

    Vector3f position() const
    {
        Vector3f pos;
        setVectorFromOde(dBodyGetPosition(body), &pos);
        return pos;
    }
    void setPosition(const Vector3f& pos)
    {
        dBodySetPosition(body, pos.x(), pos.y(), pos.z());
    }
    Vector3f velocity() const
    {
        Vector3f v;
        setVectorFromOde(dBodyGetLinearVel(body), &v);
        return v;
    }
    void setVelocity(const Vector3f& v);
    void setMass(float mass);

    Matrix3f rotationMatrix() const;
    void setRotation(const Vector3f& rot);
    void setDirection(const Vector3f& dir);

    Vector3f direction() const;
    Vector3f up() const;

    Model* model() const  { return mModel; }
    virtual void setModel(Model* m);

    float modelScale() const  { return mModelScale; }

public:
    float radius;
    int id;

    // ODE stuff
    dBodyID body;
    dGeomID geom;

protected:
    Model* mModel;
    float mModelScale;
    bool mTriMeshGeom;
};

#endif
