/*
    Copyright 2009  Rivo Laks <rivolaks@hot.ee>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "menuview.h"

#include "displaywidget.h"
#include "datastore.h"

#include <kgllib/widgetproxy.h>

#include <KDebug>

#include <QVBoxLayout>
#include <QPushButton>
#include <QGraphicsProxyWidget>
#include <QFile>
#include <QLabel>


MenuView::MenuView(KGLLib::GLWidget* parent): GLView(parent)
{
    mMainMenuProxy = 0;
}

MenuView::~MenuView()
{
    // Deletes the embedded widget as well
    delete mMainMenuProxy;
}

KGLLib::WidgetProxy* MenuView::proxy() const
{
    return static_cast<DisplayWidget*>(glWidget())->proxy();
}

QGraphicsProxyWidget* MenuView::createMenu(QFrame* menuContainer)
{
    menuContainer->setObjectName("menu");

    QWidget* menu = new QWidget();
    menu->setWindowOpacity(0.8);
    menu->setObjectName("menuContainer");
    menu->setLayout(new QVBoxLayout);
    menu->setStyleSheet(mStyleSheet);
    menu->layout()->addWidget(menuContainer);

    return proxy()->addWidget(menu);
}

void MenuView::init()
{
    loadStyleSheet();

    // Main menu
    QFrame* mainMenu = new QFrame();
    mainMenu->setLayout(new QVBoxLayout);

    QLabel* header = new QLabel("AstroDodge");
    header->setObjectName("header");
    mainMenu->layout()->addWidget(header);

    QPushButton* startButton = new QPushButton("New game", mainMenu);
    connect(startButton, SIGNAL(clicked(bool)), this, SIGNAL(startNewGame()));
    mainMenu->layout()->addWidget(startButton);

    QPushButton* quitButton = new QPushButton("Quit", mainMenu);
    connect(quitButton, SIGNAL(clicked(bool)), this, SIGNAL(quitGame()));
    mainMenu->layout()->addWidget(quitButton);

    mMainMenuProxy = createMenu(mainMenu);
}

void MenuView::loadStyleSheet()
{
    QFile f(DataStore::instance()->dataDir() + "menus.css");
    if (!f.open(QIODevice::ReadOnly)) {
        kError() << "Couldn't open" << f.fileName();
        return;
    }

    mStyleSheet = f.readAll();
}

void MenuView::render()
{
}

void MenuView::resize(int w, int h)
{
    mMainMenuProxy->setPos((w - mMainMenuProxy->boundingRect().width()) / 2, (h - mMainMenuProxy->boundingRect().height()) / 2);
}

void MenuView::hide()
{
    setMainMenuVisible(false);
}

void MenuView::setMainMenuVisible(bool visible)
{
    mMainMenuProxy->setVisible(visible);
}
