/*
 * Copyright 2008  Rivo Laks <rivolaks@hot.ee>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef MYMATH_H
#define MYMATH_H


#include <cstdlib>
#include <cmath>


inline float randFloat()
{
    return rand() / (float)RAND_MAX;
}

inline float randFloat(float min, float max)
{
    return min + randFloat() * (max - min);
}

// Taken from http://www.taygeta.com/random/gaussian.html
inline float gaussFloat(float mean, float variance)
{
    float x1, x2, w;
    do {
        x1 = 2.0 * randFloat() - 1.0;
        x2 = 2.0 * randFloat() - 1.0;
        w = x1 * x1 + x2 * x2;
    } while (w >= 1.0);

    w = sqrt((-2.0f * logf(w)) / w);
    return (x1 * w) * variance + mean;
}

inline float clampedGaussFloat(float mean, float variance, float min, float max)
{
    float g = gaussFloat(mean, variance);
    if (g < min) {
        return min;
    } else if (g > max) {
        return max;
    } else {
        return g;
    }
}


#endif
