/*
 * Copyright 2008-2009  Rivo Laks <rivolaks@hot.ee>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "ship.h"

#include <KDebug>


Ship::Ship() : GameObject(8)
{
    acceleration = 400;
    thrust = 1.0f;
    rotvelocity = Vector3f(0, 0, 0);

    dodgesAvailable = 3.0f;
    timeToDodgeReload = 0.0f;

    lastshoottime = -1000;
    health = 100;
    lightOn = false;
}

Ship::~Ship()
{
}

void Ship::advance(float elapsed)
{
    GameObject::advance(elapsed);

    timeToDodgeReload -= elapsed;
    if (timeToDodgeReload <= 0.0f) {
        dodgesAvailable = qMin(3.0, dodgesAvailable + elapsed * (0.4 + dodgesAvailable * 0.25));
    }

    addForce(direction() * acceleration * thrust);

    const dReal* angvel = dBodyGetAngularVel(body);
    const float damp = 0.995f;
    dBodySetAngularVel(body, angvel[0] * damp, angvel[1] * damp, angvel[2] * damp);

    const dReal* linvel = dBodyGetLinearVel(body);
    const float linDamp = 0.98f;
    dBodySetLinearVel(body, linvel[0]*linDamp, linvel[1]*linDamp, linvel[2]*linDamp);
}

void Ship::collision(const Vector3f& force)
{
    float forceNorm = force.norm();
    float damage = forceNorm / 1000.0f;

    health = qMax(0.0f, health - damage);
    kDebug() << "force:" << forceNorm << "= damage" << damage << ";  health is now" << health;
}

void Ship::addRotation(const Vector3f& rot)
{
    // We apply force to the rear end of the ship, thus rot has to be negated
    dBodyAddRelForceAtRelPos(body, -rot.x(), -rot.y(), 0, 0, 0, 1);
}

void Ship::dodge(int x, int y)
{
    if (dodgesAvailable < 1.0f) {
        return;
    }

    const int multiplier = 400000;
    dBodyAddRelForce(body, x*multiplier, y*multiplier, 0);

    timeToDodgeReload = 2.0f;
    dodgesAvailable -= 1.0f;
}

