/***************************************************************************
 *   Copyright 2006-2007 Dmitry Suzdalev <dimsuz@gmail.com>                *
 *                                                                         *
 *   This file is part of the KDE project "KReversi"                       *
 *   Was adapted to backgammon game needs a bit                            *
 *   by Alexander Smirnov <alexander.v.smirnov@gmail.com>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
 ***************************************************************************/
#ifndef BOARDRENDERER_H
#define BOARDRENDERER_H

#include <QMap>
#include <QPixmap>

#include "checker.h"
#include "renderer.h"

class KSvgRenderer;
class KPixmapCache;

class BoardRenderer: public Renderer
{
public:
    static void renderBackground ( QPainter *p, const QRectF& r ) ;
    static void renderBoard ( QPainter *p, const QRectF& r ) ;
    static void renderBoardLabels ( QPainter *p, const QRectF& r ) ;
    static void renderPossibleMove ( QPainter *p, const QRectF& r ) ;
    static void renderChip( QPainter *p, const QRectF& r, Checker::Color color ) ;
    static void renderDie( QPainter *p, const QRectF& r, int die ) ;
    static void renderCube( QPainter *p, const QRectF& r, int cube ) ;

    static QRectF defaultBoardRect() ;
    static QRectF defaultCellRect(qint8 cellIdx) ;

    static QSize defaultBoardSize() ;
    static QSize defaultBoardNumbersSize() ;
    static QSize defaultTableSize() ;
    static QSize defaultCheckerSize() ;
    static QSize defaultDiceSize() ;
    static QSize defaultCubeSize() ;

private:

    static BoardRenderer* self();

    // disable copy - it's singleton
    BoardRenderer();
    BoardRenderer ( const BoardRenderer& );
    BoardRenderer& operator= ( const BoardRenderer& );
    ~BoardRenderer();

     QMap<Checker::Color, QString> checkerElementId;
};

#endif
