/***************************************************************************
 *   Copyright (C) 2009 by Alexander Smirnov                               *
 *   alexander.v.smirnov@gmail.com                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
 ***************************************************************************/
#ifndef CHECKER_H
#define CHECKER_H

#include <QGraphicsPixmapItem>
#include <QGraphicsItemAnimation>

/**
	@author Alexander Smirnov <alexander.v.smirnov@gmail.com>
*/
class Checker : public QObject, public QGraphicsPixmapItem
{
    Q_OBJECT
public:

    enum Color{Black=-1, NoColor=0, White=1};

    Checker(Color color, double initialSize);

    virtual ~Checker();

    int cell() const;
    void setCell(int cellIdx);

    Color color() const;

    int order() const;
    void setOrder(int o);

    void animate(const QPointF& to);

    void updatePixmap(double size);

    QString toString() const;

    //TODO: override boundingRect()?
    enum { Type = UserType + 1 };
    virtual int type() const {return Type;}
    
    static Color getAnotherColor( Color color );
    
public slots:
    void slotStopAnimation();

private:
    static QImage getImage (Checker::Color color, double size);

    int m_cell;
    int m_order;
    const Color m_color;

    QGraphicsItemAnimation *m_animation;
    QPointF m_animationDest;

    bool m_temporal;
};

#endif
