/***************************************************************************
 *   Copyright (C) 2009 by Alexander Smirnov                               *
 *   alexander.v.smirnov@gmail.com                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program. If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/

#ifndef DICE_H
#define DICE_H

#include <QObject>
#include <QGraphicsPixmapItem>
#include <QTimeLine>

/**
	@author Alexander Smrinov <alexander.v.smirnov@gmail.com>
*/
class Die : public QObject, public QGraphicsPixmapItem
{
Q_OBJECT
public:
    Die(qint8 order);
    ~Die();
    enum { Type = UserType + 2 };
    virtual int type() const {return Type;}

    void updatePixmap(int num, double size);
    void updatePixmap(double size);
    qint8 order() const {return m_order;}
    qint8 number() const { return m_number; }

    void animate(QTimeLine::Direction direction);
    void show();
    void hide();

    void paint( QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget );
    
    void mousePressEvent ( QGraphicsSceneMouseEvent * event );
    
public slots:
    void slotFade(qreal);
    void slotStopAnimation();
private:
    static QImage getImage (int num, double size);

    const qint8 m_order;
    int m_number;
    qreal m_opacity;
    QTimeLine *m_timer;
};

#endif
