/*
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA  02110-1301, USA.

    ---
    Copyright (C) 2011, Dmitry Chernov diman4ik.chernov@gmail.com
 */
#include <QGraphicsSceneMouseEvent>
#include <KLocale>

#include "backgammon_local.h"
#include "iplayer.h"
#include "humanplayer.h"


#include <ctime>

#include <KDebug>


BackgammonLocal::BackgammonLocal( QObject* parent ) : BackgammonGame(parent), m_currentPlayer(0)
{}

BackgammonLocal::~BackgammonLocal()
{}

void BackgammonLocal::processUIEvent( QGraphicsSceneMouseEvent* event, BackgammonScene* board )
{
    IPlayer& player = getCurrentPlayer();

    qint8 cell = Board::cellNumber( event->scenePos() );
    
    kDebug() << "cell = " << cell << " state " << getState();
    
    if( cell == Board::bar( player.color() ) )
        cell = getBarIndex(player);
    
    switch( getState() )
    {
        case BackgammonGame::Roll:
        {
            roll();
            switchToNextState();
            
            emit(   gameEvent( i18n( "Player " ) + player.name() + i18n( " rolls " ) + 
                    QString::number( getFirstDie() ) + " " + QString::number( getSecondDie() ) ) );
                    
            break;
        }
        case BackgammonGame::Move1:
        case BackgammonGame::Move2:
        case BackgammonGame::Move3:
        case BackgammonGame::Move4:
            if( !gotCurrentMoveSrc() )
            {
                // Need extra specification to know what die to use
                setCurrentMoveSrc(cell);
                showProposedMoves( cell, player );
                break;
            }
            else if( !gotCurrentMoveDst() )
            {
                setCurrentMoveDst(cell);
            }

            if( moveReady( player ) )
            {
                kDebug() << "move ready";
                unshowProposedMoves();
                
                if( createMove( player ) )
                {
                    kDebug() << "move creation succesessful";
                    
                    QVector<Move> changes;
                    
                    Move mv = makeMove( player, changes );
                    
                    emit(   gameEvent( i18n( "Player " ) + player.name() + 
                            i18n( " moves " ) + QString::number( mv.src ) + " " + QString::number( mv.dst ) ) );
                    
                    if( changes.size() > 0 )
                        showBoardChanges( player, changes );
                    
                    showField();
                    
                    switchToNextState();
                }
                else
                    emit( engineMessage( i18n( "Illegal move. Try again" ) ) );
            }
            else
                kDebug() << "move not ready";
                
            if( !gotMoves( player ) )
            {
                kDebug() << "you 've done with moves";
                switchToState( Commit );
            }
            break;
        case BackgammonGame::Commit:
        {
            Board::hideDice();

            emit( moveFinished("") );
            
            switchToNextPlayer();
            
            if( !checkGameOver() )
            {
                switchToState( Roll );            
                emit( engineMessage( i18n( "Player " ) + getCurrentPlayer().name() + i18n( " turn." ) 
                    + i18n( "Click to roll." ) ) );
            }
            else
            {
                kDebug() << "game over!!! " << player.name() << " wins";
                emit( gameEvent( i18n( "Game over. " ) + player.name() + i18n( " wins" ) ) );
                switchToState( _Max );
            }
            
            updatePip();
            
            break;
        }
        default:
            Q_ASSERT_X(false,"processUIEvent", "Incorrect game state reached");
    }
}

void BackgammonLocal::startNewGame()
{
    //TODO: get player types from Settings, create approprieate players
    // Currently it is human-vs-human game
    m_players[0] = new HumanPlayer(Checker::Black, "Player1"); //TODO:get it from game state
    m_players[1] = new HumanPlayer(Checker::White, "Player2"); //TODO:get it from game state
    
    m_players[0]->setDirection( IPlayer::negative );
    m_players[1]->setDirection( IPlayer::positive );
    
    resetBoard();
    
    updatePip();
    switchToState(Roll);
    
    emit( setCheckerPositions( BackgammonGame::initialBoardState() ) );
}

IPlayer& BackgammonLocal::getCurrentPlayer()
{
    return *m_players[ m_currentPlayer ];
}

void BackgammonLocal::switchToNextPlayer()
{
    m_currentPlayer = ( ++m_currentPlayer )%2;
}
