
#include "backgammon_fibs_connect.h"


#include <QTcpSocket>

#include <KDebug>


BackgammonFibsConnect::BackgammonFibsConnect( QObject *parent )
{
    /* 
     * create the socket and set it non-blocking - ignore errors
     */
    m_socket = new QTcpSocket( this );//(*host, port, 1000);
    
    connect( m_socket, SIGNAL( hostFound() ), SLOT( hostFound() ) );
    connect( m_socket, SIGNAL( connected() ), SLOT( connected() ) );
    connect( m_socket, SIGNAL( readyRead() ), SLOT( receiveData() ) );
    connect( m_socket, SIGNAL( disconnected() ), SLOT( connectionLost() ) );
    connect( m_socket, SIGNAL( error( QAbstractSocket::SocketError ) ), SLOT( error( QAbstractSocket::SocketError ) ) );
}

BackgammonFibsConnect::~BackgammonFibsConnect()
{
    if( m_socket->state() == QAbstractSocket::ConnectedState ) //QAbstractSocket::ConnectedState
        m_socket->disconnectFromHost();
}

int BackgammonFibsConnect::status()
{
    return m_socket->state();
}

void BackgammonFibsConnect::sendData( const QString & line )
{
    kDebug() << "send data " << line;
    if( m_socket->state() == QAbstractSocket::ConnectedState )
        m_socket->write( line.toLatin1() + "\r\n" );        
}

void BackgammonFibsConnect::receiveData( )
{
    QByteArray line = m_socket->readAll();
    
    //kDebug() << line;
    
    if( line.length() != 0 )
    {
        rxBuffer.append(line);
        emit( newData( rxBuffer ) );
    }
    else
        kDebug() << "empty message??";
}

void BackgammonFibsConnect::hostFound()
{
    kDebug() << "host found";
}

void BackgammonFibsConnect::connectionLost()
{
    kDebug() << "disconnected";
}

void BackgammonFibsConnect::error( QAbstractSocket::SocketError )
{
    kDebug() << "error " << m_socket->errorString();
}

void BackgammonFibsConnect::connected()
{
    kDebug() << "connected";
    emit( connectionEstablished() );
}

void BackgammonFibsConnect::makeConnect( const QString& host, qint16 port )
{
    kDebug() << "connecting " << host << " : " << port;
    m_socket->connectToHost( host, port );
}
