/*
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA  02110-1301, USA.

    ---
    Copyright (C) 2011, Dmitry Chernov diman4ik.chernov@gmail.com
 */

#include "fibs_widget.h"
#include "backgammon_fibs.h"
#include "own_info_list.h"
#include "match_length_dialog.h"

#include <QTextEdit>
#include <QMenu>
#include <KDebug>


FibsWidget::FibsWidget( QWidget* parent ) : QDialog(parent)
{
    m_ui.setupUi(this);
    
    connect( m_ui.pushButton, SIGNAL( pressed() ), SLOT( sendReady() ) );

    connect( m_ui.say_button, SIGNAL( pressed() ), SLOT( letsSay() ) );
    connect( m_ui.shout_button, SIGNAL( pressed() ), SLOT( letsShout() ) );
    connect( m_ui.kibitz_button, SIGNAL( pressed() ), SLOT( letsKibitz() ) );
    connect( m_ui.whisper_button, SIGNAL( pressed() ), SLOT( letsWhisper() ) );
    
    connect( m_ui.pip_box, SIGNAL( toggled( bool ) ), SLOT( pipChecked( bool ) ) );
    connect( m_ui.autodouble_box, SIGNAL( toggled( bool ) ), SLOT( autodoubleChecked( bool ) ) );
    connect( m_ui.automove_box, SIGNAL( toggled( bool ) ), SLOT( automoveChecked( bool ) ) );
    connect( m_ui.away_box, SIGNAL( toggled( bool ) ), SLOT( awayChecked( bool ) ) );
    connect( m_ui.crawford_box, SIGNAL( toggled( bool ) ), SLOT( crawfordChecked( bool ) ) );
    connect( m_ui.double_box, SIGNAL( toggled( bool ) ), SLOT( doubleChecked( bool ) ) );
    connect( m_ui.greedy_box, SIGNAL( toggled( bool ) ), SLOT( greedyChecked( bool ) ) );
    connect( m_ui.moreboards_box, SIGNAL( toggled( bool ) ), SLOT( moreboardsChecked( bool ) ) );
    connect( m_ui.moves_box, SIGNAL( toggled( bool ) ), SLOT( movesChecked( bool ) ) );
    connect( m_ui.notify_box, SIGNAL( toggled( bool ) ), SLOT( notifyChecked( bool ) ) );
    connect( m_ui.ratings_box, SIGNAL( toggled( bool ) ), SLOT( ratingsChecked( bool ) ) );
    connect( m_ui.ready_box, SIGNAL( toggled( bool ) ), SLOT( readyChecked( bool ) ) );
    connect( m_ui.report_box, SIGNAL( toggled( bool ) ), SLOT( reportChecked( bool ) ) );
    connect( m_ui.silent_box, SIGNAL( toggled( bool ) ), SLOT( silentChecked( bool ) ) );

    
    m_infoList = new WhoInfoList(this);
    
    QTableView* view = m_ui.tableView;
    
    view->setModel( m_infoList );
    view->setSortingEnabled(true);
    view->setSelectionBehavior( QAbstractItemView::SelectRows );
    view->horizontalHeader()->setStretchLastSection(true);
    view->verticalHeader()->hide();
    view->setEditTriggers( QAbstractItemView::NoEditTriggers );
    view->setSelectionMode( QAbstractItemView::SingleSelection );
    view->resizeColumnsToContents();
    view->horizontalHeader()->stretchLastSection();
    view->setContextMenuPolicy( Qt::CustomContextMenu );
    
    connect( view, SIGNAL( customContextMenuRequested( const QPoint & ) ), SLOT( listItemMenu( const QPoint & ) ) );
    
    m_ui.tabWidget->setCurrentIndex(0);
}

FibsWidget::~FibsWidget()
{}

void FibsWidget::add( QString line )
{
    m_ui.textEdit->append( line );
}

void FibsWidget::sendReady()
{
    emit( sendLine( m_ui.lineEdit->text() ) );
}

void FibsWidget::letsSay()
{
    //kDebug() << "lets say";
    emit( say( m_ui.lineEdit->text() ) );
}

void FibsWidget::letsShout()
{
    //kDebug() << "lets shout";
    emit( shout( m_ui.lineEdit->text() ) );
}

void FibsWidget::letsKibitz()
{
    //kDebug() << "lets kibitz";
    emit( kibitz( m_ui.lineEdit->text() ) );
}

void FibsWidget::letsWhisper()
{
    //kDebug() << "lets whisper";
    emit( whisper( m_ui.lineEdit->text() ) );
}

void FibsWidget::info( QString line )
{
    m_ui.textEdit->setTextColor( Qt::darkGray );
    m_ui.textEdit->append( line );
    m_ui.textEdit->setTextColor( Qt::black );
}

void FibsWidget::chat( QString line )
{
    m_ui.textEdit->setTextColor( Qt::darkBlue );
    m_ui.textEdit->append( line );
    m_ui.textEdit->setTextColor( Qt::black );    
}

void FibsWidget::game( QString line )
{
    m_ui.textEdit->setTextColor( Qt::red );
    m_ui.textEdit->append( line );
    m_ui.textEdit->setTextColor( Qt::black );    
}

void FibsWidget::pipChecked( bool state )
{
    emit( toggle( BackgammonFibs::allowpip, state ) );
}

/*void FibsWidget::autoboardChecked( bool state )
{
    emit( toggle( BackgammonFibs::autoboard, state ) );
}*/

void FibsWidget::autodoubleChecked( bool state )
{
    emit( toggle( BackgammonFibs::autodouble, state ) );
}

void FibsWidget::automoveChecked( bool state )
{
    emit( toggle( BackgammonFibs::automove, state ) );
}

void FibsWidget::awayChecked( bool state )
{
    emit( toggle( BackgammonFibs::away, state ) );
}

/*void FibsWidget::bellChecked( bool state )
{
    emit( toggle( BackgammonFibs::bell, state ) );
}*/

void FibsWidget::crawfordChecked( bool state )
{
    emit( toggle( BackgammonFibs::crawford, state ) );
}

void FibsWidget::doubleChecked( bool state )
{

    emit( toggle( BackgammonFibs::edouble, state ) );    
}

void FibsWidget::greedyChecked( bool state )
{

    emit( toggle( BackgammonFibs::greedy, state ) );
}

void FibsWidget::moreboardsChecked( bool state )
{
    emit( toggle( BackgammonFibs::moreboards, state ) );
}

void FibsWidget::movesChecked( bool state )
{
    emit( toggle( BackgammonFibs::moves, state ) );
}

void FibsWidget::notifyChecked( bool state )
{
    emit( toggle( BackgammonFibs::notify, state ) );
}

void FibsWidget::ratingsChecked( bool state )
{
    emit( toggle( BackgammonFibs::ratings, state ) );
}

void FibsWidget::readyChecked( bool state )
{
    emit( toggle( BackgammonFibs::ready, state ) );
}

void FibsWidget::reportChecked( bool state )
{
    emit( toggle( BackgammonFibs::report, state ) );
}

void FibsWidget::silentChecked( bool state )
{
    emit( toggle( BackgammonFibs::silent, state ) );
}

void FibsWidget::setCheckedStates( const QMap<int, bool>& checkeds )
{
    m_ui.pip_box->setChecked( checkeds[BackgammonFibs::allowpip] );
    //m_ui.autoboard_box->setChecked( checkeds[BackgammonFibs::autoboard] );
    m_ui.autodouble_box->setChecked( checkeds[BackgammonFibs::autodouble] );
    m_ui.automove_box->setChecked( checkeds[BackgammonFibs::automove] );
    m_ui.away_box->setChecked( checkeds[BackgammonFibs::away] );
    //m_ui.bell_box->setChecked( checkeds[BackgammonFibs::bell] );
    m_ui.crawford_box->setChecked( checkeds[BackgammonFibs::crawford] );
    m_ui.double_box->setChecked( checkeds[BackgammonFibs::edouble] );
    m_ui.greedy_box->setChecked( checkeds[BackgammonFibs::greedy] );
    m_ui.moreboards_box->setChecked( checkeds[BackgammonFibs::moreboards] );
    m_ui.moves_box->setChecked( checkeds[BackgammonFibs::moves] );
    m_ui.notify_box->setChecked( checkeds[BackgammonFibs::notify] );
    m_ui.ratings_box->setChecked( checkeds[BackgammonFibs::ratings] );
    m_ui.ready_box->setChecked( checkeds[BackgammonFibs::ready] ); 
    m_ui.report_box->setChecked( checkeds[BackgammonFibs::report] );
    m_ui.silent_box->setChecked( checkeds[BackgammonFibs::silent] );   
}

void FibsWidget::newWhoInfo( WhoInfo info )
{
    QList<WhoInfo >& list = m_infoList->getList();

    int index = 0;
    
    if( ( index = list.indexOf(info) ) == -1 )
    {
        m_infoList->insertRows( 0, 1, QModelIndex() );
        index = 0;
    }

    QModelIndex model_index = m_infoList->index( index, 0, QModelIndex() );
    m_infoList->setData( model_index, info.m_name, Qt::EditRole );
    
    model_index = m_infoList->index( index, 1, QModelIndex() );
    m_infoList->setData( model_index, info.m_opponent, Qt::EditRole);
    
    model_index = m_infoList->index( index, 2, QModelIndex() );
    m_infoList->setData( model_index, info.m_watching, Qt::EditRole);
    
    model_index = m_infoList->index( index, 3, QModelIndex() );
    m_infoList->setData( model_index, info.m_ready, Qt::EditRole);
    
    model_index = m_infoList->index( index, 4, QModelIndex() );
    m_infoList->setData( model_index, info.m_away, Qt::EditRole);
    
    model_index = m_infoList->index( index, 5, QModelIndex() );
    m_infoList->setData( model_index, info.m_rating, Qt::EditRole);
    
    model_index = m_infoList->index( index, 6, QModelIndex() );
    m_infoList->setData( model_index, info.m_experience, Qt::EditRole);
    
    model_index = m_infoList->index( index, 7, QModelIndex() );
    m_infoList->setData( model_index, info.m_idle, Qt::EditRole);
    
    model_index = m_infoList->index( index, 8, QModelIndex() );
    m_infoList->setData( model_index, info.m_login, Qt::EditRole);
    
    model_index = m_infoList->index( index, 9, QModelIndex() );
    m_infoList->setData( model_index, info.m_hostname, Qt::EditRole);
    
    model_index = m_infoList->index( index, 10, QModelIndex() );
    m_infoList->setData( model_index, info.m_client, Qt::EditRole);
    
    model_index = m_infoList->index( index, 11, QModelIndex() );
    m_infoList->setData( model_index, info.m_email, Qt::EditRole);
}

void FibsWidget::logouts( const QString& name )
{
    QList<WhoInfo >& list = m_infoList->getList();

    int index = 0;
    
    WhoInfo info;
    
    info.m_name = name;
    
    if( ( index = list.indexOf(info) ) == -1 )
    {
        kDebug() << name << " removed";
        
        m_infoList->removeRows( index, 1, QModelIndex() );
    }    
}

void FibsWidget::resumeInvite( QString name )
{
    //kDebug() << name + " wants to resume a saved match with you.";
    game( name + " wants to resume a saved match with you." );
}

void FibsWidget::playInvite( QString name, int length )
{
    game( name + " invites you for " + QString::number( length ) + " point match" );
}

void FibsWidget::opponentMoves( const QVector<Move>& moves )
{
    game( "opponent moved: " );
    
    for( int i = 0; i < moves.size(); ++i )
        game( QString::number( moves[i].src ) + " - " + QString::number( moves[i].dst ) );
}
    
void FibsWidget::roll( int first, int second )
{
    game( "you rolled " + QString::number(first) + " " + QString::number(second) );
}
    
void FibsWidget::yourTurn()
{
    game( "your turn" );
}

void FibsWidget::listItemClicked( const QModelIndex & index )
{
    kDebug() << "item clicked " << index.column() << ":" << index.row();
}

void FibsWidget::listItemMenu( const QPoint & pnt )
{
    kDebug() << "point " << pnt.x() << ":" << pnt.y();
    
    QMenu menu( m_ui.tableView );
    menu.addAction( "invite" );
    menu.addAction( "tell" );
    menu.addAction( "message" );
    menu.addAction( "blind" );
    
    connect( &menu, SIGNAL( triggered( QAction* ) ), SLOT( fibsPlayerListMenu( QAction* ) ) );
    
    menu.exec( QCursor::pos() );
    //int row_at = m_ui.tableView->rowAt( pnt.y() );
}

void FibsWidget::fibsPlayerListMenu( QAction* action )
{
    kDebug() << action->text();
    QModelIndex mi = m_infoList->index( m_ui.tableView->currentIndex().row(), 0, QModelIndex() );
    
    //kDebug() << m_infoList->data( m_ui.tableView->currentIndex().sibling(), Qt::DisplayRole ).toString();
    kDebug() << m_infoList->data( mi, Qt::DisplayRole ).toString();
    
    QString name = m_infoList->data( mi, Qt::DisplayRole ).toString();
    
    if( action->text().compare( "invite" ) == 0 )
    {
        // Show dialog choosing match length
        MatchLengthDialog dlg(this);
        
        dlg.exec();
        
        int length = dlg.length();
        
        kDebug() << "invite " << name << length;
        emit( invite( name, length ) );
    }
    else if( action->text().compare( "tell" ) == 0 )
    {
        QString message_text = m_ui.lineEdit->text();
        kDebug() << "tell " << name << message_text;
        emit( tell( name, message_text ) );
    }
    else if( action->text().compare( "message" ) == 0 )
    {
        QString message_text = m_ui.lineEdit->text();
        kDebug() << "message " << name << message_text;
        emit( message( name, message_text ) );        
    }   
    else if( action->text().compare( "blind" ) == 0 )
    {
        kDebug() << "blind " << name;
    }
}

