
#include "own_info_list.h"

#include <KDebug>


WhoInfoList::WhoInfoList( QObject* parent ) : QAbstractTableModel(parent)
{
    
}

WhoInfoList::~WhoInfoList()
{
    
}

int WhoInfoList::rowCount( const QModelIndex &parent ) const
{
    Q_UNUSED(parent);
    return m_whoInfoList.size();
}
    
int WhoInfoList::columnCount( const QModelIndex &parent ) const
{
    Q_UNUSED(parent);
    return 12;    
}
    
QVariant WhoInfoList::data( const QModelIndex &index, int role ) const
{
    if( !index.isValid() )
        return QVariant();

    if( index.row() >= m_whoInfoList.size() || index.row() < 0 )
        return QVariant();

    if( role == Qt::DisplayRole ) 
    {
        WhoInfo info = m_whoInfoList.at( index.row() );
        
        if( index.column() == 0 )
            return info.m_name;
        else if( index.column() == 1 )
            return info.m_opponent;
        else if( index.column() == 2 )
            return info.m_watching;
        else if( index.column() == 3 )
        {
            if( info.m_ready )
                return "     v  ";
            return " ";
        }
        else if( index.column() == 4 )
        {
            if( info.m_away )
                return "     v  ";
            return " ";
        }
        else if( index.column() == 5 )
            return info.m_rating;        
        else if( index.column() == 6 )
            return info.m_experience;
        else if( index.column() == 7 )
            return info.m_idle;
        else if( index.column() == 8 )
            return info.m_login;
        else if( index.column() == 9 )
            return info.m_hostname;
        else if( index.column() == 10 )
            return info.m_client;
        else if( index.column() == 11 )
            return info.m_email;        
    }
     
    return QVariant();
}

QVariant WhoInfoList::headerData( int section, Qt::Orientation orientation, int role ) const
{
    if( role != Qt::DisplayRole )
         return QVariant();

    if( orientation == Qt::Horizontal ) 
    {
        switch (section) 
        {
            case 0:
                return tr( "Name" );
            case 1:
                return tr( "Playing with" );
            case 2:
                return tr( "Watching at" );
            case 3:
                return tr( "Ready" );
            case 4:
                return tr( "Away" );
            case 5:
                return tr( "Rating" );
            case 6:
                return tr( "Experience" );
            case 7:
                return tr( "Idle" );
            case 8:
                return tr( "Login time" );
            case 9:
                return tr( "Hostname" );
            case 10:
                return tr( "Client type" );
            case 11:
                return tr( "E-mail" );
                
            default:
                return QVariant();
         }
     }
     
     return QVariant();
}
    
Qt::ItemFlags WhoInfoList::flags( const QModelIndex &index ) const
{
    if( !index.isValid() )
        return Qt::ItemIsEnabled;

    return QAbstractTableModel::flags(index) | Qt::ItemIsEditable;
}
    
bool WhoInfoList::setData( const QModelIndex &index, const QVariant &value, int role )
{
    if( index.isValid() && role == Qt::EditRole ) 
    {
        int row = index.row();

        WhoInfo info = m_whoInfoList.value(row);

        if( index.column() == 0 )
            info.m_name = value.toString();
        else if( index.column() == 1 )
            info.m_opponent = value.toString();
        else if( index.column() == 2 )
            info.m_watching = value.toString();
        else if( index.column() == 3 )
            info.m_ready = value.toBool();
        else if( index.column() == 4 )
            info.m_away = value.toBool();
        else if( index.column() == 5 )
            info.m_rating = value.toDouble();        
        else if( index.column() == 6 )
            info.m_experience = value.toInt();
        else if( index.column() == 7 )
            info.m_idle = value.toInt();
        else if( index.column() == 8 )
            info.m_login = value.toDateTime();
        else if( index.column() == 9 )
            info.m_hostname = value.toString();
        else if( index.column() == 10 )
            info.m_client = value.toString();
        else if( index.column() == 11 )
            info.m_email = value.toString(); 
        else
            return false;

        m_whoInfoList.replace( row, info );
                 
        emit( dataChanged( index, index ) );

        return true;
    }
   
    return false;
}

bool WhoInfoList::insertRows( int position, int rows, const QModelIndex &index )
{
    Q_UNUSED(index);
    
    beginInsertRows( QModelIndex(), position, position + rows - 1 );

    for( int row = 0; row < rows; ++row ) 
    {
         WhoInfo info;
         m_whoInfoList.insert( position, info );
    }

    endInsertRows();
    return true;
}

bool WhoInfoList::removeRows( int position, int rows, const QModelIndex &index )
{
    Q_UNUSED(index);
    
    beginRemoveRows( QModelIndex(), position, position + rows - 1 );

    for( int row = 0; row < rows; ++row ) 
    {
        m_whoInfoList.removeAt( position );
    }

    endRemoveRows();
    return true;    
}

QList<WhoInfo >& WhoInfoList::getList()
{
    return m_whoInfoList;
}
