/***************************************************************************
 *   Copyright (C) 2009 by Alexander Smirnov                               *
 *   alexander.v.smirnov@gmail.com                                         *
 *                                                                         *
 *   Copyright (C) 2010, Dmitry Chernov                                    *
 *   diman4ik.chernov@gmail.com                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef IPLAYER_H
#define IPLAYER_H

#include <QString>

// Defines methods applicable for backgammon-like players
class IPlayer
{
public:
    enum direction
    {
        negative = -1,
        positive = 1,
    };
    
    virtual ~IPlayer(){}

    /** 
     * return list of player turns in the following format:
     * cell11-cell12:cell21-cell22:cell31-cell32:cell41-cell42
     * turn,depending on game type, consists of several checker movements. 
     * Movements are divided by ":" sign. Expression above describes 4 movements
     * movement consists of cell numbers divided by "-" sign
     * cellij is cell number 
     *      i stands for movement number
     *      j stands for original/destination cell number
     * Example: 1-5:5-6. this describes move from cell#1 to cell#5 and sequental move from cell#5 to cell#6. Dice might show 5 and 1
    */
    virtual QString makeTurn() = 0;

    /** 
     * make player aware of result of dice rolling
     *
     * @param d1, d2 Dice numbers, from 1 to 6 inclusive
     */
    //virtual void setDice(qint8 d1, qint8 d2) = 0;

    /**
     * get player's name
     */
    virtual QString name() const = 0;
    
    virtual void setName( const QString& name ) = 0;

    /**
     * Get player's direction.
     * -1 for clockwize
     *  1 for counter-clockwise
     */
    virtual direction getDirection() const = 0;
    
    virtual void setDirection( direction dir ) = 0;

    /**
     * get player's checker color
     */
    virtual Checker::Color color() const = 0;
    
    static direction getOpposite( direction dir )
    {   return (direction )-dir;    }
};



#endif
