/***************************************************************************
 *   Copyright (C) 2009 by Alexander Smirnov                               *
 *   alexander.v.smirnov@gmail.com                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program. If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/

#include "historywidget.h"
#include <QListWidget>
#include <kdebug.h>

HistoryWidget::HistoryWidget( const QString & title, QWidget * parent, Qt::WindowFlags flags): QDockWidget ( title, parent, flags)
{
    setAllowedAreas(Qt::RightDockWidgetArea | Qt::LeftDockWidgetArea| Qt::BottomDockWidgetArea);
    setObjectName("game_history");
    m_historyView = new QListWidget(this);
    m_historyView->setSizePolicy( QSizePolicy::Maximum, QSizePolicy::Expanding );
    setWidget(m_historyView);
}

void HistoryWidget::addHistoryLine(QString line)
{
    QString numStr = QString::number( m_historyView->count()+1 ) + ". ";
    m_historyView->addItem( numStr + line );
    QListWidgetItem *last = m_historyView->item( m_historyView->count()-1 );
    m_historyView->setCurrentItem( last );
    m_historyView->scrollToItem( last );
}

void HistoryWidget::clear()
{
    m_historyView->clear();
}

void HistoryWidget::closeEvent(QCloseEvent *event)
{
    QDockWidget::closeEvent(event);
    emit closed();
}
