/***************************************************************************
 *   Copyright (C) 2009 by Alexander Smirnov                               *
 *   alexander.v.smirnov@gmail.com                                         *
 *                                                                         *
 *   Copyright (C) 2011, Dmitry Chernov                                    *
 *   diman4ik.chernov@gmail.com                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
 ***************************************************************************/

#ifndef BACKGAMMON_MAIN_WINDOW_H
#define BACKGAMMON_MAIN_WINDOW_H


#include <kxmlguiwindow.h>

#include "ui_prefs_base.h"

class BackgammonView ;
class BackgammonScene ;
class KToggleAction;
class KUrl;
class HistoryWidget;
class IGameEngine;
class FibsWidget;
class BackgammonFibs;
class BackgammonLocal;

/**
 * This class serves as the main window for backgammon.  It handles the
 * menus, toolbars, and status bars.
 *
 * @short Main window class
 * @author Andreas Pakulat <apaku@gmx.de>
 * @version 0.1
 */
class BackgammonMainWindow : public KXmlGuiWindow
{
    Q_OBJECT
public:
    /**
     * Default Constructor
     */
    BackgammonMainWindow();

    /**
     * Default Destructor
     */
    virtual ~BackgammonMainWindow();

public slots:
    void slotHistoryVisible(bool);
    void slotShowLegalMoves(bool newValue);
    void slotUncheckHistoryMenuItem();
    void slotChangeStatus( const QString& info );
    void slotShowPip( const QString& info );
    void slotAddHistoryLine( const QString& msg );
    void slotNewGame();
    void slotUndo();
    void slotToggleDemoMode();
    void slotPreferences();
    void newFIBSGame();
    
    void resumeInvite( const QString& name );
    void playInvite( const QString& name, int length );
    
    void showRollOrDoubleQuestion();
    void showAcceptRejectQuestion();
                          
signals:
    void questionConfirmed( const QString& question );
    void questionRejected( const QString& question );
    
private:
    void setupActions();
    IGameEngine* selectedGame();
private:
    Ui::prefs_base ui_prefs_base ;
    BackgammonView *m_view;
    BackgammonScene *m_scene;

    KToggleAction *m_toolbarAction;
    KToggleAction *m_statusbarAction;

    QAction* m_undoAct;
    KToggleAction * m_demoAct;
    KToggleAction *m_showHistoryAct;
    KToggleAction *m_showLegalMovesAct;

    HistoryWidget *m_history;
    
    FibsWidget* m_fibsWidget;
    
    BackgammonFibs* m_backgammonFibs;
    BackgammonLocal* m_backgammonLocal;
    
    BackgammonFibs* cretateBackgammonFIBS();
};

#endif // BACKGAMMON_MAIN_WINDOW_H
