/***************************************************************************
 *   Copyright (C) 2005 by Piotr Banasiak <piotrbanasiak@onet.pl>          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to                             *
 *   the Free Software Foundation, 51 Franklin Street, Fifth Floor,        *
 *   Boston, MA 02110-1301, USA.                                           *
 ***************************************************************************/
#ifndef ABSTRACTENGINE__TH
#define ABSTRACTENGINE__TH

#include <QList>
#include <QHash>
#include <QtGui>

#include "boardt.h"
#include "legalmovet.h"
#include "legalmove.h"

#include "hashnode.h"
#include "engineinfo.h"
#include "aengine.h"
#include "historyheuristic.h"
#include "timemanager.h"

/**
base class of draught TEMPLATE thinking engine.

  @author Piotr Banasiak <piotrbanasiak@onet.pl>
*/



class EngineInfo;

//#define test_on
#define DRAW_AFTER 30
#define HMEM (1024*1024* 80)
// T should be T quint64 or int72 bSize 
template <class T, int bSize>
class AbstractEngineT :public AEngine
{
public :
enum
{
  WinScore = 1000000,
  LoseScore=-1000000,
  INF      = 4000000

};
    AbstractEngineT();
    virtual ~AbstractEngineT();
    
    virtual LegalMoveT<T> findBestMoveFixDepth( const BoardT<T>* const pBoard, int depthLevel , int& bestScore);
    virtual LegalMove findBestMoveFixDepth2( const Board* const pBoard, int depthLevel , int& bestScore);
    
    virtual LegalMoveT<T> findBestMoveFixTime( const BoardT<T>* const pBoard, int thinkTime, TimeManager &tm  );
    virtual LegalMove findBestMoveFixTime2( const Board* const pBoard, int thinkTime,TimeManager &tm);
    LegalMove LegalMove_Short2Long( const Board& B, LegalMoveT<T> lmT);
    LegalMoveT<T> LegalMove_Long2Short( const LegalMove& lm )const;
    void fillBoard( Board* pB, const BoardT<T>& B);
    
    virtual void genPossibleMoves( QList<LegalMoveT<T> > &MoveList ,const BoardT<T>* const pBoard )=0;
    virtual void genPossibleMoves( QList<LegalMove> &MoveList ,const Board* const pBoard )=0;
    virtual void makeMove(BoardT<T> *pB,LegalMoveT<T> *pLM);/// FIXME legal move should be const
    virtual void makeMove( Board*, const LegalMove& );
    
    void stopThinking(){stopThinkingRequested=true;};
    virtual int gamePhase(const BoardT<T>* const pBoard)const;
    void setHash( QHash<BoardT<T>,HashNode<LegalMoveT<T> > > *pHash ){pH=pHash;};
    virtual void fillBook( QList<QList<LegalMoveT<T> > > &Book );
    const EngineInfo* getEngineInfo()const;
protected:
    int MTDF( QList<LegalMoveT<T> > &MoveList, int& bestMove, const BoardT<T>* const pBoard,int  firstGuess, int depth, QList<LegalMoveT<T> > &PV);
    virtual  int AlfaBeta(QList<LegalMoveT<T> > &MoveList, int& bestMove, const BoardT<T>* const pBoard,int level, int alfa,int beta, int Depth, QList<LegalMoveT<T> > &PV);
    virtual int pureAlfaBeta(QList<LegalMoveT<T> > &MoveList, int& bestMove, const BoardT<T>* const pBoard,int level, int alfa,int beta, int Depth);
    virtual void reFillSQV( int size,int phase );
    virtual int materialScore( const BoardT<T> * const pBoard )const=0;
    bool aroundGameOver(int score){return qAbs(score) > WinScore -1000;};

    virtual bool canScore(const BoardT<T> * const pBoard, int level, int moves, int requestedDepth, bool opponenCanCapt);
    bool canPromote(const BoardT<T> &B);
    T whitePromoteCandidates(const BoardT<T> &B)const;
    T blackPromoteCandidates(const BoardT<T> &B)const;
    virtual bool canOpponentCapture( const BoardT<T> &B )const=0;

    QList<LegalMoveT<T> > PrincipalVariation;
    HistoryHeuristic HH;
    
    const int incrOnCapture;
    bool focusedAllowed;
    bool setFocusAllowed( int newFocus );
    int randomness;
    int focus1,focus2;
    
    bool stopThinkingRequested;
    EngineInfo *pEI;
    QHash<BoardT<T>,HashNode<LegalMoveT<T> > >*pH;
    const int m_size;
    
 // square/byte number of x y position
 int m_byteNr[bSize][bSize];
 int Bx[bSize*bSize/2];// board coordinate 
 int By[bSize*bSize/2];

//coordinates of firs and second moves 
 int LU[ bSize*bSize/2 ];
 int RU[ bSize*bSize/2 ];
 int LD[ bSize*bSize/2 ];
 int RD[ bSize*bSize/2 ];
 
 //mask of squares than are just before promotion
 T m_whiteBeforeLastRow;
 T m_blackBeforeLastRow;
 T m_firstRow;
 T m_lastRow;
  
 // square values separately for white and black, for speed
 int SqvWm[ bSize*bSize/2 ];
 int SqvBm[ bSize*bSize/2 ];
 int SqvK[ bSize*bSize/2 ];//common for black and white kings
 private:
   void initDirectionTables();
   void initPromoteMask();
   int toInt( T mask );
   void orderMoves( QList<LegalMoveT<T> > &MoveList, int level);
   void printMoves( QList<LegalMoveT<T> > MoveList );
   int updateWinLoseScores( int value , int currentLevel );
};

/////////////////////////////////////////////////////////////////////////////////////////////
// implementation 
/////////////////////////////////////////////////////////////////////////////////////////////

template <class T, int bSize>
AbstractEngineT<T, bSize>::AbstractEngineT():AEngine(),incrOnCapture(0),m_size(bSize)
{
   pH=0;
   pEI=new EngineInfo();
   focusedAllowed=true;
   randomness=0;//5
   
   focus1=800;
   focus2=3400;
   
   initDirectionTables();
   initPromoteMask();
}
//-----------------------------------------------------------------
template <class T, int bSize>
AbstractEngineT<T, bSize>::~AbstractEngineT()
{
   if( pEI )delete pEI;pEI=0;
   if( pH )delete pH;pH=0;
   
}

//----------------------------------------------------------------
//-----------------------------------------
template <class T, int bSize>
void AbstractEngineT<T, bSize>::initDirectionTables()
{
//squares are numbered from zero, like bytes
// -0- 1- 2- 3- 4
//.....
// 28-29-30-31-

  int c=0;
  for(int y=bSize-1;y>=0;y--)
  for(int x=0;x<bSize;x++)
  {
    if((x+y)%2)
      m_byteNr[x][y]=-1;//not used squares
    else
    {
      m_byteNr[x][y]= c;
      Bx[c]=x;
      By[c]=y;
      c++;
    }
  }
  
  // now all jump directions
  for( int x=0;x< bSize;x++ )
  for( int y=0;y< bSize;y++ )
  {
     int sqNr=m_byteNr[x][y];
     if(sqNr<0) continue;
     
     // left up
     int x1=x-1;
     int y1=y+1;
     if( x1>=0 and x1<bSize and y1>=0 and y1<bSize)
     {
        int dNr=m_byteNr[x1][y1];
        LU[sqNr]=dNr;
     }
     else LU[sqNr]= -1;
     
     // right up
     x1=x+1;
     y1=y+1;
     if( x1>=0 and x1<bSize and y1>=0 and y1<bSize)
     {
        int dNr=m_byteNr[x1][y1];
        RU[sqNr]=dNr;
     }
     else RU[sqNr]= -1;
     
     // left down
     x1=x-1;
     y1=y-1;
     if( x1>=0 and x1<bSize and y1>=0 and y1<bSize)
     {
        int dNr=m_byteNr[x1][y1];
        LD[sqNr]=dNr;
     }
     else LD[sqNr]= -1;
     
     // right down
     x1=x+1;
     y1=y-1;
     if( x1>=0 and x1<bSize and y1>=0 and y1<bSize)
     {
        int dNr=m_byteNr[x1][y1];
        RD[sqNr]=dNr;
     }
     else RD[sqNr]= -1;
  }
}
//-----------------------------------------------------------
template <class T, int bSize>
void AbstractEngineT<T, bSize>::initPromoteMask()
{
   ///@todo remember init
   m_whiteBeforeLastRow=0;
   m_blackBeforeLastRow=0;
   m_firstRow=0;
   m_lastRow=0;


  for(int x=0;x<bSize;x++)
  {
     int sqNr=m_byteNr[x][ bSize-2 ];
     if(sqNr>=0)
     {
       T M=1; M <<= sqNr;
       m_whiteBeforeLastRow |= M;
     }
     
     sqNr=m_byteNr[x][ 1 ];
     if(sqNr>=0)
     {
       T M=1; M <<= sqNr;
       m_blackBeforeLastRow |= M;
     }

     sqNr=m_byteNr[x][ bSize-3 ];
     if(sqNr>=0)
     {
       T M=1; M <<= sqNr;
       m_whiteBeforeLastRow |= M;
     }
     
     sqNr=m_byteNr[x][ 2 ];
     if(sqNr>=0)
     {
       T M=1; M <<= sqNr;
       m_blackBeforeLastRow |= M;
     }
     
     
     // last and first row for King promotion
     sqNr=m_byteNr[x][ bSize-1 ];
     if(sqNr>=0)
     {
       T M=1; M <<= sqNr;
       m_lastRow |= M;
     }
     
     sqNr=m_byteNr[x][ 0 ];
     if(sqNr>=0)
     {
       T M=1; M <<= sqNr;
       m_firstRow |= M;
     }
  }
}






//-----------------------------------------------------------------
template <class T, int bSize>
int AbstractEngineT<T, bSize>::MTDF( QList<LegalMoveT<T> > &MoveList, int& bestMove, const BoardT<T>* const pBoard,int  firstGuess, int depth, QList<LegalMoveT<T> > &PV)
{
    bestMove=-1;
    int moves=MoveList.count();
    //bool oldFocus=focusedAllowed;
    //setFocusAllowed(false);// MTDF crashes with focus
    
      int g = firstGuess;
      int upperbound = +INF;
      int lowerbound = -INF;
      
#ifdef test_on
       int correctBM=-2;
       int correctScore = pureAlfaBeta(MoveList, correctBM, pBoard, 0 , -INF ,INF ,depth);
//       qDebug("before moves ON m:%d correctScore:%d ",correctBM, correctScore);
//       //LegalMoveT<T> & cLM= MoveList[correctBM];cLM.print();
// 
//     for(int m=0;m<moves;m++)
//     {
//        LegalMoveT<T> & LM= MoveList[m];
// 
//        Board *pNewB=new Board(pBoard);
//        makeMove(pNewB,&LM);
//        QList<LegalMoveT<T> > newMoveList;
//        genPossibleMoves( newMoveList , pNewB );
//     
//        int dummyMove;
//        int cScore = -pureAlfaBeta(newMoveList, dummyMove, pNewB, 0, -INF, +INF ,depth-1);
//        delete pNewB;pNewB=NULL;
//        qDebug("test ON m:%d cScore(L-1) :%d ",m, cScore);
//        LM.print();
//     }
#endif    
      do
      {
         int beta;
         if( g == lowerbound) 
           beta = g + 1;
         else 
           beta = g;
         PV.clear();
         int mv=-1;
         g = AlfaBeta( MoveList, bestMove, pBoard, 0   ,beta - 1, beta, depth, PV );
         //qDebug( " MTDF:-----> bestMove:%d/%d  beta: %d  g: %d lowerbound:%d upperbound:%d",
         //         bestMove,MoveList.count(), beta, g, lowerbound,  upperbound);
         
         if( g < beta) upperbound = g ;//bestMove=mv;}
         else lowerbound = g;


      }while( lowerbound < upperbound );
      
      

#ifdef test_on
      Q_ASSERT( bestMove>=0 );
      if( g!=correctScore )
        qDebug( " error g:%d != correctScore:%d",g,correctScore );
      else
        qDebug( "g:%d == correctScore:%d",g,correctScore );
#endif        
      //setFocusAllowed(oldFocus);
      //qDebug( "hash size %d sizeof %d", pH->size() , sizeof(*pBoard) );
      return g;
}

//------------------------------------------------------------------------------------------------
template <class T, int bSize>
LegalMoveT<T> AbstractEngineT<T, bSize>::findBestMoveFixDepth( const BoardT<T>* const pBoard, int depthLevel , int &bestScore)
{
    stopThinkingRequested=false;
    bestScore=-INF;
    
    //if only one move is available don't waste time
    QList<LegalMoveT<T> > MoveList;
    genPossibleMoves( MoveList , pBoard );
    if(MoveList.count()==1)
      return MoveList[0];
      
    int phase=gamePhase( pBoard );
    reFillSQV( bSize, phase );
    

#ifdef test_on
int correctScore=666;
    {// BEGIN TEST TEST tEST
      if(pH)delete pH;pH=0;
      int alfa= -INF;
      int beta= +INF;
      int bestMove=-1;
  
      correctScore = pureAlfaBeta(MoveList, bestMove, pBoard, 0 , alfa ,beta ,depthLevel);
      qDebug( "correctScore: %d DEPTH %d",correctScore ,  depthLevel);
    }// END TEST TEST tEST
#endif

    if(pH)delete pH;pH=0;
    if( pH==0 )
    pH=new QHash<BoardT<T>,HashNode<LegalMoveT<T> > > ;//<--- Hash table is created here
    HH.clear();


   QTime time;
   time.start();
   pEI->start();
   int bestMove=-1;
   int level=1;
   PrincipalVariation.clear();

   float prevNodes=1;
   do
   {
      int alfa= -INF;
      int beta= +INF;

      
      int tmpBM=-1;
      QList<LegalMoveT<T> > PV;
      //bestScore=AlfaBeta(MoveList, tmpBM, pBoard, 0 , alfa ,beta ,level, PV);
      bestScore=MTDF(MoveList, tmpBM, pBoard, bestScore , level, PV);
      
      pEI->stop( bestScore );
      pEI->Depth=level;
      PrincipalVariation=PV;
      
      if(!stopThinkingRequested) bestMove=tmpBM;
      if( aroundGameOver( bestScore)) break;

   qDebug( "bestScore: %d  bestMove:%d level %d MaxDepth %d SS:%d hNodes: %d NODES: %d coeff: %f\n",
          bestScore, bestMove, level, pEI->maxDepth ,
          pEI->unnecessarySacryfice , pEI->hashNodes, pEI->nodes, 
          (float)pEI->nodes/prevNodes );
          prevNodes=pEI->nodes;
#ifdef test_on
          correctScore = pureAlfaBeta(MoveList, bestMove, pBoard, 0 , -INF , +INF ,level);
          if( correctScore!= bestScore )
          {
              qDebug("correct: %d  best: %d \n",correctScore, bestScore);
              Q_ASSERT( false );
          }
#endif
      level++;
   }while( level< depthLevel );
   if(pH)delete pH;pH=0;
   
   int t1=time.elapsed();
   qDebug("depthLevel=%d elapsed:%d ",depthLevel, t1);
 

   
   if(bestMove<0) return LegalMoveT<T>();//can happen if stopThinking
   printMoves(PrincipalVariation);
    
    return MoveList[bestMove];
}
//------------------------------------------------------------------------------------------------
template <class T, int bSize>
LegalMoveT<T> AbstractEngineT<T, bSize>::findBestMoveFixTime( const BoardT<T>* const pBoard, int thinkTime, TimeManager &tm )
{
    stopThinkingRequested=false;
    
    //if only one move is available don't waste time
    QList<LegalMoveT<T> > MoveList;
    genPossibleMoves( MoveList , pBoard );
    if(MoveList.count()==1)
      return MoveList[0];
      
   Q_ASSERT(MoveList.count()>0);

   int phase=gamePhase( pBoard );
   if( phase == 2 ) focusedAllowed=false;
   else focusedAllowed=true;
   reFillSQV( bSize, phase );
    
    if(pH)delete pH;pH=0;
    if( pH==0 )
    pH=new QHash<BoardT<T>,HashNode<LegalMoveT<T> > > ;//<--- Hash table is created here
    HH.clear();
    
   static int depthLevel=1;
   int bestMove=-1;
   static int bestScore = 0;
   PrincipalVariation.clear();
   
   tm.start();
   pEI->start();
   static int prevDepth=0;
   depthLevel=2;//qMax( prevDepth-0, 1 );
   do
   {
      int alfa= -INF;
      int beta= +INF;
      
      int tmpBM=-1;
      QList<LegalMoveT<T> > PV;
      
      for(int j=0;j<PrincipalVariation.count() ; j++)
      {
          LegalMoveT<T>& LM=PrincipalVariation[j];
          LM.used=0;
      }
      
      //bestScore=AlfaBeta(MoveList, tmpBM, pBoard, 0 , alfa ,beta ,depthLevel, PV);
      //qDebug("depthLevel %d",depthLevel);
      bestScore=MTDF(MoveList, tmpBM, pBoard, bestScore , depthLevel, PV);
      pEI->stop( bestScore );
      pEI->Depth=depthLevel;
      
#ifdef test_on
      int correctScore = pureAlfaBeta(MoveList, bestMove, pBoard, 0 , alfa ,beta ,depthLevel);


      if(!( correctScore==bestScore) )
      {
         // tests against DAM shows MTD does not work with focused
         qDebug( "correctScore:%d bm:%d     bestScore:%d move:%d", correctScore, bestMove, bestScore, tmpBM);
         Q_ASSERT( correctScore==bestScore) ;
      }
#endif      
      PrincipalVariation=PV;
      if(!stopThinkingRequested) bestMove=tmpBM;
      if( aroundGameOver( bestScore)) break;
      
      //qDebug("depthLevel=%d elapsed:%d coeff: %f expected:%d",depthLevel, t2, coeff, expected);
      prevDepth=depthLevel-1;
      depthLevel++;
   }while( tm.canContinue(depthLevel) and !stopThinkingRequested );
   if(pH)delete pH;pH=0;
   
   Q_ASSERT( bestMove>=0 );
   
   qDebug( "bestScore: %d  DEPTH %d MaxDepth %d SS:%d elapsed: %d PV.count: %d\n",
          bestScore,  depthLevel-1, pEI->maxDepth ,
          pEI->unnecessarySacryfice , tm.time.elapsed(), PrincipalVariation.count() );

   tm.stop();
   if( tm.savedTime < 0 )prevDepth -= 2;
   return MoveList[bestMove];
}

//---------------------------------------------------------------------------------
/*!
    \fn AbstractEngineT<T, bSize>::Game()
    function decides in what game phse it is. ( middleGame, gambit  or beginning )
    It is help
 */
template <class T, int bSize>
int AbstractEngineT<T, bSize>::gamePhase(const BoardT<T>* const /*pBoard*/ )const
{
    //result of that is used to fill square Values.AbstractEngineT<T, bSize>
    //default is not recognizing any phases
    return 0;
}

//------------------------------------------------------------------------
template <class T, int bSize>
void AbstractEngineT<T, bSize>::reFillSQV( int /*size*/ ,int /*phase*/ )
{
  Q_ASSERT( bSize==m_size);
  
  // just for safety not recognize any phase
  for(int j=0;j< bSize*bSize/2 ; j++)
  {
     SqvWm[j]=0;
     SqvBm[j]=0;
     SqvK[j]=0;
  }
}

//-----------------------------------------------------------------------
/*!
    \fn AbstractEngineT<T, bSize>::fillBook()
    default implementation does not do anything
    implemets book moves specyfic to the engine
 */
template <class T, int bSize>
void AbstractEngineT<T, bSize>::fillBook( QList<QList<LegalMoveT<T> > > & )//Book
{
   return;
}
//----------------------------------------------------------------------------------------
// MoveList must already be generated
template <class T, int bSize>
int AbstractEngineT<T, bSize>::pureAlfaBeta(QList<LegalMoveT<T> > &MoveList, int& bestMove, const BoardT<T>* const pBoard,int level, int alfa,int beta, int Depth)
{
  bestMove=-1;//invalid move number
  
  if (stopThinkingRequested and level>0 )return alfa;

  bool gOver=false;
  gOver= MoveList.isEmpty();
  int moves=MoveList.count();
  
  qApp->processEvents();
  if(gOver)
  {
    return  LoseScore+level;
  }

  if( pBoard->movesToDraw<=0)
  {
    return 0;
  }
  
  LegalMoveT<T> *pLM= &(MoveList[0]);
  bool mustCapture = pLM->captured;
  
  int newDepth=Depth;
  if(mustCapture)
    newDepth += incrOnCapture;
    
  bool canOpCapt=false;
  if( level>=newDepth )
     canOpCapt = canOpponentCapture( *pBoard );
     
  int matScore=-INF;
  if( !mustCapture )
  {
    matScore=materialScore( pBoard );
    bool can=canScore( pBoard,level, moves, newDepth, canOpCapt);

    if( can  )
    {
      matScore -= (canOpCapt? 100 : 0);
      return matScore;
    }
  }
  
  
  //---- checks all moves 
  for(int m=0;m<moves;m++)
  {
    pLM= &(MoveList[m]);

    BoardT<T> *pNewB=new BoardT<T>( *pBoard );
    makeMove(pNewB,pLM);
    
    QList<LegalMoveT<T> > newMoveList;
    genPossibleMoves( newMoveList , pNewB );
    int dummyMove;
    int score = -pureAlfaBeta(newMoveList, dummyMove, pNewB, level+1, -beta, -alfa ,newDepth);
    delete pNewB;pNewB=NULL;

    /* beta cut */
    if(score>=beta)
    {
       return(score);
    }
    
    if(score>alfa)
    {
       alfa=score;
       bestMove=m;
    }
  }// for moves
  
  return alfa;
}
//------------------------------------------------------------------------------
template <class T, int bSize>
bool AbstractEngineT<T, bSize>::setFocusAllowed( int newFocus )
{
   bool oldFocus=focusedAllowed;
   focusedAllowed=newFocus;
   
   return oldFocus;
}
//--------------------------------------------------------------------------------------------------------------
template <class T, int bSize>
const EngineInfo* AbstractEngineT<T, bSize>::getEngineInfo()const 
{
   return pEI;
}

//-----------------------------------------------------------------------------------------------------
template <class T, int bSize>
int AbstractEngineT<T, bSize>::AlfaBeta(QList<LegalMoveT<T> > &MoveList, int& bestMove, const BoardT<T>* const pBoard,int level, 
                                  int alfa,int beta, int Depth, QList<LegalMoveT<T> > &PV)
{
  if( alfa >= beta ) return beta;
  Q_ASSERT( alfa < beta);
  bestMove=-1;//invalid move number
  int bestScore=-INF;
  int alfa0=alfa;
  
  
  if (stopThinkingRequested and level>0 )return alfa;
  if( pH )//pH
  if( pH->contains( *pBoard) )
  {
     if(pEI) pEI->hashNodes++;
     HashNode<LegalMoveT<T> > hN=pH->value( *pBoard );
     
      MoveList = hN.MoveList;
      // to calculate how many moves from Current (not current game pos)  position is needed to win
      hN.min = updateWinLoseScores(hN.min, -level);
      hN.max = updateWinLoseScores(hN.max, -level);
    
      #ifdef test_on
      if ( hN.depth == Depth-level )//and level>pBoard->moveNr )//ourRequested Depth
      {
          
          int tmpBestMove=-1;
          int correct=pureAlfaBeta( MoveList, tmpBestMove, pBoard, level, -INF, INF,  hN.depth+level );
          if( correct<hN.min  or correct>hN.max  )
          {
              uint key = qHash(*pBoard);qDebug( "key %d", key);
              qDebug("level %d hN.depth %d/%d min %d,max %d,correct %d",
                      level,  hN.depth, Depth-level,hN.min,hN.max,correct);
              Q_ASSERT( correct>=hN.min  and correct<=hN.max );
          }
      #else
      if ( hN.depth >= Depth-level )//and level>pBoard->moveNr )//ourRequested Depth
      {
      #endif
      
          if(hN.min>=beta)
          { 
             return hN.min;
          }
          if(hN.max<=alfa)
          { 
             return hN.max;
          }
          if(hN.min>alfa)
          { 
             alfa=hN.min;
             //update alfa and don't return
          }
          if(hN.max<beta)
          { 
             beta=hN.max;
             //update alfa and don't return
          }
      }

  }
  else//End of playing with HashTable
    if(pEI) pEI->nodes++;
  
  if(pEI) pEI->maxDepth=qMax(pEI->maxDepth,level);

  bool gOver=false;
  gOver= MoveList.isEmpty();
  int moves=MoveList.count();
  
  qApp->processEvents();/// FIXME implement as separate process
  if(gOver)
  {
    if(pEI) pEI->leafNodes++;
    return  LoseScore+level;
  }
  //Draw detection -> GameOver
  if( pBoard->movesToDraw<=0)
  {
    if(pEI) pEI->leafNodes++;
    return 0;
  }
  
  LegalMoveT<T> *pLM= &(MoveList[0]);
  bool mustCapture = pLM->captured;
  
  int newDepth=Depth;
  //very limited moves can be part of one shoot  (moves<=2 or 1)
  if(mustCapture)
    newDepth += incrOnCapture;
   
  // for example is white to move, 
  // white have no capturing move but black would have if black to move
  // and will have if white does not  protect well
  bool canOpCapt=false;
  if( level>=newDepth or focusedAllowed)
     canOpCapt = canOpponentCapture( *pBoard );// check only when level>Depth
     

  int matScore=-INF;
  
  // focused search 
  // current player is not in forced move , so perhaps opponent made unnecessary sacryfice
  if( focusedAllowed && (moves>2) && (!mustCapture) && !canOpCapt )//&& (!canOpCapt)
  {
    matScore=materialScore( pBoard );
    if( (matScore > beta+focus1 ) )
    {
         newDepth=Depth-2;
         pEI->unnecessarySacryfice++;
    }
    if( (matScore > beta+focus2 ) )
    {
         newDepth=Depth-3;
         pEI->unnecessarySacryfice++;
    }
  }
  

  if(!mustCapture)
  {
    bool can=canScore( pBoard,level, moves, newDepth, canOpCapt);

    if( can  )
    {
      //qDebug( "leaf level:%d score:%d", level, matScore);
      //when calculate material score then does not return bestmove
      matScore=materialScore( pBoard );
      if(pEI) pEI->leafNodes++;
      
//       HashNode<LegalMoveT<T> > hn;
//       hn.depth=0;
//       hn.min = matScore;
//       hn.max = matScore;
//       hn.MoveList=MoveList;
//       if(pH)
//       {
//           if( pH->size() < HMEM/( sizeof(*pBoard)) ) pH->insert( *pBoard, hn );
//       }
      matScore -= (canOpCapt? 100 : 0);
      return matScore;
    }
  }

  

  orderMoves( MoveList, level );
  
  for(int m=0;m<moves;m++)// ----------------- for every move-----------------
  {
    pLM= &(MoveList[m]);


    BoardT<T> *pNewB=new BoardT<T>( *pBoard );
    makeMove(pNewB,pLM);
    
    QList<LegalMoveT<T> > newMoveList;
    genPossibleMoves( newMoveList , pNewB );
    int dummyMove;
    #ifdef test_on
    int correctBestMove=-1;
    int correctScore = -pureAlfaBeta(newMoveList, correctBestMove, pNewB, level+1 , -INF ,INF ,newDepth);
    uint key = qHash(*pNewB);
    #endif
    
    QList<LegalMoveT<T> > pv;
    int score = -AlfaBeta(newMoveList, dummyMove, pNewB, level+1, -beta, -alfa0 ,newDepth, pv);

     

#ifdef test_on
    correctBestMove=-1;
    correctScore = -pureAlfaBeta(newMoveList, correctBestMove, pNewB, level+1 , -INF ,INF ,newDepth);
    
    
    qDebug("level %*d m:%d/%2d newDepth %d, alfaBeta(%d,%d)score %d correctScore %d key %u",
           2*level+1,level,m,moves,newDepth, alfa,beta, score, correctScore,key);
    Q_ASSERT( correctScore==score or (correctScore<=alfa or correctScore>=beta) );
#endif
    delete pNewB;pNewB=NULL;
    
    if(score>bestScore)
    {
       bestScore=score;
       bestMove=m;
       PV = pv ;
       PV.prepend( MoveList[m] );
       pLM->value=bestScore;
    }
    
    alfa0=qMax( alfa0 , bestScore );
    
    /* beta cut */
    if( bestScore>=beta )
    {
      HH.goodMove( pLM->src ,pLM->dst, level, Depth );
      break;
    }
    else
       HH.badMove( pLM->src ,pLM->dst, level, Depth );
  }// for moves

  
  if( bestScore <= alfa  )//and bestMove>=0
  {
    HashNode<LegalMoveT<T> > hn;
    hn.depth=Depth-level;
    hn.min = -INF;
    hn.max = bestScore;
    hn.MoveList=MoveList;

    
    #ifdef test_on
    int tmpBestMove=-1;
    int correctScore=pureAlfaBeta( MoveList, tmpBestMove, pBoard, level, -INF, INF,  newDepth );
    qDebug("level %*d  newDepth %d, alfaBeta(%d,%d)bestScore %d correctScore %d",
            2*level+1,level,newDepth, alfa,beta, bestScore, correctScore);
    Q_ASSERT( correctScore<=hn.max  );
    #endif
    
    hn.min = updateWinLoseScores(hn.min, level);
    hn.max = updateWinLoseScores(hn.max, level);
    if(pH) if( pH->size() < HMEM/( sizeof(*pBoard)) ) pH->insert( *pBoard, hn );
  }
  
  /* Found an accurate minimax value - will not occur if called with zero window */
  if( bestScore > alfa and bestScore<beta )
  {
    HashNode<LegalMoveT<T> > hn;
    hn.depth=Depth-level;
    hn.min = bestScore;
    hn.max = bestScore;
    hn.MoveList=MoveList;

    
    #ifdef test_on
    int tmpBestMove=-1;
    int correctScore=pureAlfaBeta( MoveList, tmpBestMove, pBoard, level, -INF, INF,  newDepth );
    qDebug("level %*d  newDepth %d, alfaBeta(%d,%d)bestScore %d correctScore %d",
            2*level+1,level,newDepth, alfa,beta, bestScore, correctScore);
    Q_ASSERT( correctScore==bestScore  );
    #endif
    
    hn.min = updateWinLoseScores(hn.min, level);
    hn.max = updateWinLoseScores(hn.max, level);
    if(pH) if( pH->size() < HMEM/( sizeof(*pBoard)) ) pH->insert( *pBoard, hn );
  }

  if( bestScore >= beta  )
  {
    HashNode<LegalMoveT<T> > hn;
    hn.depth=Depth-level;
    hn.min=bestScore;
    hn.max=+INF;
    hn.MoveList=MoveList;


    #ifdef test_on
    int tmpBestMove=-1;
    int correctScore=pureAlfaBeta( MoveList, tmpBestMove, pBoard, level, -INF, INF,  newDepth );
    uint key = qHash(*pBoard);
    qDebug("level %*d  newDepth %d, alfaBeta(%d,%d)bestScore %d correctScore %d key %u",
            2*level+1,level,newDepth, alfa,beta, bestScore, correctScore, key);
    Q_ASSERT( correctScore>=hn.min );
    #endif
    
    hn.min = updateWinLoseScores(hn.min, level);
    hn.max = updateWinLoseScores(hn.max, level);
    if(pH) if( pH->size() < HMEM/( sizeof(*pBoard)) ) pH->insert( *pBoard, hn );
  }
  return bestScore;
}

//-----------------------------------------------------------------------------------------------------
// this is temporary solution, 
template <class T, int bSize>
LegalMove AbstractEngineT<T, bSize>::findBestMoveFixDepth2( const Board* const pBoard, int depthLevel , int& bestScore)
{
   Q_ASSERT( pBoard );
   
   BoardT<T> B( pBoard );
   LegalMoveT<T> bestMove;
   
   bestMove = findBestMoveFixDepth( &B, depthLevel , bestScore);
   // transform back to LegalMove
   LegalMove LM=LegalMove_Short2Long( pBoard, bestMove );
   
   return LM;
}
//--------------------------------------------------------
// this is temporary solution, 
template <class T, int bSize>
LegalMove AbstractEngineT<T, bSize>::findBestMoveFixTime2( const Board* const pBoard, int thinkTime, TimeManager &tm)
{
   Q_ASSERT( pBoard );
   
   BoardT<T> B( pBoard );
   LegalMoveT<T> bestMove;
   
   bestMove = findBestMoveFixTime( &B, thinkTime, tm );
   
   // transofm back to LegalMove
   LegalMove LM=LegalMove_Short2Long( pBoard, bestMove );
   
   return LM;
}
//-----------------------------------------------------------------------------------------------------
template <class T, int bSize>
void AbstractEngineT<T, bSize>::makeMove( Board* pB, const LegalMove& lm)
{
   BoardT<T> B( pB );
   LegalMoveT<T> lmT=LegalMove_Long2Short( lm );
   
   makeMove( &B, &lmT );
   fillBoard( pB, B );
}
//--------------------------------------------
/** makes given move on the given board */
template <class T, int bSize>
void AbstractEngineT<T, bSize>::makeMove( BoardT<T> *pB,LegalMoveT<T> *pLM )
{
  Q_ASSERT( pLM );
  Q_ASSERT( pB );
  
  T scrMask(1);
  scrMask <<= pLM->src;
  
  T dstMask(1);
  dstMask <<= pLM->dst;
  
  T c= pLM->captured;
  c |= scrMask;// removing captured and source piece
  T inv = ~c;

  pB->captured=0;

  if( pB->isWhiteTurn() )
  {
    T mMask = pB->whiteM & scrMask;
    T kMask = pB->whiteK & scrMask;
    pB->blackK &= inv;//removing captured and moving piece from src position
    pB->blackM &= inv;
    pB->whiteK &= inv;
    pB->whiteM &= inv;
    
    if( mMask ) pB->whiteM |= dstMask;
    if( kMask ) pB->whiteK |= dstMask;
    
    // king promotion
    T mProm=m_lastRow & pB->whiteM;
    if( mProm )
    {
       pB->whiteM &= (~mProm);
       pB->whiteK |= mProm;
    }
  }
  else
  {
    T mMask = pB->blackM & scrMask;
    T kMask = pB->blackK & scrMask;
    pB->blackK &= inv;
    pB->blackM &= inv;
    pB->whiteK &= inv;
    pB->whiteM &= inv;
    
    if( mMask ) pB->blackM |= dstMask;
    if( kMask ) pB->blackK |= dstMask;
    
    // king promotion
    T mProm=m_firstRow & pB->blackM;
    if( mProm )
    {
       pB->blackM &= (~mProm);
       pB->blackK |= mProm;
    }
  }
  pB->moveNr++;
}
//-----------------------------------------------------------------------------------------------------
// conversion from bity representation into XY board representation 
template <class T, int bSize>
void AbstractEngineT<T, bSize>::fillBoard( Board* pB, const BoardT<T>& B)
{
   Q_ASSERT( pB );
   Q_ASSERT( pB->size() == bSize );
  
   pB->clear();
   pB->movesToDraw=B.movesToDraw;
   pB->moveNr=B.moveNr;
   
   bool f;
   for( int j=0;j<bSize*bSize/2;j++ )
   {
      T M=1;
      M <<=j;
      int x=Bx[j];
      int y=By[j];
      
      f = B.whiteM & M;
      if( f ) pB->at( x, y ) = Board::whiteM;
      
      f = B.whiteK & M;
      if( f ) pB->at( x, y ) = Board::whiteK;
      
      f = B.blackM & M;
      if( f ) pB->at( x, y ) = Board::blackM;
      
      f = B.blackK & M;
      if( f ) pB->at( x, y ) = Board::blackK;
   }
}
//-----------------------------------------------------------------------------------------------------
// transform bit/short version of move into long used with all intermediate info
// what is used in Gui
template <class T, int bSize>
LegalMove AbstractEngineT<T, bSize>::LegalMove_Short2Long( const Board& B, LegalMoveT<T> lmT)
{
   Q_ASSERT( B.size() == bSize );
      
   QList<LegalMove> longList;
   genPossibleMoves( longList , &B );
   
   int moves=longList.count();
   
   for(int m=0;m<moves;m++)
   {
      LegalMove lmLong = longList[m];
      
      LegalMoveT<T> lmShort;
      lmShort = LegalMove_Long2Short( lmLong );
      
      //chceck if  are the same
      
      if( lmT == lmShort )
      return lmLong;
   }
   
   for(int m=0;m<moves;m++)
   {
      LegalMove lmLong = longList[m];
      
      LegalMoveT<T> lmShort;
      lmShort = LegalMove_Long2Short( lmLong );
      
      //chceck if  are the same
      
      if( lmT == lmShort )
      return lmLong;
   }

   Q_ASSERT( false );
   return LegalMove();
}
//-----------------------------------------------------------------------------------------------------
// transform long version of move used with all intermediate info into bit/short
template <class T, int bSize>
LegalMoveT<T> AbstractEngineT<T, bSize>::LegalMove_Long2Short( const LegalMove& lm)const
{
  int pathSize = lm.path.size();
  Q_ASSERT( pathSize>=2 );
  
  QPoint src=lm.path[0]; //first square
  QPoint dst=lm.path[pathSize-1];// last square
  
  int srcN = m_byteNr[src.x()][src.y()];
  int dstN = m_byteNr[dst.x()][dst.y()];
  
  if( !(srcN >=0 and srcN<(bSize*bSize/2) ) )lm.print();
  if( !(dstN >=0 and dstN<(bSize*bSize/2) ) )
  {
     qDebug( "srcN: %d dstN: %d   dst.x()=%d dst.y()=%d", srcN, dstN , dst.x(),dst.y()  );
     lm.print();
  }
  Q_ASSERT( srcN >=0 and srcN<(bSize*bSize/2) );
  Q_ASSERT( dstN >=0 and dstN<(bSize*bSize/2) );
  
  LegalMoveT<T> lmShort;
  
  // setting the bits of first and last squares
  lmShort.src = srcN;
  lmShort.dst = dstN;
  
  lmShort.captured=0;
  int capSize=lm.takenPieces.size();
  for( int j=0; j< capSize; j++ )
  {
     QPoint c = lm.takenPieces[j];
     int cN = m_byteNr[c.x()][c.y()];
     Q_ASSERT( cN >=0 and cN<(bSize*bSize/2) );
     
     T M=1;
     M <<= cN;
     lmShort.captured |= M;
  }
  
  return lmShort;
}

//----------------------------------------------------------------
// this is exact version of prototype , already proved very well
template <class T, int bSize>
bool AbstractEngineT<T, bSize>::canScore(const BoardT<T> * const pBoard, int level, int moves, int requestedDepth, bool opponenCanCapt)
{
  if( level<requestedDepth ) return false;

  if(moves<=1)
    return false;
    
  if(level >=requestedDepth +2)
    return true;
    
  if( opponenCanCapt) return false; 
    
  if(level >=requestedDepth +2)
    return true;

  //if we look deeper into canPromote, we should analyze only promoting moves
  //or set a limit on depth
  bool canProm=canPromote(*pBoard);
  if( canProm )return false;
  
  // very limited moves can be part of one shoot
  if(moves<=2)
    return false;

  return true;
}
//---------------------------------------------
// check if current or opposite player can pass to last row
template <class T, int bSize>
bool AbstractEngineT<T, bSize>::canPromote(const BoardT<T> &B)
{ 
  int sqNr=0;
  Q_ASSERT ( !B.captured );
  T squares=B.whiteM | B.whiteK | B.blackM | B.blackK;
  

   {
      T candidates = B.whiteM & m_whiteBeforeLastRow;
      T shift=candidates;
      T M=0x01;
      while (shift)
      {
          if(candidates & M)
          {
             //check if the particular candidate can go
             int d1=LU[sqNr];
             T dMask= 1;
             dMask<<=d1;
             if( d1>=0 and !(dMask & squares) )
                return true; 
             
             int d2=RU[sqNr];// what happen if d2==-1 , will shift operator throw exception ?
             dMask= 1;
             dMask<<=d2;
             if( d2>=0 and !(dMask & squares) )
                return true;
          }
          
          M<<=1;
          shift >>= 1; sqNr++;
      }
   }

   {
      T candidates = B.blackM & m_blackBeforeLastRow;
      T shift=candidates;
      T M=0x01;
      while (shift)
      {
          if(candidates & M)
          {
             //check if the particular candidate can go
             int d1=LD[sqNr];
             T dMask= 1;
             dMask<<=d1;
             if( d1>=0 and !(dMask & squares) )
                return true;
             
             int d2=RD[sqNr];
             dMask= 1;
             dMask<<=d2;
             if( d2>=0 and !(dMask & squares) )
                return true;
          }
          
          M<<=1;
          shift >>= 1; sqNr++;
      }
   }
   return false;
}




//----------------------------------------------------
// returns bits of all white candidates to promotion
template <class T, int bSize>
T AbstractEngineT<T, bSize>::whitePromoteCandidates(const BoardT<T> &B)const
{ 
  int sqNr=0;
  Q_ASSERT ( !B.captured );
  T squares=B.whiteM | B.whiteK | B.blackM | B.blackK;
  T moveableCandidates=0;
  
  T candidates = B.whiteM & m_whiteBeforeLastRow;
  T shift=candidates;
  T M=0x01;
  while (shift)
  {
    if(candidates & M)
    {
      //check if the particular candidate can move
      int d1=LU[sqNr];
      T dMask= 1;
      dMask<<=d1;
      if( d1>=0 and !(dMask & squares) )
        moveableCandidates |= M;
      
      int d2=RU[sqNr];
      dMask= 1;
      dMask<<=d2;
      if( d2>=0 and !(dMask & squares) )
        moveableCandidates |= M;
    }
    
    M<<=1;
    shift >>= 1; sqNr++;
  }
  return moveableCandidates;
}
//----------------------------------------------------
// returns bits of all black candidates to promotion
template <class T, int bSize>
T AbstractEngineT<T, bSize>::blackPromoteCandidates(const BoardT<T> &B)const
{ 
  int sqNr=0;
  Q_ASSERT ( !B.captured );
  T candidates = B.blackM & m_blackBeforeLastRow;
  T moveableCandidates=0;
  T squares= B.whiteM | B.whiteK | B.blackM | B.blackK;

  T shift=candidates;
  T M=0x01;
  while (shift)
  {
      if(candidates & M)
      {
          //check if the particular candidate can move
          int d1=LD[sqNr];
          T dMask= 1;
          dMask<<=d1;
          if( d1>=0 and !(dMask & squares) )
            moveableCandidates |= M;
          
          int d2=RD[sqNr];
          dMask= 1;
          dMask<<=d2;
          if( d2>=0 and !(dMask & squares) )
            moveableCandidates |= M;
      }
      
      M<<=1;
      shift >>= 1; sqNr++;
  }
   
   return moveableCandidates;
}

//-------------------------------
// probably good idea to change representation from bit mask into sqare numbers
template <class T, int bSize>
int AbstractEngineT<T, bSize>::toInt( T mask )
{
    if( !( mask) )return -1;//invalid 
    int nr = -1;
    while( mask )
    {
      nr++;
      mask >>=1;
    }
    return nr;
}
//------------------------------------------------------
template <class T, int bSize>
void AbstractEngineT<T, bSize>::orderMoves( QList<LegalMoveT<T> > &MoveList ,int level)
{
   
   int moves=MoveList.size();
   if( moves <=1 ) return;// nothing to sort
   
   for(int j=0; j<moves; j++)
   {
      LegalMoveT<T> &LM=MoveList[j];
      int cnt=HH.getCnt( LM.src, LM.dst, level );
      LM.killer=cnt;
   }
   qSort( MoveList.begin(), MoveList.end() );
   //printMoves( MoveList );
   
  /// @todo sort  first moves from PV if applicable
  
  LegalMoveT<T> f=MoveList.first();//and moves>1
  if( PrincipalVariation.count() > level )/// @todo that is previous Principal Variations
  {
      LegalMoveT<T>& k1=PrincipalVariation[level];
      if( k1.used!=100 )
      {
        k1.used=100;
        if( k1 == f  )
        {
          //qDebug("equal level:%d",level);
        }
        else
        {
          //move PV move to first 
          int idx= MoveList.indexOf( k1 );
          //qDebug("swapping level:%d idx %d",level, idx);
          if(idx>0)
          {
            MoveList.swap( 0, idx );
          }
        }
      }
  }
  
  //qDebug("\n level %d:",level);
  /*
  //after sorted
  qDebug("\n level %d:",level);
  for(int j=0;j<MoveList.count() ; j++)
  {
      LegalMoveT<T>& LM=MoveList[j];
      qDebug("sorted [%d] value %d killer %d",j, MoveList[j].value, MoveList[j].killer );
      if( MoveList[j].value > -4000000 ) qDebug("===============");
  }
  qDebug("\n");
  */
}
//------------------------------------------------------
template <class T, int bSize>
void AbstractEngineT<T, bSize>::printMoves( QList<LegalMoveT<T> > MoveList )
{
   qDebug( "Begin move ---------------\n" );
   for(int j=0;j<MoveList.size();j++)
   {
      LegalMoveT<T> &LM=MoveList[j];
      qDebug( "move:%d   (%d -> %d)value %d    killer %d \n", j, LM.src, LM.dst, LM.value, LM.killer );
   }
   qDebug( "End move ---------------\n" );
}

// win / lose scores are calculated as WinScore - howManyMovesToWin
// 999999 means we win in one move , 999992 win in 8 moves
// the problem is when storing those value in hash table
// the value can NOT be reused if the same board position is aproached again but at different level
template <class T, int bSize>
int AbstractEngineT<T, bSize>::updateWinLoseScores(int value, int currentLevel)
{
    if( abs( value ) == INF )
       return value;
    
    
    // to calculate how many moves from currentLevel is needed to win
    // so 99993 ( means win in 7 moves )  seen at level 2
    // will be replaced with 99995 means in 5 moves from current level
    // when the value is used from Hash table, current level must be subtracted from score
    if(value >= WinScore-100 ) 
        value += currentLevel;
    
    
    if( value <= LoseScore+100 )
        value-=currentLevel;

    return value;
}

#endif
