/***************************************************************************
 *   Copyright (C) 2005 by Piotr Banasiak <piotrbanasiak@onet.pl>          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to                             *
 *   the Free Software Foundation, 51 Franklin Street, Fifth Floor,        *
 *   Boston, MA 02110-1301, USA.                                           *
 ***************************************************************************/

#include "board.h"

uint qHash ( const Board& B )
{
   uint tmp=0;
   for(int j=0; j<B.b.size() ; j++)
  {
      int piece=B.b.at(j);
      tmp ^= piece;//XOR
      int mask= (1<< (8*sizeof(int)-1));
      bool MSB=tmp & mask;
      tmp <<=1;  // left shift 
      tmp |=piece;
      if(MSB)tmp |= 1; //put the oldest bit back 
  }
  //qDebug("qHash %d",tmp);
  return tmp;
}
bool Board::operator== ( const Board & B2 ) const
{
   //qDebug("oper==");
   //the moveNr does not have to be the same, but player to move must be
   if( moveNr%2 != B2.moveNr%2) return false;
   
   if( m_size != B2.m_size) return false;
   
   for( int j=0;j<b.size(); j++ ) 
   {
      char a1=b[j];
      char a2=B2.b[j];
      if( a1!=a2) return false;
   }
   return true;
}

Board::Board(int bSize): b()
{
	m_size=bSize;
	startPos();//put pieces on Board
}
//----------------------------------------------
Board::Board(const Board *const pB)
{
	b = pB->b;
	m_size = pB->m_size;
	
	moveNr=pB->moveNr;
	movesToDraw=pB->movesToDraw;
}
//-----------------------------------------------
Board::~Board()
{
}

//-----------------------------------------------
// how many pieces are at start
int Board::startingPiecesNumber() const
{
	int rows=m_size/2 -1;
	int inRow=m_size/2;
	
	int total=rows*inRow*2;
	return total;
}
//-----------------------------------------------
void Board::startPos()
{
	movesToDraw=30;///@todo implement later
  moveNr=0;
	clear();

//lewy dolny rog to 0,0 prawy gorny to 7,7,(B_SIZE-1,B_SIZE-1)

int Rows;


//BEGIN of normal
//dla 8x8 na pierwszych trzech stawiamy biale
// dla 10x10 na pierwszych czterech stawiamy biale

	Rows=m_size/2 -1;// test

	for(int x=0;x<m_size;x++)
		for(int y=0;y<Rows;y++)
		{
			if(! ((x+y)%2) )
				at(x,y) = whiteM;
		}
		
//na pierwszych trzech gornych stawiamy czarne
	for(int x=0;x<m_size;x++)
		for(int y=m_size-Rows;y<m_size;y++)
		{
			if(! ((x+y)%2) )
				at(x,y)=blackM;
		}

// END of normal

/*
     at(0,0)=whiteK;
     at(3,3)=blackM;
     at(5,3)=blackM;
//     at(4,0)=whiteM;
//     at(8,4)=whiteM;
//     at(4,2)=blackK;
//     
//     at(2,4)=blackM;
//     at(4,4)=blackM;
//     at(1,5)=0;
//     at(5,5)=0;
    */
}
//---------------------------------
// remove all pieces
void Board::clear(void)
{
    b.fill( '\0',m_size *m_size );
}
//----------------------------------
void Board::countPieces(int &white_M, int& white_K, int &black_M, int& black_K)const
{
  white_M=white_K=black_M=black_K=0;
	
	for(int idx=0;idx<m_size *m_size;idx++)
	{
	    int piece=b.at(idx);
			if(piece == whiteM)
				white_M++;
				
			if(piece == whiteK)
  	  	white_K++;

      if(piece == blackM)
        black_M++;

      if(piece == blackK)
        black_K++;
	}
}
//----------------------------------
QByteRef Board::at(int x, int y)
{
    Q_ASSERT( x>=0 );Q_ASSERT( x<m_size );
    Q_ASSERT( y>=0 );Q_ASSERT( y<m_size );
    int idx=x*m_size+y;
    return b[ idx ];
}

QByteRef Board::at( QPoint &xy )
{ 
    Q_ASSERT( xy.x() >= 0 );Q_ASSERT( xy.x() < m_size );
    Q_ASSERT( xy.y() >= 0 );Q_ASSERT( xy.y() < m_size );
    int idx=xy.x()*m_size+xy.y();
    return b[ idx ];
}
//-------------------------------------------
// return square number (starting from 0 )
// current implementation is very slow
int Board::sqNumber( int xx, int yy )const
{
  int s=size();
  int nr=0;
  for( int y=s-1 ; y>=0;y-- )
  for( int x=0;x< s;x++ )
  {
    if( (x+y)%2 ==0 )
    {
      if( x==xx  && y==yy)
        return nr;
      nr++;
    }
  }
  return -1;
}
