/***************************************************************************
 *   Copyright (C) 2005 by Piotr Banasiak <piotrbanasiak@onet.pl>          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to                             *
 *   the Free Software Foundation, 51 Franklin Street, Fifth Floor,        *
 *   Boston, MA 02110-1301, USA.                                           *
 ***************************************************************************/
#ifndef QWIDGETBOARDWIDGET_H
#define QWIDGETBOARDWIDGET_H

#include <QAbstractItemView>
#include <QHeaderView>

/**
Displays Game board

	@author Piotr Banasiak <piotrbanasiak@onet.pl>
*/

//class BoardModel;
class PieceDelegate;
class BoardView : public QAbstractItemView
{
  Q_OBJECT
public:
    BoardView(QWidget *parent,  PieceDelegate *delegate);
    virtual QSize minimumSizeHint() const;
    ~BoardView();
    
    virtual QRect visualRect(const QModelIndex &index) const;
    virtual void scrollTo(const QModelIndex &index, ScrollHint hint = EnsureVisible);
    virtual QModelIndex indexAt(const QPoint &point) const ;
    virtual QModelIndex moveCursor(CursorAction cursorAction, Qt::KeyboardModifiers modifiers);
    virtual int horizontalOffset() const;
    virtual int verticalOffset() const ;
    virtual bool isIndexHidden(const QModelIndex &index) const ;
    void setSelection(const QRect &rect, QItemSelectionModel::SelectionFlags command);
    virtual QRegion visualRegionForSelection(const QItemSelection &selection) const;
    bool isRowHidden(int row) const;
    bool isColumnHidden(int column) const;
    int rowViewportPosition(int row) const;
    int columnViewportPosition(int column) const;
    //bool showGrid() const;
    //void setColumnWidth(int column, int width);
    int columnWidth(int column) const;
    //void setRowHeight(int row, int height);
    int rowHeight(int row) const;
    int rowAt(int y) const;
    int columnAt(int x) const;
    Qt::PenStyle gridStyle() const;
    void setGridStyle(Qt::PenStyle style);
    void updateGeometries();
    void setModel(QAbstractItemModel *model);
    
    QHeaderView *horizontalHeader() const;
    QHeaderView *verticalHeader() const;
private:
    void setHorizontalHeader();
    void setVerticalHeader();//
  int rows,cols;
  bool m_inverted;
  QHeaderView * hHeader;
  QHeaderView * vHeader;
  QHeaderView * hHeader2;
  QHeaderView * vHeader2;
  
  bool m_showGrid;
  Qt::PenStyle m_gridStyle;
  void paintEvent(QPaintEvent *e);
  QSize m_cellSizeHint;
  
  void setSelectionModel(QItemSelectionModel *selectionModel);
public slots:
    void columnCountChanged( int oldCount, int newCount);
    void rowCountChanged ( int oldCount, int newCount );
    //void column2CountChanged( int oldCount, int newCount);
    //void row2CountChanged ( int oldCount, int newCount );
    
    void rowResized( int row, int, int);
    void columnResized( int column, int, int);
    void row2Resized( int row, int, int);
    void column2Resized( int column, int, int);
    void invertBoard( bool );
};


#endif
