/***************************************************************************
 *   Copyright (C) 2005 by Piotr Banasiak <piotrbanasiak@onet.pl>          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to                             *
 *   the Free Software Foundation, 51 Franklin Street, Fifth Floor,        *
 *   Boston, MA 02110-1301, USA.                                           *
 ***************************************************************************/
#include "dxp.h"
#include <QByteArray>
#include <QDataStream>
#include <iostream>
#include <QTextStream>
#include <QString>
#include "config.h"

dxpGameReq::dxpGameReq()
{
  header.fill('R',1);
  
  version.fill('0',2);
  version[0]='0';
  version[1]='1';
  
  initiatorName.fill(' ',32);
  initiatorName.replace(0,4,"dupa");
  
  followerColor.fill('W',1); // w or Z
  thinkingTime.fill('6',3);  // time in minutes for whole game
  numberOfMoves.fill('0',3); //fot the whole game, can be Zero
  startingPosition.fill('A',1);// A -normal , B , use the next two fields
  
  colorToMoveFirst.fill('W',1);
  position.fill('e',50);
}


dxpGameReq::~dxpGameReq()
{
}

void dxpGameReq::setFollowerWhite(bool followerIsWhite)
{
   if( followerIsWhite )
      followerColor.fill('W',1);
   else
      followerColor.fill('Z',1);
}

QDataStream& dxpGameReq::writeTo( QDataStream& out )const
{ 
  out.writeRawData( header , 1);
  out.writeRawData( version, 2);
  out.writeRawData( initiatorName, 32) ;
  //out << buf;
  
  //out << initiatorName;
  out.writeRawData( followerColor,1);
  out.writeRawData( thinkingTime, 3);
  out.writeRawData( numberOfMoves,3);
  out.writeRawData( startingPosition,1);
  
  if( startingPosition[0]=='A') 
  {
     const char zero='\0';
     out.writeRawData( &zero, 1);
     return out;
  }
  
  out.writeRawData( colorToMoveFirst,1);
  out.writeRawData( position,50);
  
  const char zero='\0';
  out.writeRawData( &zero, 1);
  return out;
}

// don't read header, it already read
bool dxpGameReq::readFrom( QDataStream& in )
{
   //QByteArray buf;
   //buf.resize(256);
   //in.readRawData( buf.data(), 256 );
   
   int bytes=0;
   bytes=in.readRawData( version.data(), 2 );
   if( bytes !=2 ) return 0;
   
   bytes=in.readRawData( initiatorName.data(), 32 );
   if( bytes !=32 ) return 0;
   
   bytes=in.readRawData( followerColor.data(), 1 );
   if( bytes !=1 ) return 0;
   
   bytes=in.readRawData( thinkingTime.data(), 3 );
   if( bytes !=3 ) return 0;
   
   bytes=in.readRawData( numberOfMoves.data(), 3 );
   if( bytes !=3 ) return 0;
   
   bytes=in.readRawData( startingPosition.data(), 1 );
   if( bytes !=1 ) return 0;
   
   char sp=startingPosition[0];
   qDebug("dxpGameReq::readFrom startingPosition: %c",sp);
   if( (sp!='A') and (sp!='B') )
   { 
     qDebug("startingPosition must be A or B is:%c",sp);
     return false;
   }
   
   if( sp =='B' )
   {
     bytes=in.readRawData( colorToMoveFirst.data(), 1 );
     if( bytes !=1 ) return 0;
     
     bytes=in.readRawData( position.data(), 50 );
     if( bytes !=50 ) return 0;
   }
   
   char zero='X';
   bytes=in.readRawData( &zero, 1 );
   if( bytes !=1 || zero!='\0' ) 
   {
     qDebug( " zero byte (end of messge) is missing");
     return false;
   }
   
   return true;
}

//---------------------------------------------------------------------------------
dxpGameAcc::dxpGameAcc()
{
  header.fill('A',1);
  initiatorName.fill('x',32);
  initiatorName.replace(0,4,"dupa");
  
  code.fill('0',1);
}


dxpGameAcc::~dxpGameAcc()
{
}

void dxpGameAcc::setReject()
{
   code.fill('1',1);
}

bool dxpGameAcc::isAccepted()
{
   return (code[0]=='0');
}

bool dxpGameAcc::readFrom( QDataStream& in )
{
   int bytes=0;
   bytes=in.readRawData( initiatorName.data(), 32 );
   if( bytes !=32 ) return 0;
   
   bytes=in.readRawData( code.data(), 1 );
   if( bytes !=1 ) return 0;
   
   QByteArray tmp=initiatorName;
   tmp.push_back('\0');

   qDebug( "dxpGameAcc::readFrom initiatorName:%s code:",tmp.data()   );

   char zero='X';
   bytes=in.readRawData( &zero, 1 );
   if( bytes !=1 || zero!='\0' ) 
   {
     qDebug( " zero byte (end of messge) is missing");
     return false;
   }
   
  return true;
}

QDataStream& dxpGameAcc::writeTo( QDataStream& out )const
{
  out.writeRawData( header , 1);
  out.writeRawData( initiatorName, 32) ;
  out.writeRawData( code, 1);
  
  const char zero='\0';
  out.writeRawData( &zero, 1);
  return out;
}
//--------------------------------------------------------------------------------
dxpMove::dxpMove()
{
  header.fill('M',1);
  
  time.fill('0',4);       //4
  fromField.fill('0',2);  // 2
  fromField.replace(0,2,"17");
  
  toField.fill('0',2);    // 2
  toField.replace(0,2,"22");
  
  numberCaptured.fill('0',2); // 2
  m_numberCapt=0;
  capturedPieces.clear();//variable
}

dxpMove::~dxpMove()
{
}


bool dxpMove::readFrom( QDataStream& in )
{
   int bytes=0;
   bytes=in.readRawData( time.data(), 4 );
   if( bytes !=4 ) return 0;
   
   bytes=in.readRawData( fromField.data(), 2 );
   if( bytes !=2 ) return 0;
   
   bytes=in.readRawData( toField.data(), 2 );
   if( bytes !=2 ) return 0;
   
   bytes=in.readRawData( numberCaptured.data(), 2 );
   if( bytes !=2 ) return 0;
   QByteArray tmpNumberCaptured=numberCaptured;
   tmpNumberCaptured.push_back('\0');
   QString nCapt( tmpNumberCaptured.data());
   m_numberCapt=nCapt.toInt();
   
   QByteArray tmpFrom=fromField;
   tmpFrom.push_back('\0');
   QByteArray tmpTo=toField;
   tmpTo.push_back('\0');
   //qDebug( "dxpMove::readFrom from:%s to:%s  numberCaptured:%d",tmpFrom.data() , tmpTo.data() ,m_numberCapt );

   capturedPieces.clear();
   for(int j=0;j<m_numberCapt;j++)
   {
      QByteArray tmpCaptured(2,'0');
      bytes=in.readRawData( tmpCaptured.data(), 2 );
      if( bytes !=2 ) return 0;
      
      capturedPieces.push_back( tmpCaptured );
   }
   
   char zero='X';
   bytes=in.readRawData( &zero, 1 );
   if( bytes !=1 || zero!='\0' ) 
   {
     qDebug( " zero byte (end of messge) is missing");
     return false;
   }
   
  return true;
}

QDataStream& dxpMove::writeTo( QDataStream& out )const
{
  const char zero='\0';
  
  QByteArray buf;
  buf.append( header);
  buf.append( time);
  buf.append( fromField);
  buf.append( toField);
  buf.append( numberCaptured);
  buf.append( capturedPieces);
  buf.append( zero );
  out.writeRawData( buf, buf.size() );
  
  return out;
}

LegalMove dxpMove::toLegalMove()
{
   int from=toInt( fromField );
   int to=toInt( toField );
   
   QPoint start=pdn2point(from);
   QPoint end=pdn2point(to);
   
   LegalMove LM( start,end);
   for(int j=0;j<m_numberCapt;j++)
   {
      QByteArray capt=capturedPieces.mid(j*2,2);
      int pieceNr=toInt( capt );
      QPoint xy=pdn2point( pieceNr );
      LM.takenPieces.append( xy );
   }
   return LM;
}

void dxpMove::fromLegalMove(const LegalMove& LM)
{
   int from=point2pdn( LM.path[0] );
   int to=point2pdn( LM.path[ LM.path.count()-1] );
   
   time=fromInt( 0,4 );
   fromField=fromInt( from,2 );
   toField=fromInt( to,2 );
   
   m_numberCapt=LM.takenPieces.size();
   numberCaptured=fromInt( m_numberCapt ,2 );
   capturedPieces.clear();
   for( int j=0; j< m_numberCapt; j++ )
   {
      QPoint cap=LM.takenPieces[j];
      int pieceNr=point2pdn( cap );
      QByteArray piece=fromInt( pieceNr,2 );
      capturedPieces.push_back( piece );
   }
}
//--------------------------------------------------------------------------------
dxpGameEnd::dxpGameEnd()
{
  header.fill('M',1);
  
  reason.fill('0',1);
  stopCode.fill('1',2);  // 1 stop entirely, 0 if want next game
}

dxpGameEnd::~dxpGameEnd()
{
}

bool dxpGameEnd::readFrom( QDataStream& in )
{
   int bytes=0;
   bytes=in.readRawData( reason.data(), 1 );
   if( bytes !=1 ) return 0;
   
   bytes=in.readRawData( stopCode.data(), 1 );
   if( bytes !=1 ) return 0;
   QByteArray tmpCode=stopCode;
   tmpCode.push_back('\0');
   QString code( tmpCode.data());
   int stopC=code.toInt();
   qDebug("stop code:%d",stopC);
   
   
   char zero='X';
   bytes=in.readRawData( &zero, 1 );
   if( bytes !=1 || zero!='\0' ) 
   {
     qDebug( " zero byte (end of messge) is missing");
     return false;
   }
   qDebug("dxpGameEnd received");
   
   return true;
}

QDataStream& dxpGameEnd::writeTo( QDataStream& out )const
{
  out.writeRawData( header , 1);
  out.writeRawData( reason, 1) ;
  out.writeRawData( stopCode, 1);
  
  const char zero='\0';
  out.writeRawData( &zero, 1);
  return out;
}

//--------------------------------------------------------------------------------

dxpChat::dxpChat()
{
  header.fill('C',1);
  
  text.fill('x',1);
}

dxpChat::~dxpChat()
{
}

void dxpChat::setText(const QString &msg)
{
   text=msg.toAscii();
}

bool dxpChat::readFrom( QDataStream& in )
{
   char c='\0';
   text.clear();
   while( in.readRawData( &c,1 ) )
   {
      text.push_back(c);
      if( c=='\0') break;
   }
   
   char end=text[ text.size()-1 ];
   if( end !='\0')
      text.push_back('\0');
      
   qDebug("dxpChat::readFrom:%s", text.data() );
   qDebug("dxpChat received");
   
   return true;
}

QDataStream& dxpChat::writeTo( QDataStream& out )const
{
  out.writeRawData( header , 1);
  out.writeRawData( text, 1) ;//text is zero terminated
  
  return out;
}

int toInt( const QByteArray& arr )
{
   QByteArray tmp=arr;
   tmp.push_back( '\0');
   QString str( tmp.data() );
   int value=str.toInt();
   return value;
}
 QByteArray fromInt( int x,int size )
{
   QByteArray arr;
   arr.fill('x',size);
   
   QString s=QString( "%1").arg( x,size ,10 ,QChar('0') );
   
   //QTextStream out(stdout);
   //out << "fromInt " << s ;
   //qDebug( "s =", s.toAscii() );
   
   arr= s.toAscii();
   return arr;
}
//-------------------------------------------------------------------
//-------------------------------------------------------------------
/*!
    \fn DxpMsg::readFrom()
 */


QPoint pdn2point(int pieceNr)
{
  int size=Conf().m_boardSize;
  
  int nr=0;
  for( int y=size-1 ; y>=0;y-- )
  for( int x=0;x< size;x++ )
  {
    if( (x+y)%2 ==0 )
    {
      nr++;
      if( nr==pieceNr )
        return QPoint( x,y );
    }
  }
  return QPoint(-1,-1);
  
}

int point2pdn( const QPoint & point)
{
   int xx=point.x();
   int yy=point.y();
   
  int size=Conf().m_boardSize;
  int nr=0;
  for( int y=size-1 ; y>=0;y-- )
  for( int x=0;x< size;x++ )
  {
    if( (x+y)%2 ==0 )
    {
      nr++;
      if( x==xx  && y==yy)
        return nr;
    }
  }
  return -1;
}

