/***************************************************************************
 *   Copyright (C) 2005 by Piotr Banasiak <piotrbanasiak@onet.pl>          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to                             *
 *   the Free Software Foundation, 51 Franklin Street, Fifth Floor,        *
 *   Boston, MA 02110-1301, USA.                                           *
 ***************************************************************************/
#include "netplayer.h"
#include "dxp.h"

#include <iostream>
#include <QTextStream>
#include <QDataStream>
#include <QFile>


netPlayer::netPlayer(QObject *parent , QAbstractSocket *pS) :AbstractPlayer(parent)
{
   Q_ASSERT( pS );
   //m_name=name;
   pSocket=pS;
   
   qDebug(  "from Constructor"  ) ;
   connect( pSocket, SIGNAL(stateChanged ( QAbstractSocket::SocketState)), this, SLOT( onState(QAbstractSocket::SocketState)) ); 
   connect ( pSocket, SIGNAL( error ( QAbstractSocket::SocketError) ) , this , SLOT( onError( QAbstractSocket::SocketError) ) );
   connect(pSocket, SIGNAL( readyRead() ),this, SLOT(onReadyRead()) );
}


netPlayer::~netPlayer()
{
   pSocket->close();
   //socket is owned by netGame
   //delete pSocket;
}


void netPlayer::onState(QAbstractSocket::SocketState socketState)
{
 // qDebug("netPlayer::onState() ");//name %s state: %d", .toAscii() , socketState);
   QTextStream out(stdout);
   out << "netPlayer::onState() "  << " , "<< socketState << "\n";  
}

void netPlayer::connectTo()
{
   pSocket->connectToHost ( "localhost",27531 );
}

void netPlayer::onError( QAbstractSocket::SocketError)
{
  qDebug("netPlayer::onError() name \n" ) ;
  QString err=pSocket->errorString();

   QTextStream out(stdout);
   out << "error " << err ;
  
}


void netPlayer::onReadyRead()
{

   bool ok=false;
   int bytes=pSocket->bytesAvailable();
   //QByteArray line;line.fill(' ',256);
   //pSocket->readLine( line.data(), 255 );
   while( (bytes=pSocket->bytesAvailable()) )
   {
      //int bytes=pSocket->bytesAvailable();
   
      QDataStream in(pSocket);
      ok=readF( in );
   }
}

void netPlayer::sendGameReq(bool followerIsWhite)
{
  qDebug("sendGameReq");
  Q_ASSERT( pSocket->isValid()  );
  
  dxpGameReq req;
  req.setFollowerWhite(followerIsWhite);
  QDataStream out( pSocket );
  req.writeTo( out );
}

bool netPlayer::readF( QDataStream& in )
{
   char c;
   int bytes=in.readRawData( &c, 1);
   if(bytes<=0)return 0;
   
   bool ok=false;
   switch ( c )
   {
      case 'R':
      {
         dxpGameReq req;
         ok=req.readFrom( in );
         bool normal= ( req.startingPosition[0]=='A' );
         bool followerIsWhite=( req.followerColor[0]=='W' );
         
         emit gameReq( normal , followerIsWhite );
         qDebug("netPlayer::readF()-> emit gameReq");
         //if(ok)
         //{
         //   dxpGameAcc acc;
         //   acc.writeTo( in );
         //}
      }
      break;
      
      case 'A':
      {
         dxpGameAcc acc;
         acc.readFrom( in );
         emit gameAcc( acc.isAccepted() );
      }
      break;
      
      case 'M':
      {
         dxpMove move;
         ok=move.readFrom( in );
         if(ok)
         {
            LegalMove LM;
            LM=move.toLegalMove();
            emit moveFinished( LM );
         }
      }
      break;
      
      case 'E':
      {
         dxpGameEnd gameEnd;
         gameEnd.readFrom( in );
      }
      break;
      
      case 'C':
      {
        dxpChat chat;
        chat.readFrom( in );
      }
      break;
      /*
      case 'B':
      pMsg=new dxpBackReq();
      pMsg->readFrom( in );
      break;
      
      case 'K':
      pMsg=new dxpBackAcc();
      pMsg->readFrom( in );
      break;
      */
      default:
      qDebug( "unrecognized character Protocol Error type:%c",c );
   }
   return ok;
}

void netPlayer::yourMove( const LegalMove *lastMove )
{
  if(!lastMove)return;
  
  dxpMove move;
  move.fromLegalMove( *lastMove );
  
  QDataStream out( pSocket );
  move.writeTo( out );
}

void netPlayer::acceptGame()
{
  dxpGameAcc acc;
  
  QDataStream out( pSocket );
  acc.writeTo( out );
}

void netPlayer::rejectGame()
{
  dxpGameAcc acc;
  
  acc.setReject();
  
  QDataStream out( pSocket );
  acc.writeTo( out );
}

#include "netplayer.moc"
