/***************************************************************************
 *   Copyright (C) 2005 by Piotr Banasiak <piotrbanasiak@onet.pl>          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to                             *
 *   the Free Software Foundation, 51 Franklin Street, Fifth Floor,        *
 *   Boston, MA 02110-1301, USA.                                           *
 ***************************************************************************/
#include "piecedelegate.h"
#include "board.h"
#include <QtGui>

PieceDelegate::PieceDelegate(QObject *parent)
 : QAbstractItemDelegate(parent)
{
    lightSQ=new QPixmap();
    darkSQ=new QPixmap();
    blackM=new QPixmap();
    whiteM=new QPixmap();
    blackK=new QPixmap();
    whiteK=new QPixmap();
    markedSQ=new QPixmap();
    indicatedSQ=new QPixmap();
    moveable=new QPixmap();

    lightSQ->load( ":/images/lightSQ.png" );
    darkSQ->load( ":/images/darkSQ.png" );
    blackM->load( ":/images/blackM.png" );
    whiteM->load( ":/images/whiteM.png" );
    blackK->load( ":/images/blackK.png" );
    whiteK->load( ":/images/whiteK.png" );
    markedSQ->load( ":/images/markedSQ.png" );
    indicatedSQ->load( ":/images/indicatedSQ.png" );
    moveable->load( ":/images/moveable.png" );
}

PieceDelegate::~PieceDelegate()
{
    delete lightSQ;
    delete darkSQ;
    delete blackM;
    delete whiteM;
    delete blackK;
    delete whiteK;
    delete markedSQ;
    delete indicatedSQ;
    delete moveable;
}

void PieceDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option,
                          const QModelIndex &index) const
{
    painter->setRenderHint(QPainter::Antialiasing);
    painter->setPen(Qt::NoPen);

    
    //int w= lightSQ->width();
    //int h= lightSQ->height();

    QVariant data=index.model()->data(index, Qt::DisplayRole);
    int piece=data.toInt();
    int mask=  Board::MAN | Board::KING | Board::WHITE | Board::BLACK ;
    int pieceKind=piece & mask;
    
    //model inform us about is that light square of dark , or any piece
    if( piece == Board::WHITE)
        painter->drawPixmap( option.rect.x(), option.rect.y(), *lightSQ );
    if( piece == Board::BLACK or (piece&Board::MAN)or piece&Board::KING )
        painter->drawPixmap( option.rect.x(), option.rect.y(), *darkSQ );

    switch (pieceKind)
    {
       case Board::whiteM:
       painter->drawPixmap( option.rect.x(), option.rect.y(), *whiteM );
       break;
       
       case Board::blackM:
       painter->drawPixmap( option.rect.x(), option.rect.y(), *blackM );
       break;
       
       case Board::whiteK:
       painter->drawPixmap( option.rect.x(), option.rect.y(), *whiteK );
       break;
       
       case Board::blackK:
       painter->drawPixmap( option.rect.x(), option.rect.y(), *blackK );
       break;
    }
    
    //uncomment if we want to mark moveable pieces
    
    //checks if the piece can move in current turn
    QVariant highlighted=index.model()->data( index, canMoveRole );
    if( highlighted.toInt() )
        painter->drawPixmap( option.rect.x(), option.rect.y(), *moveable );
    
    
    //mark the current piece
    QVariant isCurrent=index.model()->data( index, currentPieceRole );
    if( isCurrent.toInt() )
        painter->drawPixmap( option.rect.x(), option.rect.y(), *markedSQ );

    //mark last moved piece
    QVariant isLast=index.model()->data( index, lastMoveRole );
    if( isLast.toInt() )
        painter->drawPixmap( option.rect.x(), option.rect.y(), *markedSQ );
        
    //mark steps/path
    QVariant isNextStep=index.model()->data( index, nextStepRole );
    if( isNextStep.toInt() )
        painter->drawPixmap( option.rect.x(), option.rect.y(), *indicatedSQ );
}

QSize PieceDelegate::sizeHint(const QStyleOptionViewItem & /* option */,
                              const QModelIndex & /* index */) const
{
    QSize size = lightSQ->size();
    return size;
}

bool PieceDelegate::editorEvent ( QEvent * , QAbstractItemModel * model, const QStyleOptionViewItem & , const QModelIndex & index )
{
    QVariant notUsed("not used");
    model->setData( index, notUsed , Qt::EditRole);
    return false;
}




