/*
 * Copyright 2008 Marijn Kruisselbrink <m.kruisselbrink@student.tue.nl>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "Exceptions.h"
#include <cstdio>
#include <errno.h>
using namespace std;

Exception::Exception(string file, int line, string message)
    : file(file), line(line), message(message)
{
}

Exception::Exception(string message)
    : file(), line(-1), message(message)
{
}

Exception::~Exception()
{
}

void Exception::printException() const
{
    fprintf(stderr, "%s: %s\n\tat %s:%d\n", getType(), message.c_str(), file.c_str(), line);
}

#define ecase(code, desc) case code: return #code ": " desc ": "
const char* CError::getDesc(int code)
{
    switch(code) {
        case 0: return "no error: ";
        ecase(EPERM,	"Operation not permitted");
        ecase(ENOENT,	"No such file or directory");
        ecase(ESRCH,	"No such process");
        ecase(EINTR,	"Interrupted system call");
        ecase(EIO,	"Input/output error");
        ecase(ENXIO,	"Device not configured");
        ecase(E2BIG,	"Argument list too long");
        ecase(ENOEXEC,	"Exec format error");
        ecase(EBADF,	"Bad file descriptor");
        ecase(ECHILD,	"No child processes");
        ecase(EDEADLK,	"Resource deadlock avoided");
        ecase(ENOMEM,	"Cannot allocate memory");
        ecase(EACCES,	"Permission denied");
        ecase(EFAULT,	"Bad address");
        ecase(ENOTBLK,	"Block device required");
        ecase(EBUSY,	"Device busy");
        ecase(EEXIST,	"File exists");
        ecase(EXDEV,	"Cross-device link");
        ecase(ENODEV,	"Operation not supported by device");
        ecase(ENOTDIR,	"Not a directory");
        ecase(EISDIR,	"Is a directory");
        ecase(EINVAL,	"Invalid argument");
        ecase(ENFILE,	"Too many open files in system");
        ecase(EMFILE,	"Too many open files");
        ecase(ENOTTY,	"Inappropriate ioctl for device");
        ecase(ETXTBSY,	"Text file busy");
        ecase(EFBIG,	"File too large");
        ecase(ENOSPC,	"No space left on device");
        ecase(ESPIPE,	"Illegal seek");
        ecase(EROFS,	"Read-only file system");
        ecase(EMLINK,	"Too many links");
        ecase(EPIPE,	"Broken pipe");
        ecase(EDOM,	"Numerical argument out of domain");
        ecase(ERANGE,	"Result too large");
        ecase(EAGAIN,	"Resource temporarily unavailable/EWOULDBLOCK: Operation would block");
        ecase(EINPROGRESS,	"Opertaion now in progress");
        ecase(EALREADY,	"Operation already in progress");
        ecase(ENOTSOCK,	"Socket operation on non-socket");
        ecase(EDESTADDRREQ,	"Destination address required");
        ecase(EMSGSIZE,	"Message too long");
        ecase(EPROTOTYPE,	"Protocol wrong type for socket");
        ecase(ENOPROTOOPT,	"Protocol not available");
        ecase(EPROTONOSUPPORT,	"Protocol not supported");
        ecase(ESOCKTNOSUPPORT,	"Socket type not supported");
        ecase(ENOTSUP,	"Operation not supported");
        ecase(EPFNOSUPPORT,	"Protocol family not supported");
        ecase(EAFNOSUPPORT,	"Address family not supported by protocol family");
        ecase(EADDRINUSE,	"Address already in use");
        ecase(EADDRNOTAVAIL,	"Can't assign requested address");
        ecase(ENETDOWN,	"Network is down");
        ecase(ENETUNREACH,	"Network is unreachable");
        ecase(ENETRESET,	"Network dropped connection on reset");
        ecase(ECONNABORTED,	"Software caused connection abort");
        ecase(ECONNRESET,	"Connection reset by peer");
        ecase(ENOBUFS,	"No buffer space available");
        ecase(EISCONN,	"Socket is already connected");
        ecase(ENOTCONN,	"Socket is not connected");
        ecase(ESHUTDOWN,	"Can't send after socket shutdown");
        ecase(ETOOMANYREFS,	"Too many references, can't splice");
        ecase(ETIMEDOUT,	"Operation timed out");
        ecase(ECONNREFUSED,	"Connection refused");
        ecase(ELOOP,	"Too many levels of symbolic links");
        ecase(ENAMETOOLONG,	"File name too long");
        ecase(EHOSTDOWN,	"Host is down");
        ecase(EHOSTUNREACH,	"No route to host");
        ecase(ENOTEMPTY,	"Directory not empty");
//		ecase(EPROCLIM,	"Too many processes");
        ecase(EUSERS,	"Too many users");
        ecase(EDQUOT,	"Disc quota exceeded");
        ecase(ESTALE,	"Stale NFS file handle");
        // many more
        default: return "Unknown error code: ";
    }
}

#ifdef checkErrorCode
#undef checkErrorCode
#endif
int checkErrorCode(string file, int line, int result, string message)
{
    if (result < 0) throw new CError(file, line, message, errno);
    return result;
}
