/*
    gamefudatabaseconfig.cpp - Database configuration

    Copyright (c) 2006      by Michaël Larouche       <larouche@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/
#include "gamefucollectionconfig.h"

// Qt includes
#include <qlayout.h>
#include <qcheckbox.h>
#include <qsqldatabase.h>
#include <qtabwidget.h>

// KDE includes
#include <kdebug.h>
#include <kgenericfactory.h>
#include <klocale.h>
#include <kcombobox.h>

// Gamefu includes
#include <kcheckdirectoryselector.h>
#include <gamefupreferences.h>
#include <databasepreferences.h>

#include "collectionconfig_collection.h"
#include "collectionconfig_database.h"

using namespace Gamefu::UI;

class GamefuCollectionConfig::Private
{
public:
	CollectionConfig_Collection *collectionConfigUi;
	CollectionConfig_Database *databaseConfigUi;
};

typedef KGenericFactory<GamefuCollectionConfig, QWidget> GamefuCollectionConfigFactory;
K_EXPORT_COMPONENT_FACTORY( kcm_gamefu_collectionconfig, GamefuCollectionConfigFactory( "kcm_gamefu_collectionconfig" ) )

GamefuCollectionConfig::GamefuCollectionConfig(QWidget* parent, const char* /*name*/, const QStringList& args)
	: KCModule( GamefuCollectionConfigFactory::instance(), parent, args ), d( new Private() )
{
	( new QVBoxLayout( this ) )->setAutoAdd( true );

	QTabWidget *tabWidget = new QTabWidget(this, "tabWidget");
	
	d->collectionConfigUi = new CollectionConfig_Collection(tabWidget);
	addConfig( GamefuPreferences::self(), d->collectionConfigUi );
	tabWidget->addTab( d->collectionConfigUi, i18n("General") );
	
	d->databaseConfigUi = new CollectionConfig_Database(tabWidget);
	addConfig( Gamefu::DatabasePreferences::self(), d->databaseConfigUi );
	tabWidget->addTab( d->databaseConfigUi, i18n("Database") );

	load();
}

GamefuCollectionConfig::~GamefuCollectionConfig()
{
	delete d;
}

void GamefuCollectionConfig::load()
{
	KCModule::load();
	
	d->collectionConfigUi->dirSelect->setDirs( GamefuPreferences::self()->collectionDirectories() );

	loadDatabaseDrivers();
}

void GamefuCollectionConfig::save()
{
	KCModule::save();

	GamefuPreferences::self()->setCollectionDirectories( d->collectionConfigUi->dirSelect->dirs() );
	GamefuPreferences::self()->writeConfig();

	Gamefu::DatabasePreferences::self()->setDriver( d->databaseConfigUi->databaseDriver->currentText() );
	Gamefu::DatabasePreferences::self()->writeConfig();
}

void GamefuCollectionConfig::loadDatabaseDrivers()
{
	d->databaseConfigUi->databaseDriver->clear();

	QStringList driverList = QSqlDatabase::drivers ();

	int currentIndex = 0;
	QStringList::ConstIterator it, itEnd = driverList.constEnd();
	for( it = driverList.constBegin(); it != itEnd; ++it )
	{
		d->databaseConfigUi->databaseDriver->insertItem( *it );
		if( *it == Gamefu::DatabasePreferences::self()->driver() )
		{
			d->databaseConfigUi->databaseDriver->setCurrentItem(currentIndex);
		}
		currentIndex++;
	}
}

#include "gamefucollectionconfig.moc"
