/*
    listcollectionjob.h - Update collection in list view.

    Copyright (c) 2006      by Michaël Larouche       <larouche@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/
#ifndef LISTCOLLECTIONJOB_H
#define LISTCOLLECTIONJOB_H

#include <threadweaver.h>

namespace Gamefu
{
	class RomMetaInformation;
}

class CollectionViewer;
/**
 * @brief Fill collection list view with items from database
 * Handle full listing of the collection or a filtered set. Pass a SQL query to the constructor if you want to filter items.
 * @author Michaël Larouche <larouche@kde.org>
*/
class ListCollectionJob : public ThreadWeaver::DependentJob
{
	Q_OBJECT
public:
	/**
	 * @brief Create a new List Collection job.
	 * @param parent Parent object.
	 * @param collectionViewer CollectionViewer instance. Used to add items to the collection list view.
	 * @param sqlQuery SQL query for filtered items. Use an empty string to get all roms.
	 */
	ListCollectionJob(QObject *parent, CollectionViewer *collectionViewer, const QString &sqlQuery = QString() );
	/**
	 * d-tor
	 */
	~ListCollectionJob();

signals:
	/**
	 * @brief Notify when the job is complete.
	 */
	void listCollectionFinished();

private:
	/**
	 * @internal
	 * Do the dirty job.
	 */
	virtual bool doJob();
	/**
	 * @internal
	 * Emit listCollectionFinished().
	 */ 
	virtual void completeJob();
	/**
	 * @internal
	 * Each item listed create a event, and the item is added to the list view by this.
	 */
	virtual void customEvent(QCustomEvent *event);

private:
	class Private;
	Private *d;
};

#endif
