/*
    gamefu_launch - Command line frontend

    Copyright (c) 2006      by Michaël Larouche       <larouche@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/
#include "gamefu_launch.h"

// Qt includes
#include <qtimer.h>

// KDE includes
#include <kaboutdata.h>
#include <kcmdlineargs.h>
#include <klocale.h>
#include <kapplication.h>
#include <kglobal.h>
#include <kinstance.h>
#include <kconfig.h>
#include <kdebug.h>

// Gamefu includes
#include <gamelauncher.h>
#include <systemmanager.h>
#include <gamefupluginmanager.h>
#include <collectionaccess.h>

#include <iostream>

GamefuLaunch::GamefuLaunch(QObject *parent)
 : QObject(parent)
{}

void GamefuLaunch::setRom(const KURL &rom)
{
	romToLaunch = rom;
}

void GamefuLaunch::run()
{
	Gamefu::PluginManager::self(); // Look for available plugins
	// SystemManager must be created after PluginManager because it listen to a signal of PluginManager
	Gamefu::SystemManager::self();

	connect(Gamefu::PluginManager::self(), SIGNAL(loadingFinished()), this, SLOT(launchRom()));

	Gamefu::PluginManager::self()->load();

	// Init Collection acces.
	Gamefu::CollectionAccess::self()->init();
}

void GamefuLaunch::launchRom()
{
	launcher = new Gamefu::GameLauncher(this);
	connect(launcher, SIGNAL(emulatorExited()), kapp, SLOT(quit()));

	if( !launcher->launchGame( romToLaunch.path() ) )
	{
		kdError() << i18n("Error while launching %1. Please make sure that Gamefu is well configured and installed.").arg( romToLaunch.path() ) << endl;
		kapp->quit();
	}
	else
	{
		std::cout << i18n("Launching %1...").arg( romToLaunch.path() ).latin1() << std::endl;
	}
}

static const char description[] =
    I18N_NOOP("Gamefu Launcher - Command line frontend");

static const char version[] = "0.2 (Beta1)";

static KCmdLineOptions options[] =
{
   { "+ROM", I18N_NOOP( "ROM to open" ), 0 },
    KCmdLineLastOption
};

int main(int argc, char **argv)
{
	// HACK to read configuration from gamefu configuration
	KAboutData about("gamefu", I18N_NOOP("Gamefu launcher"), version, description,
		     KAboutData::License_GPL, I18N_NOOP("(C) 2005-2006 Gamefu Developers"), 0, "http://kamefu.pwsp.net/");
	about.addAuthor("Michaël Larouche", I18N_NOOP("Project Founder, Lead Developer"), "larouche@kde.org", "http://www.tehbisnatch.org/");
	about.setTranslator( I18N_NOOP("_: NAME OF TRANSLATORS\nYour names"), I18N_NOOP("_: EMAIL OF TRANSLATORS\nYour emails") );

	KCmdLineArgs::init(argc, argv, &about);
	KCmdLineArgs::addCmdLineOptions( options );

	KApplication app; // Need GUI because some plugins have GUI

	KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
	
	GamefuLaunch gamefuLaunch;

	if( args->count() > 0 )
	{
		KURL romToLoad = args->url(0);
		gamefuLaunch.setRom( romToLoad );
	}
	else
	{
		return 1;
	}

	QTimer::singleShot(0, &gamefuLaunch, SLOT(run()));

	return app.exec();
}

#include "gamefu_launch.moc"
