/*
    gamefuthumbnail.cpp - Create a KIO Thumb from the titlescreen in Gamefu Database.

    Copyright (c) 2006      by Michaël Larouche       <larouche@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/
#include "gamefuthumbnail.h"

// Qt includes
#include <qimage.h>
#include <qsqldatabase.h>
#include <qsqldriver.h>
#include <qsqlquery.h>
#include <qsqlfield.h>

// KDE includes
#include <kdemacros.h>
#include <kdebug.h>
#include <kinstance.h>
#include <kconfig.h>
#include <kstandarddirs.h>

extern "C"
{
KDE_EXPORT ThumbCreator *new_creator() { return new GamefuThumbCreator; }
}

bool GamefuThumbCreator::create(const QString &path, int, int, QImage &img)
{
	if( !createDbConnection() )
	{
		kdDebug() << "Connection to database failed." << endl;
		return false;
	}

	QSqlField *theField = new QSqlField("rom_url", QVariant::String);
	theField->setValue(path);
	QString formattedUrl = QSqlDatabase::database()->driver()->formatValue(theField);

	QSqlQuery sqlQuery;
	if( !sqlQuery.exec( QString::fromUtf8("SELECT rom_titlescreenpath FROM rom WHERE rom_url LIKE(%1)").arg( formattedUrl) ) )
	{
		kdDebug() << "SQL query failed." << endl;
		return false;
	}

	if( !sqlQuery.first() )
	{
		kdDebug() << "SQL query return 0 items." << endl;
		return false;
	}
	
	QString titleScreenPath = sqlQuery.value(0).toString();

	kdDebug() << "Found titlesreenPath: " << titleScreenPath << endl;

	if( !img.load( titleScreenPath ) )
	{
		kdDebug() << "Image failed to load." << endl;
		return false;
	}
	if( img.depth() != 32 )
		img = img.convertDepth( 32 );
	
	return true;
}

ThumbCreator::Flags GamefuThumbCreator::flags() const
{
	return static_cast<Flags>(BlendIcon);
}

bool GamefuThumbCreator::createDbConnection()
{
	KInstance fakeGamefu("gamefu");
	QString driver, hostname, database, username, password;

	KConfig *config = fakeGamefu.config();
	config->setGroup("Database");
	driver = config->readEntry("driver", "QSQLITE");
	database = config->readEntry( "database", locateLocal("data", "gamefu/gamefudatabase.dat", &fakeGamefu) );
	hostname = config->readEntry( "hostname", "localhost" );
	username = config->readEntry( "username", "root" );
	password = config->readEntry( "password", "" );

	QSqlDatabase *db = QSqlDatabase::addDatabase(driver);
	db->setHostName(hostname);
	db->setDatabaseName(database);
	db->setUserName(username);
	db->setPassword(password);

	if( !db->open() )
	{
		return false;
	}

	return true;
}
