/*
    ndsfile.h - Read a Nintendo DS(NDS) ROM file

    Copyright (c) 2006      by Thomas Köckerbauer     <k0055217@students.uni-linz.ac.at>

    Ported to Qt and imported in Gamefu by:
    Copyright (c) 2006      by Michaël Larouche       <larouche@kde.org>

     *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/
#ifndef NDSFILE_H
#define NDSFILE_H

class QString;
class QImage;

/**
 * @brief Nintendo DS ROM file metainformation reader.
 *
 * @author Thomas Köckerbauer
 * @author Michaël Larouche <larouche@kde.org>
 */
class NDSFile
{
public:
	enum LongTitleLang {
		LANG_Japanese = 0, LANG_English = 1, LANG_French = 2,
		LANG_German = 3, LANG_Italian = 4, LANG_Spanish = 5
	};

	NDSFile(const QString &path);
	~NDSFile();

	/**
	 * @brief Read the complete information from file.
	 */
	void readInfo();

	bool isValid() const;

	QString internalName() const;
	QString gameCode() const;
	QString maker() const;
	QString countryCode() const;
	QString title(LongTitleLang language) const;
	QString description(LongTitleLang language) const;

	bool isDsDownloadPlay() const;
	bool isGstDump() const;
	bool hasEmbeddedLoader() const;
	bool hasNDSmallLoader() const;
	
	/**
	 * @brief Get the rom size in MBit.
	 */
	int romSize() const;

	bool loadIcon(QImage &imag);

private:
	struct Maker {
		const char *makercode;
		const char *name;
	};

	struct CountryCode {
		const char countrycode;
		const char *name;
	};

	static CountryCode countries[];
	static Maker makers[];

	class Private;
	Private *d;
};

#endif
