/*
    systemmanager.h - Singleton class to System information

    Copyright (c) 2005-2006     by Michaël Larouche       <larouche@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/
#ifndef GAMEFUSYSTEMMANAGER_H
#define GAMEFUSYSTEMMANAGER_H

#include <qobject.h>
#include <kdemacros.h>

namespace Gamefu
{

class System;
class Plugin;

typedef QValueList<System*> SystemList;
/**
 * @brief This class manage all loaded system plugins.
 *
 * This a singleton class, access it with SystemManager::self()
 *
 * @author Michaël Larouche
*/
class KDE_EXPORT SystemManager : public QObject
{
	Q_OBJECT
	
public:
	static SystemManager *self();
	~SystemManager();

	/**
	 * Return a list of all loaded system plugins.
	 */
	SystemList loadedSystems() const;

	/**
	 * Return the list of all registred MIME type by the system plugins.
	 */
	QStringList getRegistredMimeTypes() const;

	/**
	 * Return a display name of a system based on the MIME type.
	 */
	QString guessSystemNameFromMimeType(const QString &mimeType) const;

private slots:
	void slotPluginLoaded(Gamefu::Plugin *plugin);

private:
	SystemManager(QObject *parent = 0, const char *name = 0);

private:
	class Private;
	Private *d;

	static SystemManager *s_self;
};

}

#endif
