/*
    imagefillerjob.h - Job that fill the database with images

    Copyright (c) 2006      by Michaël Larouche       <larouche@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/
#ifndef IMAGEFILLERJOB_H
#define IMAGEFILLERJOB_H

// Gamefu includes
#include <threadweaver.h>

#include "imagetypeenum.h"

class KURL;
class ImageFillerStrategy;

/**
 * @brief Job that fill images(titlescreen, screenshot) in database.
 * 
 * You must fill all the parameters before starting this job !
 * See setImageDirectory(), setImageType(), setImageFillerStrategy()
 *
 * @author Michaël Larouche <larouche@kde.org>
 */
class ImageFillerJob : public ThreadWeaver::DependentJob
{
	Q_OBJECT
public:
	/**
	 * @brief Create a new ImageFiller job.
	 * @param parent parent
	 */ 
	ImageFillerJob(QObject *parent);
	/**
	 * d-tor
	 */
	~ImageFillerJob();

	/**
	 * @brief Set the image directory.
	 *
	 * This is the source directory for the image.
	 * @param directory Image directory
	 */
	void setImageDirectory(const KURL &directory);
	/**
	 * @brief Set the image type
	 *
	 * Image type is either a titlescreen or a screenshot.
	 * @param imageType Value in the ImageEnum::ImageType enum.
	 */
	void setImageType(ImageEnum::ImageType imageType);
	/**
	 * @brief Set the image filler strategy.
	 *
	 * A Image filler strategy is a strategy to format the image
	 * filename from the ROM filename.
	 *
	 * @param strategy A instance of ImageFillerStrategy
	 */
	void setImageFillerStrategy(ImageFillerStrategy *strategy);

protected:
	/**
	 * @internal
	 * @brief Do the actual job
	 */
	virtual bool doJob();
	/**
	 * @internal
	 * Do nothing
	 */
	virtual void completeJob();

private:
	class Private;
	Private *d;
};

#endif
