/*
    imagefillerstrategy.cpp - Image filename format strategy

    Copyright (c) 2006      by Michaël Larouche       <larouche@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/
#include "imagefillerstrategy.h"

// Qt includes
#include <qregexp.h>

// Offline List regexp: ^(\\d+)
// Filename without extension: (.*)\\..+

QString FilenameImageFillerStrategy::getImageFilename(const QString &fileName, ImageEnum::ImageType imageType)
{
	Q_UNUSED(imageType);

	QString result;
	
	QRegExp filenameExp( QString::fromLatin1("(.*)\\..+") );

	// Get the filename without extensiom
	if( filenameExp.search(fileName) > -1 )
	{
		result = filenameExp.cap(1) + QString::fromLatin1(".png");
	}

	return result;
}

QString ReleaseImageFillerStrategy::getImageFilename(const QString &fileName, ImageEnum::ImageType imageType)
{
	QString result;
	
	QRegExp releaseNumberExp( QString::fromLatin1("^(\\d+)") );
	if( releaseNumberExp.search(fileName) > -1 )
	{
		// We need to convert the release number to a integer then
		// convert it back to a string to remove the trailling zeros.
		// 0001 -> 1
		QString tempReleaseNumber = releaseNumberExp.cap(1);
		int releaseNumber = tempReleaseNumber.toInt();

		// Offline List use suffixes to determine if the image
		// is a titlescreen or a screenshot.
		QChar filenameSuffix;
		switch(imageType)
		{
			case ImageEnum::Titlescreen:
				filenameSuffix = QChar('a');
				break;
			case ImageEnum::Screenshot:
				filenameSuffix = QChar('b');
				break;
		}

		result = QString::number(releaseNumber) + filenameSuffix + QString::fromLatin1(".png");
	}

	return result;
}