/*
    randomgameplugin.cpp - Random game plugin

    Copyright (c) 2007      by Michaël Larouche       <larouche@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/
#include "randomgameplugin.h"

// Qt includes
#include <qsqlquery.h>

// KDE includes
#include <kaction.h>
#include <kdebug.h>
#include <kgenericfactory.h>
#include <kiconloader.h>
#include <klocale.h>
#include <kxmlguiclient.h>
#include <kmessagebox.h>

// Gamefu includes
#include <collectionaccess.h>
#include <notifymanager.h>
#include <gamelauncher.h>

typedef KGenericFactory<RandomGamePlugin> randomGameFactory;
K_EXPORT_COMPONENT_FACTORY(gamefu_randomgame, randomGameFactory( "gamefu_randomgame" ))

RandomGamePlugin::RandomGamePlugin(QObject *parent, const char *name, const QStringList &/*args*/)
 : Gamefu::Plugin( randomGameFactory::instance(), parent, name)
{
	// Create actions
	new KAction(i18n("Select Random Game..."), SmallIcon("roll"), 0, this, SLOT(collectionRandomGame()), actionCollection(), "collectionRandomGame");

	setXMLFile("randomgameui.rc");
}

RandomGamePlugin::~RandomGamePlugin()
{
}

void RandomGamePlugin::collectionRandomGame()
{
	QString sqlQuery = QString::fromUtf8("SELECT rom_name, rom_url FROM rom ORDER BY random() LIMIT 1");

	QSqlQuery randomQuery = Gamefu::CollectionAccess::self()->executeQuery(sqlQuery);
	if( randomQuery.next() )
	{
		QString romName = randomQuery.value(0).toString();
		QString romUrl = randomQuery.value(1).toString();

		int result = KMessageBox::questionYesNo(0, i18n("First arg is game name and second is rom url", "Do you want to play this game?\nName: %1\nLocation: %2\n").arg(romName).arg(romUrl) );
		if( result == KMessageBox::Yes )
		{
			Gamefu::GameLauncher *launcher = new Gamefu::GameLauncher(this);
			launcher->launchGame(romUrl);
		}
	}
	else
	{
		Gamefu::NotifyManager::self()->longMessage( i18n("Could not get a random game from the collection. Check if you have created the collection or if the collection database file is accessible."), Gamefu::NotifyManager::Error ); 
	}
}

#include "randomgameplugin.moc"
