

#include "hiq.h"
#include "pref.h"

#include <qdragobject.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <qpushbutton.h>
#include <QtGui/QPrinter>
#include <QtGui/QPrintDialog>

#include <kglobal.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kdeversion.h>
#include <kstatusbar.h>
#include <kaccel.h>
#include <kio/netaccess.h>
#include <kfiledialog.h>
#include <kconfig.h>
#include <kurl.h>
#include <kurldrag.h>
#include <kurlrequesterdlg.h>

#include <kstdaccel.h>
#include <qpopupmenu.h>
#include <kmenubar.h>
#include <kaction.h>
#include <kstdaction.h>
#include "peg.h"
#include <kmessagebox.h>
#include <qcheckbox.h>
#include <qradiobutton.h>
#include <iostream>
using namespace std;

#define defaultAdvancePd 50
#define defaultAdvancePdFast 20


HiQ::HiQ()
: KMainWindow( 0, i18n("HiQ") ) {

    //config->setGroup("hiq");
    readProperties(kapp->config());

    m_view = new HiQView(new QCanvas(), // TODO: find right dimensions
                         this, 0, animateAll, m_style, kapp, advancePeriod, advancePeriodFast);

    m_style = m_view->style();

    setFixedSize(minimumSize());

    // accept dnd
    setAcceptDrops(true);

    // tell the KMainWindow that this is indeed the main widget
    setCentralWidget(m_view);

    // then, setup our actions
    setupActions();

    // and a status bar
    statusBar()->show();

    // Apply the create the main window and ask the mainwindow to
    // automatically save settings if changed: window size, toolbar
    // position, icon size, etc.  Also to add actions for the statusbar
    // toolbar, and keybindings if necessary.
    //setupGUI();
    createGUI();

    //toolBar()->insertSeparator(7);
    //actionCollection()->action("toggle_editable")->plug(toolBar(), 8);

}

HiQ::~HiQ() {}

void HiQ::load(const KURL& url) {}

void HiQ::setupActions() {
    KStdAction::openNew(this, SLOT(fileNew()), actionCollection());
    KStdAction::open(this, SLOT(fileOpen()), actionCollection());
    KStdAction::save(this, SLOT(fileSave()), actionCollection());
    KStdAction::saveAs(this, SLOT(fileSaveAs()), actionCollection());
    KStdAction::print(this, SLOT(filePrint()), actionCollection());
    KStdAction::quit(kapp, SLOT(quit()), actionCollection());
    KStdAction::preferences(this, SLOT(optionsPreferences()), actionCollection());
    KAction *undoAction = KStdAction::undo(m_view, SLOT(undo()), actionCollection());
    connect(m_view, SIGNAL(undoIsLegal(bool)), undoAction, SLOT(setEnabled(bool)));
    undoAction->setEnabled(false);
    KAction *redoAction = KStdAction::redo(m_view, SLOT(redo()), actionCollection());
    connect(m_view, SIGNAL(redoIsLegal(bool)), redoAction, SLOT(setEnabled(bool)));
    redoAction->setEnabled(false);

    editable = new KToggleAction(i18n("&Toggle Editable"), "lock",
                                 CTRL+Key_E, 0, 0, actionCollection(), "toggle_editable");
    connect( editable, SIGNAL( toggled(bool) ), m_view, SLOT( setEditable(bool) ) );
    connect( editable, SIGNAL( toggled(bool) ), this, SLOT( setEditableIcon(bool) ) );

}

void HiQ::setEditableIcon(bool isEditable) {
    editable->setIcon(isEditable?"unlock":"lock");
}

void HiQ::saveProperties(KConfig *config) {
    config->setGroup("Settings");
    config->writeEntry("style", m_style);
    config->writeEntry("animate", animateAll);
    config->writeEntry("delayAnimateAll", advancePeriod);
    config->writeEntry("delayAnimateOne", advancePeriodFast);
    config->sync();
}
void HiQ::readProperties(KConfig *config) {
    config->setGroup("Settings");
    m_style = config->readEntry("style", "marble");
    animateAll = config->readBoolEntry("animate", false);
    advancePeriod = config->readNumEntry("delayAnimateAll", defaultAdvancePd);
    advancePeriodFast = config->readNumEntry("delayAnimateOne", defaultAdvancePdFast);
}
void HiQ::updateProperties() {
    m_view->setAnimated(animateAll);
    m_view->setStyle(m_style);
    m_view->setUpdateDelay(advancePeriod, advancePeriodFast);
}


void HiQ::fileNew() {
    // this slot is called whenever the File->New menu is selected,
    // the New shortcut is pressed (usually CTRL+N) or the New toolbar
    // button is clicked

    // create a new window
    m_view->reset();
}

void HiQ::fileOpen() {
    KURL url = KFileDialog::getOpenURL(QString::null, QString::null, this, i18n("Open Location"));
}

void HiQ::fileSave() {}

void HiQ::fileSaveAs() {
    // this slot is called whenever the File->Save As menu is selected,
    KURL file_url = KFileDialog::getSaveURL();
}

void HiQ::filePrint() {
    // this slot is called whenever the File->Print menu is selected,
    // the Print shortcut is pressed (usually CTRL+P) or the Print toolbar
    // button is clicked
    /*
    if (!m_printer)
        m_printer = new QPrinter;
    QPrintDialog printDialog(&printer, this);
    printDialog.setWindowTitle(i18n("Print Dialog Title"));
    if (printDialog.exec()) {
        // setup the printer.  with Qt, you always "print" to a
        // QPainter.. whether the output medium is a pixmap, a screen,
        // or paper
        QPainter p;
        p.begin(m_printer);

        // we let our view do the actual printing
        QPaintDeviceMetrics metrics(m_printer);
        m_view->print(&p, metrics.height(), metrics.width());

        // and send the result to the printer
        p.end();
    }
    */
}

void HiQ::optionsPreferences() {
    // popup some sort of preference dialog, here
    HiQPreferences dlg(animateAll, advancePeriod, advancePeriodFast, m_style);
    if (dlg.exec()) {
        animateAll = (dlg.m_prefGeneral->anim)->isChecked();
        advancePeriod = (dlg.m_prefGeneral->animSlider)->value();
        advancePeriodFast = (dlg.m_prefGeneral->noAnimSlider)->value();
        if (dlg.m_prefTheme->metal->isChecked()) {
            m_style = "metal";
        } else if (dlg.m_prefTheme->wood->isChecked()) {
            m_style = "wood";
        } else {
            m_style = "marble";
        }
        updateProperties();
        saveProperties(kapp->config());
    }
}

void HiQ::changeStatusbar(const QString& text) {
    // display the text on the statusbar
    statusBar()->message(text);
}

void HiQ::changeCaption(const QString& text) {
    // display the text on the caption
    setCaption(text);
}
#include "hiq.moc"
