#include "peg.h"
#include <qpushbutton.h>
#include <qlabel.h>
#include <qimage.h>
#include <qmovie.h>
#include <cstdlib>
#include <qcanvas.h>
#include "historypart.h"
using namespace std;

const int numCols = 7;
const int numRows = 7;

Peg::Peg(QCanvas *canv, int row, int col, Peg **pegArr, QCanvasPixmapArray *mov) {
    curRow = row;
    curCol = col;
    pegArray = pegArr;
    sprite = new PegSprite(mov, canv);
    pegHeight = sprite->width();
    pegWidth = sprite->height();
    moveBack();
    getSprite()->setVisible(true);
}

Peg::~Peg() {}
void Peg::destroy() {
    getSprite()->hide();
    //delete this;
}
int Peg::getRow() {
    return curRow;
}
int Peg::getCol() {
    return curCol;
}
HistoryPart* Peg::moveTo(int row, int col) {
    if (getRow() != row || getCol() != col) {
        if (checkLegalMove(row, col)) {
            HistoryPart * part = performMove(row, col);
            moveBack();
            return part;
        } else {
            moveBack();
            return 0;
        }
    } else {
        moveBack();
        return 0;
    }
}
PegSprite* Peg::getSprite() {
    return sprite;
}
// returns -1 if illegal row & col
int Peg::convert(int row, int col) {
    int index = 0;
    if (!((row >= 2 && row <= 4) || (col >= 2 && col <= 4))) {
        return -1;
    }
    for (int i = 0; i < numRows; i++) {
        for (int j = 0; j < numCols; j++) {
            if (i == row && j == col) {
                return index;
            }
            if (i >= 2 && i <= 4 ||
                    j >= 2 && j <= 4) {
                index++;
            }
        }
    }
    return -1;
}

HistoryPart * Peg::performMove(int toRow, int toCol) {
    pegArray[convert((toRow+curRow)/2, (toCol+curCol)/2)]->destroy();
    pegArray[convert((toRow+curRow)/2, (toCol+curCol)/2)] = 0; // remove jumpover piece
    pegArray[convert(toRow, toCol)] = this;
    pegArray[convert(curRow, curCol)] = 0;
    HistoryPart *part = new HistoryPart(curRow, curCol, toRow, toCol);
    curRow = toRow;
    curCol = toCol;
    return part;
}

void Peg::moveBack() {
    sprite->setX(curCol*pegWidth);
    sprite->setY(curRow*pegHeight);
    sprite->setZ(1);
}

// checks whether a move from "from" to "to" is legal
bool Peg::checkLegalMove(int toRow, int toCol) {
    if (toRow >= 0 && toRow <=7 && toCol >= 0 && toCol <= 7 &&
            (toRow >= 2 && toRow <= 4 ||
             toCol >= 2 && toCol <= 4)) {
        if (pegArray[convert(toRow, toCol)] == 0) {
            if (toCol == curCol) {
                if (toRow - curRow == 2) {
                    return pegArray[convert(toRow-1, toCol)] != 0;
                } else if (curRow - toRow == 2) {
                    return pegArray[convert(toRow+1, toCol)] != 0;
                } else {
                    return false;
                }
            } else if (toRow == curRow) {
                if (toCol - curCol == 2) {
                    return pegArray[convert(toRow, toCol-1)] != 0;
                } else if (curCol - toCol == 2) {
                    return pegArray[convert(toRow, toCol+1)] != 0;
                } else {
                    return false;
                }
            } else {
                return false;
            }
        } else {
            return false;
        }
    } else {
        return false;
    }
}
//#include "peg.moc"
