
/* Yo Emacs, this -*- C++ -*-

  Copyright (C) 1999-2001 Jens Hoefkens
  jens@hoefkens.com

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

  $Id: kbgfibschat.cpp 883734 2008-11-13 13:29:35Z asmirnov $

*/


#include "kbgfibschat.h"

//QT includes
#include <QApplication>
#include <QClipboard>
#include <QLabel>
#include <QMultiHash>
#include <QRegExp>
#include <QPainter>

//Added by qt3to4:
  #include <QGridLayout>
  #include <QHideEvent>
  #include <QShowEvent>
  #include <q3listbox.h>

//KDE includes
#include <kaction.h>
#include <kactioncollection.h>
#include <kconfig.h>
#include <kconfiggroup.h>
#include <kdebug.h>
#include <klocale.h>
#include <kstandardaction.h>
#include <ktoggleaction.h>

//kdegames includes
#include <kchatbasemodel.h>
#include <kchatbaseitemdelegate.h>

#include "kbgfibs.h"
#include "clip.h"
#include "version.h"

class KBgChatPrivate
{
public:
    /*
     * Name of the users
     */
    QString mName[2];

    /*
     * Hold and assemble info text
     */
    QString mText;

    /*
     * Numbers of the private action list.
     */
    enum Privact {Inquire, InviteD, Invite1, Invite2, Invite3, Invite4,
                  Invite5, InviteR, InviteU, Silent,
                  Talk, Gag, Ungag, Cleargag, Copy, Clear, Close, MaxAction};

    /*
     * Available actions
     */
    QAction *mAct[MaxAction];

    /*
     * Context menu and invitation menu
     */
    QMenu *mChat, *mInvt;

    /*
     * list of users we do not want to hear shouting
     */
    QStringList mGag;

    /*
     * Listbox needed by the setup dialog
     */
    Q3ListBox *mLb;

    /*
     * Internal ID to name mapping
     */
    QMultiHash<QString, int *> *mName2ID;

    const KBgEngineFIBS*  engine;
};


class kBgFibsChatItemDelegate:public KChatBaseItemDelegate
{
private:
//the only method overrided here is paint
public:
    explicit kBgFibsChatItemDelegate(QObject *parent = 0):KChatBaseItemDelegate (parent){}

    virtual void paint ( QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
    {
        //kDebug(10500) << "kBgFibsChatItemDelegate::paint";

        const KChatBaseMessage m  = index.model()->data(index, Qt::DisplayRole).value<KChatBaseMessage>();


        painter->setFont ( ( ( KChatBaseModel* ) index.model() )->nameFont() ); 
        painter->setPen (QColor(0, 0, 255, 127)); //TODO: move color to font settings
        painter->drawText ( option.rect.x(), QFontMetrics ( option.font ).height() +option.rect.y(), i18n ( "%1: ",m.first ));

        painter->setFont ( ( ( KChatBaseModel* ) index.model() )->messageFont() );
        painter->setPen (QColor(0, 0, 255, 127)); //TODO: move color to font settings
        painter->drawText ( option.rect.x() + 3 + QFontMetrics ( ( ( KChatBaseModel* ) index.model() )->nameFont() ).width ( i18n ( "%1: ",m.first ) ),
                            QFontMetrics ( option.font ).height() +option.rect.y(), m.second );
    }
};

/*
 * Constructor of the chat window.
 */
KBgFibsChat::KBgFibsChat(const KBgEngineFIBS* engine, QWidget *parent)
    : KChat(parent, new KChatBaseModel(parent), new kBgFibsChatItemDelegate(parent), false),
    d(new KBgChatPrivate)
{
    d->mName[0].clear();
    d->mChat = 0;
    d->mInvt = new QMenu();
    d->engine = engine;

    setAutoAddMessages(false); // we get an echo from FIBS
    setFromNickname(i18n("%1 user", QLatin1String(PROG_NAME)));

    if (!insertSendingEntry(i18n("Kibitz to watchers and players"), CLIP_YOU_KIBITZ))
        kDebug(10500) << "adding kibitz";
    if (!insertSendingEntry(i18n("Whisper to watchers only"), CLIP_YOU_WHISPER))
        kDebug(10500) << "adding whisper";

    connect(this, SIGNAL(signalSendMessage(int, const QString &)),
            this, SLOT(handleCommand(int, const QString &)));

    d->mName2ID = new QMultiHash<QString, int *>;

    /*
     * some eye candy :)
     */
    setWindowIcon(qApp->windowIcon().pixmap(IconSize(KIconLoader::Small),IconSize(KIconLoader::Small)));
    setWindowTitle(i18n("Chat Window"));

    this->setWhatsThis( i18n("This is the chat window.\n\n"
                               "The text in this window is colored depending on whether "
                               "it is directed at you personally, shouted to the general "
                               "FIBS population, has been said by you, or is of general "
                               "interest. If you select the name of a player, the context "
                               "contains entries specifically geared towards that player."));
    /*
     * Define set of available actions
     */
    KActionCollection* actions = new KActionCollection(this);

    d->mAct[KBgChatPrivate::Inquire] = actions->addAction("fibs_info_on");
    d->mAct[KBgChatPrivate::Inquire]->setIcon(KIcon("help-contents"));
    d->mAct[KBgChatPrivate::Inquire]->setText(i18n("Info On"));
    connect(d->mAct[KBgChatPrivate::Inquire], SIGNAL(triggered(bool) ), SLOT(slotInquire()));

    d->mAct[KBgChatPrivate::Talk]    = actions->addAction("fibs_talk");
    d->mAct[KBgChatPrivate::Talk]->setIcon(KIcon(PROG_NAME "-chat.png"));
    d->mAct[KBgChatPrivate::Talk]->setText(i18n("Talk To"));
    connect(d->mAct[KBgChatPrivate::Talk], SIGNAL(triggered(bool) ), SLOT(slotTalk()));

    d->mAct[KBgChatPrivate::InviteD] = actions->addAction("fibs_use_dialog");
    d->mAct[KBgChatPrivate::InviteD]->setText(i18n("Use Dialog"));
    connect(d->mAct[KBgChatPrivate::InviteD], SIGNAL(triggered(bool) ), SLOT(slotInviteD()));

    d->mAct[KBgChatPrivate::Invite1] = actions->addAction("fibs_pm_1");
    d->mAct[KBgChatPrivate::Invite1]->setText(i18n("1 Point Match"));
    connect(d->mAct[KBgChatPrivate::Invite1], SIGNAL(triggered(bool) ), SLOT(slotInvite1()));

    d->mAct[KBgChatPrivate::Invite2] = actions->addAction("fibs_pm_2");
    d->mAct[KBgChatPrivate::Invite2]->setText(i18n("3 Point Match"));
    connect(d->mAct[KBgChatPrivate::Invite2], SIGNAL(triggered(bool) ), SLOT(slotInvite2()));

    d->mAct[KBgChatPrivate::Invite3] = actions->addAction("fibs_pm_3");
    d->mAct[KBgChatPrivate::Invite3]->setText(i18n("5 Point Match"));
    connect(d->mAct[KBgChatPrivate::Invite3], SIGNAL(triggered(bool) ), SLOT(slotInvite3()));

    d->mAct[KBgChatPrivate::Invite4] = actions->addAction("fibs_pm_4");
    d->mAct[KBgChatPrivate::Invite4]->setText(i18n("7 Point Match"));
    connect(d->mAct[KBgChatPrivate::Invite4], SIGNAL(triggered(bool) ), SLOT(slotInvite4()));

    d->mAct[KBgChatPrivate::Invite5] = actions->addAction("fibs_pm_5");
    d->mAct[KBgChatPrivate::Invite5]->setText(i18n("9 Point Match"));
    connect(d->mAct[KBgChatPrivate::Invite5], SIGNAL(triggered(bool) ), SLOT(slotInvite5()));

    d->mAct[KBgChatPrivate::InviteU] = actions->addAction("fibs_unlimited");
    d->mAct[KBgChatPrivate::InviteU]->setText(i18n("Unlimited"));
    connect(d->mAct[KBgChatPrivate::InviteU], SIGNAL(triggered(bool) ), SLOT(slotInviteU()));

    d->mAct[KBgChatPrivate::InviteR] = actions->addAction("fibs_resume");
    d->mAct[KBgChatPrivate::InviteR]->setText(i18n("Resume"));
    connect(d->mAct[KBgChatPrivate::InviteR], SIGNAL(triggered(bool) ), SLOT(slotInviteR()));

    d->mInvt->addAction( d->mAct[KBgChatPrivate::InviteD] );

    d->mInvt->addSeparator();

    d->mInvt->addAction( d->mAct[KBgChatPrivate::Invite1] );
    d->mInvt->addAction( d->mAct[KBgChatPrivate::Invite2] );
    d->mInvt->addAction( d->mAct[KBgChatPrivate::Invite3] );
    d->mInvt->addAction( d->mAct[KBgChatPrivate::Invite4] );
    d->mInvt->addAction( d->mAct[KBgChatPrivate::Invite5] );

    d->mInvt->addSeparator();

    d->mInvt->addAction( d->mAct[KBgChatPrivate::InviteU] );
    d->mInvt->addAction( d->mAct[KBgChatPrivate::InviteR] );

    d->mAct[KBgChatPrivate::Gag] = actions->addAction("fibs_gag");
    d->mAct[KBgChatPrivate::Gag]->setText(i18n("Gag"));
    connect(d->mAct[KBgChatPrivate::Gag], SIGNAL(triggered(bool) ), SLOT(slotGag()));

    d->mAct[KBgChatPrivate::Ungag] = actions->addAction("fibs_ungag");
    d->mAct[KBgChatPrivate::Ungag]->setText(i18n("Ungag"));
    connect(d->mAct[KBgChatPrivate::Ungag], SIGNAL(triggered(bool) ), SLOT(slotUngag()));

    d->mAct[KBgChatPrivate::Cleargag] = actions->addAction("fibs_clear_gags");
    d->mAct[KBgChatPrivate::Cleargag]->setText(i18n("Clear Gag List"));
    connect(d->mAct[KBgChatPrivate::Cleargag], SIGNAL(triggered(bool) ), SLOT(slotCleargag()));

    d->mAct[KBgChatPrivate::Copy]     = KStandardAction::copy(this, SLOT(slotCopy()), this);
    actions->addAction(d->mAct[KBgChatPrivate::Copy]->objectName(), d->mAct[KBgChatPrivate::Copy]);

    d->mAct[KBgChatPrivate::Clear] = actions->addAction("fibs_clear");
    d->mAct[KBgChatPrivate::Clear]->setText(i18n("Clear"));
    connect(d->mAct[KBgChatPrivate::Clear], SIGNAL(triggered(bool) ), SLOT(slotClear()));

    d->mAct[KBgChatPrivate::Close] = KStandardAction::close(this, SLOT(hide()), this);
    actions->addAction(d->mAct[KBgChatPrivate::Close]->objectName(), d->mAct[KBgChatPrivate::Close]);

    d->mAct[KBgChatPrivate::Silent] = new KToggleAction(i18n("Silent"), this);
    actions->addAction(QString(), d->mAct[KBgChatPrivate::Silent]);
    connect(d->mAct[KBgChatPrivate::Silent], SIGNAL(triggered(bool) ), SLOT(slotSilent()));

    //addMessage("test_user", "test_message");
}


/*
 * Destructor
 */
KBgFibsChat::~KBgFibsChat()
{
    delete d->mName2ID;
    delete d->mChat; // safe to delete NULL pointers
    delete d->mInvt;
    delete d;
}


// == configuration handling ===================================================

/*
 * Restore the previously stored settings
 */
void KBgFibsChat::readConfig(KConfig* conf) //TODO: use conf
{
    KConfigGroup config = KGlobal::config()->group("chat window");

    QPoint pos = config.readEntry("ori", QPoint( 10, 10 ));
    setGeometry(pos.x(), pos.y(), config.readEntry("wdt",460), config.readEntry("hgt",200));

    config.readEntry("vis", false) ? show() : hide();

    ((KToggleAction *)d->mAct[KBgChatPrivate::Silent])->setChecked(config.readEntry("sil", false));

    d->mGag = config.readEntry("gag",QStringList());
}

/*
 * Save the current settings to disk
 */
void KBgFibsChat::saveConfig(KConfig* conf) //TODO: use conf
{
    KConfigGroup config = KGlobal::config()->group("chat window");

    config.writeEntry("ori", pos());
    config.writeEntry("hgt", height());
    config.writeEntry("wdt", width());
    config.writeEntry("vis", isVisible());

    config.writeEntry("sil", ((KToggleAction *)d->mAct[KBgChatPrivate::Silent])->isChecked());

    config.writeEntry("gag", d->mGag);
}


/*
 * Setup dialog page of the player list - allow the user to select the
 * columns to show
 *
 * FIXME: need to be able to set font here KChatBase::setBothFont(const QFont& font)
 */
void KBgFibsChat::getSetupPages(QTabWidget *nb, int space)
{
    /*
     * Main Widget
     * ===========
     */
    QWidget *w = new QWidget(nb);
    QGridLayout *gl = new QGridLayout(w);
    gl->setMargin(space);

    d->mLb = new Q3ListBox(w);
    d->mLb->setMultiSelection(true);

    d->mLb->insertStringList(d->mGag);

    QLabel *info = new QLabel(w);
    info->setText(i18n("Select users to be removed from the gag list."));

    w->setWhatsThis( i18n("Select all the users you want "
                            "to remove from the gag list "
                            "and then click OK. Afterwards "
                            "you will again hear what they shout."));

    gl->addWidget(d->mLb, 0, 0);
    gl->addWidget(info, 1, 0);

    /*
     * put in the page
     * ===============
     */
    gl->activate();
    w->adjustSize();
    w->setMinimumSize(w->size());
    nb->addTab(w, i18n("&Gag List"));
}

/*
 * Remove all the selected entries from the gag list
 */
void KBgFibsChat::setupOk()
{
    for (uint i = 0; i < d->mLb->count(); ++i) {
        if (d->mLb->isSelected(i))
            d->mGag.removeAll(d->mLb->text(i));
    }
    d->mLb->clear();
    d->mLb->insertStringList(d->mGag);
}

/*
 * Don't do anything
 */
void KBgFibsChat::setupCancel()
{
    // empty
}

/*
 * By default, all players stay in the gag list
 */
void KBgFibsChat::setupDefault()
{
    d->mLb->clearSelection();
}

/*
 * Catch hide events, so the engine's menu can be update.
 */
void KBgFibsChat::showEvent(QShowEvent *e)
{
    Q_UNUSED(e)
    //Q3Frame::showEvent(e);
    emit windowVisible(true);
}

/*
 * Catch hide events, so the engine's menu can be update.
 */
void KBgFibsChat::hideEvent(QHideEvent *e)
{
    Q_UNUSED(e)
    emit windowVisible(false);
    //Q3Frame::hideEvent(e);
}

/*
 * At the beginning of a game, add the name to the list and switch to
 * kibitz mode.
 */
void KBgFibsChat::startGame(const QString &name)
{
    int *id = getIdByName(d->mName[1] = name);
    if (!id) {
        id = new int(nextId());
        d->mName2ID->insert(name, id);
        insertSendingEntry(i18n("Talk to %1", name), *id);
    }
    setSendingEntry(CLIP_YOU_KIBITZ);
}

/*
 * At the end of a game, we switch to talk mode.
 */
void KBgFibsChat::endGame()
{
    int *id = getIdByName(d->mName[1]);
    if (id)
        setSendingEntry(*id);
    else
        setSendingEntry(SendToAll);
}

/*
 * returns pointer to user id by name
 * TODO add const attr
 */
int* KBgFibsChat::getIdByName(const QString& name){
    return d->mName2ID->value(name);
}

/*
 * Set the chat window ready to talk to name
 */
void KBgFibsChat::fibsTalk(const QString &name)
{
    int *id = getIdByName(name); 
    if (!id) {
        id = new int(nextId());
        d->mName2ID->insert(name, id);
        insertSendingEntry(i18n("Talk to %1", name), *id);
    }
    setSendingEntry(*id);
}

/*
 * Remove the player from the combo box when he/she logs out.
 */
//TODO: check if KChat remove player is the same
void KBgFibsChat::deletePlayer(const QString &name)
{
    int *id = getIdByName(name);
    if (id) {
        removeSendingEntry(*id);
        delete d->mName2ID->take(name);
    }
}

/*
 * Take action when the user presses return in the line edit control.
 */
void KBgFibsChat::handleCommand(int id, const QString& msg)
{
    Q_UNUSED(id)
    int realID = sendingEntry();

    switch (realID) {
    case SendToAll:
        emit fibsCommand("shout " + msg);
        break;
    case CLIP_YOU_KIBITZ:
        emit fibsCommand("kibitz " + msg);
        break;
    case CLIP_YOU_WHISPER:
        emit fibsCommand("whisper " + msg);
        break;
    default:
        QHashIterator<QString, int *> it(*(d->mName2ID));

	while (it.hasNext()) {
	    it.next();
            if (*it.value() == realID) {
                emit fibsCommand("tell " + it.key() + ' ' + msg);
                return;
            }
        }
        kDebug(10500) << "unrecognized ID in KBgFibsChat::handleCommand: "<<realID<<", msg="<<msg;
    }
}


// == handle strings from the server ===========================================

/*
 * A message from the server that should be handled by us. If necessary,
 * we replace the CLIP number by a string and put the line into the window.
 *
 * This function emits the string in rich text format with the signal
 * personalMessage - again: the string contains rich text!
 */
void KBgFibsChat::handleData(const QString &msg)
{
    QString clip = msg.left(msg.indexOf(' ')), user, cMsg = msg;
    QDateTime date;

    bool flag = false;
    int cmd = clip.toInt(&flag);

    if (flag) {
        cMsg.replace(0, cMsg.indexOf(' ')+1, "");

        user = cMsg.left(cMsg.indexOf(' '));

        switch (cmd) {
        case CLIP_SAYS: //TODO:red
            if (!d->mGag.contains(user)) {
                cMsg = i18n("%1 tells you: %2", user, cMsg.replace(QRegExp('^' + user), ""));
                emit personalMessage(cMsg);
            } else
                cMsg = "";
            break;

        case CLIP_SHOUTS: //TODO:black
            if ((!((KToggleAction *)d->mAct[KBgChatPrivate::Silent])->isChecked()) && (!d->mGag.contains(user))) {
                cMsg = i18n("%1 shouts: %2", user, cMsg.replace(QRegExp('^' + user), ""));
            } else
                cMsg = "";
            break;

        case CLIP_WHISPERS: //TODO:red
            if (!d->mGag.contains(user)) {
                cMsg = i18n("%1 whispers: %2", user, cMsg.replace(QRegExp('^' + user), ""));
                emit personalMessage(cMsg);
            } else
                cMsg = "";
            break;

        case CLIP_KIBITZES: //TODO:red
            if (!d->mGag.contains(user)) {
                cMsg = i18n("%1 kibitzes: %2", user, cMsg.replace(QRegExp('^' + user), ""));
                emit personalMessage(cMsg);
            } else
                cMsg = "";
            break;

        case CLIP_YOU_SAY: //TODO:darkgreen
            cMsg = i18n("%1 tell %2: %3",d->engine->getLoginName(), user, cMsg.replace(QRegExp('^' + user), ""));
            emit personalMessage(cMsg);
            user.clear();
            break;

        case CLIP_YOU_SHOUT://TODO:darkgreen
            cMsg = i18n("%1 shout: %2",d->engine->getLoginName(), cMsg);
            emit personalMessage(cMsg);
            user.clear();
            break;

        case CLIP_YOU_WHISPER://TODO:darkgreen
            cMsg = i18n("%1 whisper: %2",d->engine->getLoginName(), cMsg);
            emit personalMessage(cMsg);
            user.clear();
            break;

        case CLIP_YOU_KIBITZ://TODO:darkgreen
            cMsg = i18n("%1 kibitz: %2",d->engine->getLoginName(), cMsg);
            emit personalMessage(cMsg);
            user.clear();
            break;

        case CLIP_MESSAGE: //TODO:red
            user = cMsg.left(cMsg.indexOf(' ')+1);
            cMsg.remove(0, cMsg.indexOf(' ')+1);
            date.setTime_t(cMsg.left(cMsg.indexOf(' ')+1).toUInt());
            cMsg.remove(0, cMsg.indexOf(' '));
            cMsg = i18n("User %1 left a message at %2: %3", user, date.toString(), cMsg);
            emit personalMessage(cMsg);
            user.clear();
            break;

        case CLIP_MESSAGE_DELIVERED: //TODO:darkgreen
            cMsg = i18n("Your message for %1 has been delivered.", user);
            emit personalMessage(cMsg);
            user.clear();
            break;

        case CLIP_MESSAGE_SAVED: //TODO:darkgreen
            cMsg = i18n("Your message for %1 has been saved.", user);
            emit personalMessage(cMsg);
            user.clear();
            break;

        default: // ignore the message
            kDebug(10500) << "Message ignored, CLIP=" << cmd << ", message="<<cMsg;
            return;
        }

    } else {

        /*
         * Special treatment for non-CLIP messages
         */
        if (cMsg.contains(QRegExp("^You say to yourself: "))) {
            cMsg.replace(QRegExp("^You say to yourself: "),
                         i18n("You say to yourself: "));
        } else {
            kDebug(user.isNull(), 10500) << "KBgFibsChat::handleData unhandled message: "
                                                  << cMsg.toLatin1();
            return;
        }
    }

    if (!cMsg.isEmpty())
        addMessage(user, cMsg);
}


// == context menu and related slots ===========================================
/*
 * RMB opens a context menu.
 * TODO:check whether this context menu could be shared between player list's context menu
 */
void KBgFibsChat::customMenuHandler(const QPoint &pos)
{
    const QPersistentModelIndex index = indexAt(pos);
    if (!index.isValid())
        return;

    const KChatBaseMessage item = index.model()->data(index).value<KChatBaseMessage>();
    kDebug(11000) << "sender"<<item.first<<", message="<<item.second;

    d->mName[0] = item.first;
    d->mText = item.second;

    /*
     * Get a new context menu every time. Safe to delete the 0
     * pointer.
     */
    delete d->mChat; d->mChat = new QMenu();

    /*
     * Fill the context menu with actions
     */
    if (!d->mName[0].isNull()) {

        d->mAct[KBgChatPrivate::Talk]->setText(i18n("Talk to %1", d->mName[0]));
        d->mChat->addAction( d->mAct[KBgChatPrivate::Talk] );

        d->mAct[KBgChatPrivate::Inquire]->setText(i18n("Info on %1", d->mName[0]));
        d->mChat->addAction( d->mAct[KBgChatPrivate::Inquire] );

        //invite menu is always the same
        d->mInvt->setTitle(i18n("Invite %1", d->mName[0]));
        d->mChat->addMenu(d->mInvt);

        d->mChat->addSeparator();

        if (d->mGag.contains(d->mName[0]) <= 0) {
            d->mAct[KBgChatPrivate::Gag]->setText(i18n("Gag %1", d->mName[0]));
            d->mChat->addAction( d->mAct[KBgChatPrivate::Gag] );
        } else {
            d->mAct[KBgChatPrivate::Ungag]->setText(i18n("Ungag %1", d->mName[0]));
            d->mChat->addAction( d->mAct[KBgChatPrivate::Ungag] );
        }
    }
    if (d->mGag.count() > 0)
        d->mChat->addAction( d->mAct[KBgChatPrivate::Cleargag] );

    if ((d->mGag.count() > 0) || (!d->mName[0].isNull()))
        d->mChat->addSeparator();

    d->mChat->addAction( d->mAct[KBgChatPrivate::Silent] );

    d->mChat->addSeparator();

    d->mChat->addAction( d->mAct[KBgChatPrivate::Copy ] );
    d->mChat->addAction( d->mAct[KBgChatPrivate::Clear] );
    d->mChat->addAction( d->mAct[KBgChatPrivate::Close] );

    d->mChat->popup(mapToGlobal(QPoint(0,0)) + pos);
}

/*
 * Clear the gag list
 */
void KBgFibsChat::slotCleargag()
{
    d->mGag.clear();
    addMessage(QString(), i18n("The gag list is now empty.")); //blue
}

/*
 * Gag the selected user
 */
void KBgFibsChat::slotGag()
{
    d->mGag.append(d->mName[0]);
    addMessage(QString(), i18n("You will not hear what %1 says and shouts.", d->mName[0])); //TODO:blue
}

/*
 * Simple interface to the actual talk slot
 */
void KBgFibsChat::slotTalk()
{
    fibsTalk(d->mName[0]);
}

/*
 * Remove selected user from gag list
 */
void KBgFibsChat::slotUngag()
{
    d->mGag.removeAll(d->mName[0]);
    addMessage(QString(), i18n("You will again hear what %1 says and shouts.", d->mName[0])); //TODO:blue
}

/*
 * Get information on selected user
 */
void KBgFibsChat::slotInquire()
{
    kDebug(d->mName[0].isNull(), 10500) << "KBgFibsChat::slotInquire: user == null";
    emit fibsCommand("whois " + d->mName[0]);
}

/*
 * Block all shouts from the chat window
 */
void KBgFibsChat::slotSilent()
{
    QString msg;
    if (((KToggleAction *)d->mAct[KBgChatPrivate::Silent])->isChecked())
        msg = i18n("You will not hear what people shout."); //TODO:blue
    else
        msg = i18n("You will hear what people shout."); //TODO:blue
    addMessage(QString(), msg);
}

/*
 * Copy the selected line to the clipboard. Strip the additional HTML
 * from the text before copying.
 */
void KBgFibsChat::slotCopy()
{
    QApplication::clipboard()->setText(d->mText);
}

/*
 * Invite the selected player.
 */
void KBgFibsChat::slotInviteD()
{
    kDebug(d->mName[0].isNull(), 10500) << "KBgFibsChat::slotInvite: user == null";
    emit fibsRequestInvitation(d->mName[0]);
}
void KBgFibsChat::slotInvite1() { emit fibsCommand("invite " + d->mName[0] + " 1"); }
void KBgFibsChat::slotInvite2() { emit fibsCommand("invite " + d->mName[0] + " 3"); }
void KBgFibsChat::slotInvite3() { emit fibsCommand("invite " + d->mName[0] + " 5"); }
void KBgFibsChat::slotInvite4() { emit fibsCommand("invite " + d->mName[0] + " 7"); }
void KBgFibsChat::slotInvite5() { emit fibsCommand("invite " + d->mName[0] + " 9"); }

void KBgFibsChat::slotInviteU() { emit fibsCommand("invite " + d->mName[0] + " unlimited"); }
void KBgFibsChat::slotInviteR() { emit fibsCommand("invite " + d->mName[0]); }


#include "kbgfibschat.moc"
// EOF
