/* Yo Emacs, this -*- C++ -*-

  Copyright (C) 1999-2001 Jens Hoefkens <jens@hoefkens.com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

  $Id: kbg.cpp 898669 2008-12-18 17:19:59Z asmirnov $

*/

#include "kbg.h"
#include "kbg.moc"

#include <QPainter>
#include <QLayout>
#include <QGroupBox>
#include <QRadioButton>
#include <QPixmap>
#include <QPushButton>

#include <QStringList>
#include <QIcon>
#include <QHideEvent>
#include <QLabel>
#include <QShowEvent>
#include <QGridLayout>
#include <QMenu>
#include <QSplitter>
#include <QApplication>

#include <kmenubar.h>
#include <ktoolbar.h>
#include <kedittoolbar.h>
#include <klocale.h>
#include <kaction.h>
#include <kactioncollection.h>
#include <kstandardaction.h>
#include <kstandardgameaction.h>
#include <kaboutdata.h>
#include <kmessagebox.h>
#include <kconfig.h>
#include <kcompletion.h>
#include <kcompletionbox.h>
#include <kmenu.h>
#include <kurllabel.h>
#include <krun.h>
#include <kstatusbar.h>
#include <klineedit.h>
#include <knuminput.h>
#include <ktoolinvocation.h>
#include <kxmlguifactory.h>
#include <kvbox.h>
#include <kglobal.h>
#include <kactionmenu.h>
#include <kpagedialog.h>
#include <kicon.h>
#include <kselectaction.h>
#include <knotifyconfigwidget.h>
#include <kdeprintdialog.h>

#include "kbgtextview.h"
#include "offline/kbgoffline.h"
#include "fibs/kbgfibs.h"
#include "gnubg/kbggnubg.h"
#include "nextgen/kbgng.h"
#include "version.h"

class KBg::KBgPrivate
{
public:
    QAction *newAction, *undoAction, *redoAction, *rollAction, *cubeAction, *endAction;

    QString engineString[KBg::MaxEngine];
    KBgEngine *engine[KBg::MaxEngine]; //TODO: do we really need array of all possible engines?
    KBg::Engine currEngine;

    QString helpTopic[KBg::MaxHelpTopic][2];

    /**
     * Notebook for the setup
     */
    KPageDialog *nb;
    KDoubleNumInput *sbt;
    QCheckBox *cbt, *cbs, *cbm;
    QRadioButton *rbEngine[KBg::MaxEngine];

    /*
    * UI elements
    */
    QSplitter     *panner;
    KBgBoardSetup *board;
    KBgTextView   *status;
    KLineEdit     *cmdLine;
    QLabel        *cmdLabel;
    QString       baseCaption; // for user friendly printing, we keep it around
    QLabel        *statusLabel;

    /**
     * Get current engine implementation
     * 
     * @return current engine, used for game
     */
    KBgEngine* currentEngine() const
    {
        return engine[currEngine];
    }
};

KBg::Engine& operator++(KBg::Engine& e)
{
    return e=(KBg::MaxEngine==e)?KBg::MaxEngine:KBg::Engine(e+1);
}


// == setup ====================================================================

/*
 * Constructor creates user interface, actions and first engine.
 */
KBg::KBg()
    : KXmlGuiWindow(), d(new KBgPrivate)
{
    /*
     * Initialize menu strings
     */
    d->engineString[Offline] = i18n("KBackgammon");
    d->engineString[FIBS   ] = i18n("FIBS");
    d->engineString[GNUbg  ] = i18n("GNU Backgammon (Experimental)");


    d->helpTopic[FIBSHome][0] = i18n("FIBS Home");
    d->helpTopic[FIBSHome][1] = "http://www.fibs.com/";

    d->helpTopic[RuleHome][0] = i18n("Backgammon Rules");
    d->helpTopic[RuleHome][1] = "http://www.bkgm.com/rules.html"; //TODO: write a handbook

    /*
     * The main view is shared between the board and a small text window
     */
    d->panner = new QSplitter(Qt::Vertical, this);
    d->panner->setObjectName("panner");
    d->board  = new KBgBoardSetup(d->panner);
    d->board->setObjectName("board");
    d->status = new KBgTextView(d->panner);
    d->status->setObjectName("status");

    d->statusLabel= new QLabel(i18n("KBackgammon"), statusBar());
    statusBar()->addWidget(d->statusLabel, 1);
    setCentralWidget(d->panner);

    /*
     * Create all actions needed by the application
     */

    // Game
    //create "new game" button on toolbar and "new game" menu item
    //show a dialog with options: online, offline, server, AI level
    //d->newAction = KStandardGameAction::gameNew(this, SLOT(newGame()), actionCollection());
    {
	KActionMenu * const newOfflineGameMenu = new KActionMenu(i18n("&Play offline"), this);
	actionCollection()->addAction("game_new_offline", newOfflineGameMenu);
	
	QAction* const offlineFriendItem = actionCollection()->addAction("new_offline_game_humans");
	offlineFriendItem->setText(i18n("With a friend"));
	offlineFriendItem->setEnabled (false); //FIXME asap
	connect(offlineFriendItem, SIGNAL(triggered(bool) ), SLOT(newAdvancedOfflineGame())); //FIXME: human vs human
	newOfflineGameMenu->menu()->addAction(offlineFriendItem);
	
	QAction* const offlineAdvancedItem = actionCollection()->addAction("new_offline_game_advanced");
	offlineAdvancedItem->setText(i18n("Advanced AI"));
	connect(offlineAdvancedItem, SIGNAL(triggered(bool) ), SLOT(newAdvancedOfflineGame()));
	newOfflineGameMenu->menu()->addAction(offlineAdvancedItem);
	
	QAction* const offlineExpertItem = actionCollection()->addAction("new_offline_game_expert");
	offlineExpertItem->setText(i18n("Expert AI"));
	connect(offlineExpertItem, SIGNAL(triggered(bool) ), SLOT(newExpertOfflineGame()));
	newOfflineGameMenu->menu()->addAction(offlineExpertItem);
    }
    {
	KActionMenu * const newOnlineGameMenu = new KActionMenu(i18n("&Play online"), this);
	actionCollection()->addAction("game_new_online", newOnlineGameMenu);
	
	QAction* const onlineAdvancedItem = actionCollection()->addAction("new_online_game_FIBS");
	onlineAdvancedItem->setText(i18n("&FIBS"));
	connect(onlineAdvancedItem, SIGNAL(triggered(bool) ), SLOT(newFIBSGame()));
	newOnlineGameMenu->menu()->addAction(onlineAdvancedItem);

// 	QAction* const onlineExpertItem = actionCollection()->addAction("new_online_game_peer");
// 	onlineExpertItem->setText(i18n("&Network..."));
// 	connect(onlineExpertItem, SIGNAL(triggered(bool) ), SLOT(newNGGame()));
// 	newOnlineGameMenu->menu()->addAction(onlineExpertItem);
    }


    //KStandardGameAction::end(this, SLOT(close()), actionCollection());
    KStandardGameAction::quit(this, SLOT(close()), actionCollection());

    // Move
    d->undoAction = KStandardGameAction::undo(this, SLOT(undo()), actionCollection());
    d->undoAction->setEnabled(false);
    d->redoAction = KStandardGameAction::redo(this, SLOT(redo()), actionCollection());
    d->redoAction->setEnabled(false);
    d->rollAction = KStandardGameAction::roll(this, SLOT(roll()), actionCollection());
    d->rollAction->setEnabled(false);
    d->endAction = KStandardGameAction::endTurn(this, SLOT(done()), actionCollection());
    d->endAction->setEnabled(false);

    d->cubeAction = actionCollection()->addAction("move_cube");
    d->cubeAction->setIcon( KIcon( PROG_NAME "-double.xpm" ) );
    d->cubeAction->setText( i18n("Double Cube") );
    connect( d->cubeAction, SIGNAL(triggered(bool)), SLOT( cube() ) );
    d->cubeAction->setEnabled(false);

    //Settings menu
    KStandardAction::showMenubar(this, SLOT(toggleMenubar()), actionCollection());
    KStandardAction::preferences(this, SLOT(setupDlg()), actionCollection());
    KStandardAction::saveOptions(this, SLOT(saveConfig()), actionCollection());
    KStandardAction::configureNotifications( this, SLOT ( configureEvents() ), actionCollection() );

    //Help menu
    KActionMenu *actMenu = new KActionMenu(i18n("&Backgammon on the Web"), this);
    actionCollection()->addAction("help_www", actMenu);

    QAction* action = actionCollection()->addAction("help_www_fibs");
    action->setText(d->helpTopic[FIBSHome][0]);
    connect(action, SIGNAL(triggered(bool) ), SLOT(wwwFIBS()));
    actMenu->menu()->addAction(action);
    action = actionCollection()->addAction("help_www_rule");
    action->setText(d->helpTopic[RuleHome][0]);
    connect(action, SIGNAL(triggered(bool) ), SLOT(wwwRule()));
    actMenu->menu()->addAction(action);

    /*
     * Set up the command line - using actions, otherwise recreating the GUI will delete them
     * (e.g. using KEditToolBar)
     */
    d->cmdLabel = new QLabel(i18n("Command: "), this);
    action = actionCollection()->addAction("command_label");
    action->setText(d->cmdLabel->text());
    qobject_cast<KAction*>( action )->setDefaultWidget( d->cmdLabel );

    d->cmdLine  = new KLineEdit( this );
    d->cmdLine->setObjectName( "commandline" );
    action = actionCollection()->addAction("command_lineedit");
    action->setText(QString());
    qobject_cast<KAction*>( action )->setDefaultWidget( d->cmdLine );
    d->cmdLine->completionObject()->setOrder(KCompletion::Weighted);
    connect(d->cmdLine, SIGNAL(returnPressed(const QString &)), this, SLOT(handleCmd(const QString &)));
    /*
     * Done with the actions, create the XML-defined parts of the
     * user interface
     */
    statusBar();
    setupGUI();

//    d->cmdLine->setFocus();

    /*
     * Initialize the engine to the default (offline). If the user
     * prefers a different engine, it will be started later
     */
    for (int i = 0; i < MaxEngine; i++)
        d->engine[i] = 0;
    d->currEngine = None;
    setupEngine(Offline);

    /*
     * Set up configuration handling.
     * FIXME: support session management
     */
    connect(this, SIGNAL(readSettings()), d->board, SLOT(readConfig()));
    connect(this, SIGNAL(saveSettings()), d->board, SLOT(saveConfig()));

    /*
     * Set up some whatis messages for the online help
     */
    d->status->setWhatsThis( i18n("This area contains the status messages for the game. "
                 "Most of these messages are sent to you from the current "
                 "engine."));
    toolBar("cmdToolBar")->setWhatsThis(
            i18n("This is the command line. You can type special "
                 "commands related to the current engine in here. "
                 "Most relevant commands are also available "
                 "through the menus."));
    toolBar("mainToolBar")->setWhatsThis(
            i18n("This is the button bar tool bar. It gives "
                 "you easy access to game related commands. "
                 "You can drag the bar to a different location "
                 "within the window."));
    statusBar()->setWhatsThis(
            i18n("This is the status bar. It shows you the currently "
                 "selected engine in the left corner."));

    /*
     * Create and initialize the context menu
     */
    QMenu* menu = (QMenu*)factory()->container("popup", this);
    d->board->setContextMenu(menu);
}

/*
 * Destructor
 */
KBg::~KBg() {
    delete d;
}


// == engine handling ==========================================================

/**
 * Set the engine according to newEngine.
 * Additional engines have to be added to the switchstatement (and only there).
 */
void KBg::setupEngine(const Engine newEngine)
{
    /*
     * Engine doesn't need to be changed?
     */
    if (newEngine == d->currEngine)
        return;

    /*
     * Check with the current engine if it can be terminated
     */
    if (d->currEngine != None && d->currentEngine() && !d->currentEngine()->queryClose()) {
        return;
    }

    /*
     * Remove the old engine, create a new one, and hook up menu and slots/signals
     */
    QMenu *commandMenu = (QMenu *)factory()->container("command_menu", this);
    commandMenu->clear();

    if (d->currEngine != None) {
        delete d->currentEngine();
        d->engine[d->currEngine] = 0;
    }

    d->currEngine = newEngine;
    switch (newEngine) {
        case Offline:
            kDebug(10500)<<"Setting engine type: Offline";
	    d->engine[newEngine] = new KBgEngineOffline(this, commandMenu);
            break;
        case FIBS:
            kDebug(10500)<<"Setting engine type: FIBS";
	    d->engine[newEngine] = new KBgEngineFIBS(this, commandMenu);
            break;
        case GNUbg:
            kDebug(10500)<<"Setting engine type: GNUbg";
	    d->engine[newEngine] = new KBgEngineGNU(this, commandMenu);
            break;
//    case NextGen:
//   kDebug()<<"Setting engine type: NextGen";
//        d->engine[d->currEngine] = new KBgEngineNg(this, commandMenu);
//        break;
            default: // FIXME: we need some kind of catch here...
                kDebug(10500)<<"Setting engine type: default";
                d->currEngine = Offline;
		d->engine[newEngine] = new KBgEngineOffline(this, commandMenu);
                break;
    }

    d->statusLabel->setText(QString(d->engineString[newEngine]));

    KConfigGroup config( KGlobal::config(), "global settings");
    if (config.readEntry("enable timeout", true))
        d->currentEngine()->setCommit(config.readEntry("timeout", 2.5));
    //d->newAction->setEnabled(d->currentEngine()->haveNewGame()); //FIXME - disable creation of new game if other game is in progress

    // engine -> this
    connect(d->currentEngine(), SIGNAL(statText(const QString &)), this, SLOT(updateCaption(const QString &)));
    connect(d->currentEngine(), SIGNAL(infoText(const QString &)), d->status, SLOT(write(const QString &)));
    connect(d->currentEngine(), SIGNAL(allowCommand(int, bool)),   this, SLOT(allowCommand(int, bool)));

    // this -> engine
    connect(this, SIGNAL(readSettings()), d->currentEngine(), SLOT(readConfig()));
    connect(this, SIGNAL(saveSettings()), d->currentEngine(), SLOT(saveConfig()));

    // board -> engine
    connect(d->board, SIGNAL(rollDice(const int)),    d->currentEngine(), SLOT(rollDice(const int)));
    connect(d->board, SIGNAL(doubleCube(const int)),  d->currentEngine(), SLOT(doubleCube(const int)));
    connect(d->board, SIGNAL(currentMove(QString *)), d->currentEngine(), SLOT(handleMove(QString *)));

    // engine -> board
    connect(d->currentEngine(), SIGNAL(undoMove()), d->board, SLOT(undoMove()));
    connect(d->currentEngine(), SIGNAL(redoMove()), d->board, SLOT(redoMove()));
    connect(d->currentEngine(), SIGNAL(setEditMode(const bool)), d->board, SLOT(setEditMode(const bool)));
    connect(d->currentEngine(), SIGNAL(allowMoving(const bool)), d->board, SLOT(allowMoving(const bool)));
    connect(d->currentEngine(), SIGNAL(getState(KBgStatus *)), d->board, SLOT(getState(KBgStatus *)));
    connect(d->currentEngine(), SIGNAL(newState(const KBgStatus &)), d->board, SLOT(setState(const KBgStatus &)));

    // now that all signals are connected, start the engine
    d->currentEngine()->start();
}


// == configuration handing ====================================================

/*
 * Save all settings that should be saved for the next start.
 */
void KBg::saveConfig()
{
    KConfigGroup config( KGlobal::config(), "global settings");

    /*
     * Save the main window options unless the user has asked not
     * to do so.
     */
    if (config.readEntry("autosave on exit", true)) {

        KConfigGroup config ( KGlobal::config(), "main window");

        config.writeEntry("origin", pos());

        config.writeEntry("font",   d->status->font());
        config.writeEntry("panner", (double)d->board->height()/(double)d->panner->height());

        saveMainWindowSettings(config);
    }

    /*
     * Save the history
     */
    config=KConfigGroup(KGlobal::config(), "command line");
    config.writeEntry("history", d->cmdLine->completionObject()->items());

    /*
     * Save current engine
     */
    config=KConfigGroup(KGlobal::config(), "engine settings");
    config.writeEntry("last engine", (int)d->currEngine);

    /*
     * Tell other objects to save their settings, too.
     */
    emit saveSettings();

    config.sync();
}

/*
 * Read the stored configuration and apply it
 */
void KBg::readConfig()
{
    KConfigGroup config( KGlobal::config(), "global settings");

    /*
     * Restore the main window settings unless the user has asked
     * not to do so.
     */
    if (config.readEntry("autosave on exit", true)) {

        config=KConfigGroup(KGlobal::config(),"main window");

        QPoint pos, defpos(10, 10);
        QFont kappFont = QApplication::font();

        pos = config.readEntry("origin", defpos);

        d->status->setFont(config.readEntry("font", kappFont));

        QList<int> l;
        l.append(qRound(   config.readEntry("panner", 0.75) *d->panner->height()));
        l.append(qRound((1-config.readEntry("panner", 0.75))*d->panner->height()));
        d->panner->setSizes(l);

        applyMainWindowSettings( config );
    }

    /*
     * Restore the history
     */
    config=KConfigGroup(KGlobal::config(),"command line");
    d->cmdLine->completionObject()->setItems(config.readEntry("history",QStringList()));

    /*
     * Tell other objects to read their configurations
     */
    emit readSettings();

    /*
     * Restore last engine
     */
    config=KConfigGroup(KGlobal::config(),"engine settings");
    setupEngine((Engine)config.readEntry("last engine",int(Offline)));
}


// == configuration ============================================================

/*
 * Connected to the setup dialog applyClicked signal. Make sure
 * that all changes are saved.
 */
void KBg::setupOk()
{
    // global settings
    KConfigGroup config( KGlobal::config(), "global settings");

    config.writeEntry("enable timeout",   d->cbt->isChecked());
    config.writeEntry("timeout",          d->sbt->value());
    config.writeEntry("autosave on exit", d->cbs->isChecked());

    //TODO: save on exit
//     config=KConfigGroup(KGlobal::config(),"engine");
//     config.writeEntry("last engine",      (int)gameType);

    // tell engine about commit timer
    d->currentEngine()->setCommit(d->cbt->isChecked() ? d->sbt->value() : -1);

    // one time requests
    if (d->cbm->isChecked())
        KMessageBox::enableAllMessages();

    // tell children to read their changes
    d->board->setupOk();

    // engines
    for (int i = 0; i < MaxEngine; i++)
        d->engine[i]->setupOk();
    // will reset engine if it has changed
//     setupEngine(gameType);

    // save it all
    saveConfig();
}

/*
 * Load default values for the user settings
 */
void KBg::setupDefault()
{
    // timeout
    d->cbt->setChecked(true);
    d->sbt->setValue(2.5);

    // messages
    d->cbm->setChecked(false);

    // auto save
    d->cbs->setChecked(true);

    // board
    d->board->setupDefault();
}

/*
 * Connected to the setup dialog cancelButtonPressed signal. There
 * isn't much to do. We tell the board to undo the changes.
 */
void KBg::setupCancel()
{
    // board
    d->board->setupCancel();

    // engines
    for (int i = 0; i < MaxEngine; i++)
        d->engine[i]->setupCancel();
}

/*
 * Setup dialog is ready to be deleted. Do it later...
 */
void KBg::setupDone()
{
    d->nb->delayedDestruct();
    for (int i = 0; i < MaxEngine; i++)
        if (i != d->currEngine) d->engine[i] = 0;
}

void KBg::configureEvents()
{
    KNotifyConfigWidget::configure ( this );
}

/*
 * Initialize and display the setup dialog
 */
void KBg::setupDlg()
{
    /*
     * Get a new notebook in which all other members can put their
     * config pages
     */
    d->nb = new KPageDialog(this);
    d->nb->setWindowTitle(i18n("KBackgammon settings"));
    d->nb->setButtons(KDialog::Ok|KDialog::Cancel|KDialog::Default|KDialog::Apply|KDialog::Help);
    d->nb->setDefaultButton(KDialog::Ok);
    d->nb->setFaceType(KPageDialog::List);
    d->nb->setModal(true);
    d->nb->showButtonSeparator(true);
    KConfigGroup config( KGlobal::config(), "global settings");

    /*
     * Main Widget
     */
    KVBox *w = new KVBox();
    KPageWidgetItem *pageItem = new KPageWidgetItem( w, i18n("General settings"));
    pageItem->setHeader(i18n("General"));
    pageItem->setIcon( KIcon( "preferences-other" ));
    d->nb->addPage(pageItem);

    /*
     * Group boxes
     */
    QGroupBox * const gbm = new QGroupBox(i18n("Messages"), w);
    QGroupBox * const gbt = new QGroupBox(i18n("Timer"), w);
    QGroupBox * const gbs = new QGroupBox(i18n("Autosave"), w);

    /*
     * Timer box
     */
    gbt->setWhatsThis( i18n("After you finished your moves, they have to be sent to the engine. "
                  "You can either do that manually (in which case you should not enable "
                  "this feature), or you can specify an amount of time that has to pass "
                  "before the move is committed. If you undo a move during the timeout, the "
                  "timeout will be reset and restarted once you finish the move. This is "
                  "very useful if you would like to review the result of your move."));

    d->cbt = new QCheckBox(i18n("Enable timeout"), gbt);
    d->cbt->setChecked(config.readEntry("enable timeout", true));

    d->sbt = new KDoubleNumInput(gbt);
    d->sbt->setRange(0.0, 60.0, 0.5);
    d->sbt->setLabel(i18n("Move timeout in seconds:"));
    d->sbt->setValue(config.readEntry("timeout", 2.5));

    connect(d->cbt, SIGNAL(toggled(bool)), d->sbt, SLOT(setEnabled(bool)));
    d->sbt->setEnabled(d->cbt->isChecked());

    QGridLayout *gl = new QGridLayout(gbt);
    gl->setMargin(10);
    gl->addWidget(d->cbt, 0, 0);
    gl->addWidget(d->sbt, 1, 0);

    /*
     * Enable messages //TODO: remove it
     */
    gbm->setWhatsThis( i18n("Check the box to enable all the messages that you have previously "
                  "disabled by choosing the \"Do not show this message again\" option."));

    QGridLayout *glm = new QGridLayout(gbm);
    glm->setMargin(d->nb->spacingHint());
    d->cbm = new QCheckBox(i18n("Reenable all messages"), gbm);
    glm->addWidget(d->cbm, 0, 0);

    /*
     * Save options on exit ? //TODO: shouldn't his be done automagically? get rid
     */
    gbm->setWhatsThis( i18n("Check the box to automatically save all window positions on program "
                  "exit. They will be restored at next start."));

    QGridLayout *gls = new QGridLayout(gbs);
    gls->setMargin(d->nb->spacingHint());
    d->cbs = new QCheckBox(i18n("Save settings on exit"), gbs);
    d->cbs->setChecked(config.readEntry("autosave on exit", true));
    gls->addWidget(d->cbs, 0, 0);

    /*
     * Board settings
     */
    d->board->getSetupPages(d->nb);

    /*
     * Hack alert: this little trick makes sure that ALL engines
     * have their settings available in the dialog.
     */
    QMenu *dummyPopup = new QMenu(d->nb);
#ifdef __GNUC__
#warning possibly the parent is wrong here. It was d->nb before but we need a XMLGui in there
#endif
    //FIXME: 
    //0. who should delete array elements before this cycle??
    //1. instead of creating engine with 'new' isn't it better to call appropriate static member?
    //2. for every engine create a mixin class/struct where settings should be stored
    //3. Weird cycle. Try to get rid of enumeration all known engines
    for (int i = 0; i < MaxEngine; i++) {
        if (d->currEngine != i) {
            switch (i) {
            case Offline:
                d->engine[i] = new KBgEngineOffline(this, dummyPopup);
                break;
            case FIBS:
                d->engine[i] = new KBgEngineFIBS(this, dummyPopup);
                break;
            case GNUbg:
                d->engine[i] = new KBgEngineGNU(this, dummyPopup);
                break;
//            case NextGen:
//                d->engine[i] = new KBgEngineNg( this, dummyPopup);
//                break;
            }
            connect(this, SIGNAL(saveSettings()), d->engine[i], SLOT(saveConfig()));
        }
        d->engine[i]->getSetupPages(d->nb);
    }

    /*
     * Connect the signals of nb
     */
    connect(d->nb, SIGNAL(okClicked()),     this, SLOT(setupOk()));
    connect(d->nb, SIGNAL(applyClicked()),  this, SLOT(setupOk()));
    connect(d->nb, SIGNAL(cancelClicked()), this, SLOT(setupCancel()));
    connect(d->nb, SIGNAL(defaultClicked()),this, SLOT(setupDefault()));

    connect(d->nb, SIGNAL(finished()), this, SLOT(setupDone()));

    d->nb->resize(d->nb->minimumSize());
    d->nb->show();
}


// == action slots =============================================================

/*
 * Toggle visibility of the menubar - be careful that the menu doesn't
 * get lost
 */
void KBg::toggleMenubar()
{
    if (menuBar()->isVisible()) {

        KMessageBox::information(this, i18n("You can enable the menubar again with the "
                            "right mouse button menu of the board."),
                     i18n("Information"), "conf_menubar_information");
        menuBar()->hide();

    } else {

        menuBar()->show();
    }
}

/*
 * Display a standard dialog for the toolbar content
 */
void KBg::configureToolbars()
{
    KConfigGroup cg( KGlobal::config(), "kedittoolbar settings"); // temp group
    saveMainWindowSettings( cg );
    KEditToolBar dlg(actionCollection(), this);
    dlg.setResourceFile(xmlFile());
    connect(&dlg,SIGNAL(newToolBarConfig()),this,SLOT(newToolbarConfig()));
    dlg.exec();
    cg.deleteGroup(); // delete temp group
}

/*
 * Called when clicking OK or Apply in the toolbar editor
 */
void KBg::newToolbarConfig()
{
    createGUI();
    applyMainWindowSettings(KGlobal::config()->group( "kedittoolbar settings") );
}

/*
 * Help slots
 */
void KBg::wwwFIBS() {showWWW(FIBSHome);}
void KBg::wwwRule() {showWWW(RuleHome);}

void KBg::showWWW(int t)
{
    KToolInvocation::invokeBrowser(d->helpTopic[t][1]);
}

/*
 * Edit slots
 */
void KBg::undo() {d->currentEngine()->undo();}
void KBg::redo() {d->currentEngine()->redo();}
void KBg::roll() {d->currentEngine()->roll();}
void KBg::cube() {d->currentEngine()->cube();}
void KBg::done() {d->currentEngine()->done();}
void KBg::load() {d->currentEngine()->load();}

/**
 * Start a new game with current engine (if possible) //TODO: check possibility of force finishing
 */
void KBg::newGame()
{
    d->currentEngine()->newGame();
}

/**
 * Start a new game with Daren's offline engine
 */
void KBg::newAdvancedOfflineGame()
{
    setupEngine(Offline);
    d->currentEngine()->newGame();
}

/**
 * Start a new game with gnubg offline engine
 */
void KBg::newExpertOfflineGame()
{
    setupEngine(GNUbg);
    d->currentEngine()->newGame();
}

/**
 * Start new FIBS game
 */
void KBg::newFIBSGame()
{
   setupEngine(FIBS);
   d->currentEngine()->newGame();
}


// == various slots - not for actions ==========================================

/*
 * Check with the engine if it is okay to close the window.
 * If so, save settings.
 */
bool KBg::queryClose()
{
    bool ret = d->currentEngine()->queryClose();
    if ( ret )
        saveConfig();
    return ret;
}

/*
 * Set the caption of the main window. If the user has requested pip
 * counts, they are appended, too.
 */
void KBg::updateCaption(const QString &s)
{
    d->baseCaption = s;
    QString msg;
    if (!s.isEmpty()) {
        msg = s;
        if (d->board->getPipCount(US) >= 0) {
            QString tmp;
            tmp.setNum(d->board->getPipCount(US  ));
            msg += " - " + tmp;
            tmp.setNum(d->board->getPipCount(THEM));
            msg += '-'  + tmp;
        }
    }
    setCaption(msg, false);
}

/*
 * Take the string from the commandline, give it to the engine, append
 * to the history and clear the buffer.
 */
void KBg::handleCmd(const QString &s)
{
    if (!s.trimmed().isEmpty()) {
        d->currentEngine()->handleCommand(s);
        d->cmdLine->completionObject()->addItem(s);
    }
    d->cmdLine->clear();
    d->cmdLine->completionBox()->close();
}

/*
 * Reflect the availability of commands in the button bar.
 */
void KBg::allowCommand(int cmd, bool f)
{
    switch (cmd) {
    case KBgEngine::Undo:
        d->undoAction->setEnabled(f);
        break;
    case KBgEngine::Redo:
        d->redoAction->setEnabled(f);
        break;
    case KBgEngine::Roll:
        d->rollAction->setEnabled(f);
        break;
    case KBgEngine::Cube:
        d->cubeAction->setEnabled(f);
        break;
    case KBgEngine::Done:
        d->endAction->setEnabled(f);
        break;
    //case KBgEngine::Load:
    //    loadAction->setEnabled(f);
    //    break;
    }
}

/*
 * Catch the hide envents. That way, the current engine can close its
 * child windows.
 */
void KBg::hideEvent(QHideEvent *e)
{
    KXmlGuiWindow::hideEvent(e);
    d->currentEngine()->hideEvent();
}

/*
 * Catch the show envents. That way, the current engine can open any
 * previously hidden windows.
 */
void KBg::showEvent(QShowEvent *e)
{
    KXmlGuiWindow::showEvent(e);
    d->currentEngine()->showEvent();
}

// EOF

