/* Yo Emacs, this -*- C++ -*-

  Copyright (C) 1999-2001 Jens Hoefkens
  hoefkens@pilot.msu.edu

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

  $Id: kbg.h 898669 2008-12-18 17:19:59Z asmirnov $

*/

#ifndef __KBG_H
#define __KBG_H

#include <kxmlguiwindow.h>

class QSplitter;
class QCheckBox;
class QMenu;
class QLabel;
class QAction;
class QRadioButton;
class KLineEdit;
class KPageDialog;
class KDoubleNumInput;

class KBgEngine;
class KBgTextView;
class KBgBoardSetup;

class KBg : public KXmlGuiWindow
{
    Q_OBJECT

public:
    /**
     * Each engine has its own identifier.
     *  - Offline engine is own builtin engine
     *  - FIBS is engine to play over internet
     *  - GNUbg is advanced offline engine, playing at worldclass-level. Requries gnubg package to be installed properly
     *  TODO: merge Offline and GBUbg. There should be two options: offline and online. 
     *  TODO: Offline in it's turn should be divided into several difficulty levels: easy(to be developed), intermediate(Daren's), advanced(GNUbg).
     */

        enum Engine {None = -1, Offline, FIBS, GNUbg, MaxEngine};

        enum HelpTpoics {FIBSHome, RuleHome, MaxHelpTopic};

    /**
     * Constructor creates the full main window
     */
    KBg();

    /**
     * Destructor
     */
    virtual ~KBg();

    /**
     * Read various settings from the configuration files or
     * set some reasonable defaults
     */
    void readConfig();

public slots:

   /**
     * Set the caption to KFIBS_NAME + string + pipcount (if requested by
     * the user)
     */
    void updateCaption(const QString &s);

    /**
     * Slot to be called by the engine - it enables/disables buttons
     * in the button bar
     */
    void allowCommand(int cmd, bool f);

    /**
     * Sets the backgammon engine to type
     */
    void setupEngine(const Engine newEngine);

    void configureEvents();

signals:

    /**
     * Tell all listeners to write their settings to disk
     */
    void saveSettings();

    /**
     * Tell all listeners to restore their settings or use reasonable
     * defaults
     */
    void readSettings();

protected:

    /*
     * Windows are to be hidden
     */
    virtual void hideEvent(QHideEvent *);

    /*
     * Redisplay the windows
     */
    virtual void showEvent(QShowEvent *);

    /*
     * Called before the window is closed. Check with the engine
     * if that is okay.
     */
    virtual bool queryClose();

protected slots:

    /**
     * Show the button bar - or not - depending on the corresponding action
     */
    void toggleMenubar();

    void configureToolbars();
    void newToolbarConfig();

    void newAdvancedOfflineGame();
    void newGame();
    void newExpertOfflineGame();
    void newFIBSGame();

    /**
     * Takes text from the commandline and hands it over to the
     * current engine
     */
    void handleCmd(const QString &);

    /**
     * Saves the user settings to disk
     */
    void saveConfig();

    /**
     * Slots for the respective actions - called by the button bar
     * and some global key shortcuts
     */
    void undo();
    void redo();
    void roll();
    void cube();
    void load();
    void done();

    /**
     * Opens and displays the respective home pages
     */
    void showWWW(int t);

    void wwwFIBS();
    void wwwRule();

    /**
     * Show the big setup dialog
     */
    void setupDlg();

    /**
     * Save the settings
     */
    void setupOk();

    /**
     * Delete the setup dialog
     */
    void setupDone();

    /**
     * Load default values for user settings
     */
    void setupDefault();

    /**
     * Undo the settings
     */
    void setupCancel();


private:
    class KBgPrivate;
    KBgPrivate * const d;
};

#endif // __KBG_H
