/* 
 * Copyright (C) 2005 <mikelima@cirulla.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation version 2.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */


#include <stdlib.h>
#include <string>

#include <klocale.h>
#include <kgenericfactory.h>
#include <kdebug.h>

#include <qcstring.h>
#include <qfile.h>

#include "kfile_glulx.h"

typedef KGenericFactory<KGlulxPlugin> GlulxFactory;

K_EXPORT_COMPONENT_FACTORY(kfile_glulx, GlulxFactory("kfile_glulx"))

KGlulxPlugin::KGlulxPlugin(
	QObject *parent, const char *name, const QStringList &args) : 
KFilePlugin(parent, name, args)
{
    // set up our mime type
    KFileMimeTypeInfo *info = addMimeTypeInfo("application/x-glulx");

    KFileMimeTypeInfo::GroupInfo *group;

    // info group
    group = addGroupInfo(info, "Info", i18n("Game Information"));
    addItemInfo(group, "InterpreterVersion", i18n("Interpreter Version"), QVariant::String);
    addItemInfo(group, "Checksum", i18n("Checksum"), QVariant::Int);
    addItemInfo(group, "CompiledWith", i18n("Compiled With"), QVariant::String);
}

bool 
KGlulxPlugin::readInfo(KFileMetaInfo& info, uint /*what*/)
{
    if (info.path().isEmpty()) return false;

    QFile f(info.path());
    if (!f.open(IO_ReadOnly)) return false;

    QDataStream stream(&f);

    // Read the Glulx file header. 
    // 
    QCString buf(5);
    stream.readRawBytes(buf.data(), 4);
    if (buf != "Glul") return false;
    uint val[8];
    stream >> val[0] >> val[1] >> val[2] >> val[3] >> val[4] >> val[5] >> val[6] >> val[7];
    QString versionString = QString("%1.%2.%3")
        .arg((val[0] >> 16) & 0xffff)
        .arg((val[0] >> 8) & 0xff)
        .arg(val[0] & 0xff);
    

    // If all is well, set the metadata
    KFileMetaInfoGroup group = appendGroup(info, "Info");
    appendItem(group, "InterpreterVersion", versionString);
    appendItem(group, "Checksum", val[7]);
    stream.readRawBytes(buf.data(), 4);
    QString compiledWith;
    if (buf == "Info") compiledWith = "Inform";
    // TODO There seem to be data on Inform version and compilation date in the
    // sample file.  Check if it is reliable, and show that too.
    
    if (!compiledWith.isEmpty()) {
        appendItem(group, "CompiledWith", compiledWith);
    }
    
    return true;
}

#include "kfile_glulx.moc"

// vi:sw=4 et:
