/***************************************************************************
 *   Copyright 2009 Zeng Huan <zh.issac@gmail.com>
 *   Copyright 2009 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef KOLF_BLENDER_TEXTURE_H
#define KOLF_BLENDER_TEXTURE_H

#include <QImage>

namespace Kolf
{
	
	/**
	 * \class BlenderTexture
	 *
	 * This class represents a texture(specific for texture blender), texture is in RGB mode, 8 bits per channel.
	 */

	enum BlenderCurve {
		LinearCurve,
		LogarithmicCurve,
		PotentialCurve,
		PolylineCurve,
		DefaultCurve = LinearCurve
	};

	class BlenderTexture : private QImage
	{
		public:
			explicit BlenderTexture(const QImage& image);

			QColor pixel(int x, int y, int z, Kolf::BlenderCurve curveShape = DefaultCurve) const;

			void setRange(int min, int opt, int max);
			int* range();
		private:
			qreal curve(int delta, int range, Kolf::BlenderCurve curveShape) const;
			int m_range[3];
	};
}

#endif // KOLF_BLENDER_TEXTURE_H
