/***************************************************************************
 *   Copyright 2009 Zeng Huan <zh.issac@gmail.com>
 *   Copyright 2009 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef KOLF_HMCREATOR_DRAWINGBOARD_H
#define KOLF_HMCREATOR_DRAWINGBOARD_H
#include "../engine/gameui-pagewidget.h"
#include "../elements/utils-heightmap.h"
#include <QPixmap>
#include <QPointF>
#include <QPainterPath>

namespace Kolf
{
	class ToolButton;

	/**
	 * The DrawingBoard is a place for painting, it provides a pixmap on which we can paint, it also records the current selected area of the image, we can not paint outside the selected area.
	 *
	 * Choosing a particular pen/brush style is left to the Kolf::ToolButtons, and the m_currentToolButton decides how to handle the events passed to it.
	 */	
	class DrawingBoard : public GameUIPageWidget
	{
		Q_OBJECT
		public:
			DrawingBoard();
			QPointF lastPoint() const {return m_lastPoint;}
			QPixmap &image() {return m_workingImage;}
			void setImage(const QPixmap& image);
			QPainterPath selection() const {return m_selection;}
			void setSelection(const QPainterPath& selection);
			///Stores the current pixmap.
			void storeImage();
			///Restores the saved pixmap.
			void restoreImage();
			///Converts the current pixmap into Kolf::HeightmapRenderer, then we can use this Kolf::HeightmapRenderer to render 3d view.
			HeightmapRenderer toHeightmapRenderer();
			///Sets the current pixmap from a Kolf::HeightmapRenderer.
			QPixmap fromHeightmapRenderer(const Kolf::HeightmapRenderer& heightmapRenderer);

			static const QPen HelperLinesPen;

			void setEditFlag(bool flag) {m_isEditedFromLastBlending = flag;}
			bool isEditedFromLastBlending() {return m_isEditedFromLastBlending;}

		Q_SIGNALS:
			void heightmapChanged(const Kolf::Heightmap& heightmap);

		protected:
			virtual void currentSceneChanged(Kolf::Scene* oldScene, Kolf::Scene* newScene);
			virtual void hideEvent(QHideEvent* event);
			///We always record the position of the mouse, it might be the leftTop point of a selectoin or a point in a painting path.
			virtual void mousePressEvent(QMouseEvent* event);
			///If the m_currentToolButton represents a selecting action, then we want to draw a dynamic selection; If the m_currentToolButton represents a painting tool, we want to do some real painting.
			virtual void mouseMoveEvent(QMouseEvent* event);
			///Makes a selection or do some real painting.
			virtual void mouseReleaseEvent(QMouseEvent* event);
			///Shows the pixmap we paint on.
			virtual void paintEvent(QPaintEvent* event);
			///When a resize event occurs, the size of the pixmap and the selection will change, we need to handle this.
			virtual void resizeEvent(QResizeEvent* event);

		public Q_SLOTS:
			void currentToolButtonChanged(ToolButton* button);

		private:
			void resizeSelection(QResizeEvent* event);
			void resizeImage(QResizeEvent* event);
			///Clears the current pixmap.
			void clearImage();
			///Initializes the pixmap.
			void initializeImage();

			QPointF m_lastPoint;
			QPixmap m_workingImage;
			QPixmap m_storeImage;
			QPainterPath m_selection;
			Kolf::ToolButton* m_currentToolButton;
			///Uses this flag to decide if need to reset the heightmap in Kolf::BlenderManger.
			bool m_isEditedFromLastBlending;
	};
}
#endif //KOLF_HMCREATOR_DRAWINGBOARD_H
