/***************************************************************************
 *   Copyright 2009 Zeng Huan <zh.issac@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "hmcreator-eraserbutton.h"

#include <QPainter>
#include <KIcon>
#include <KLocalizedString>

Kolf::EraserButton::EraserButton(QWidget* parent)
	: ToolButton(parent)
	, m_eraserWidth(10)
{
	setIcon(KIcon("draw-eraser"));
	setText(i18n("Eraser"));
	setToolTip(i18n("Paint on areas of the heightmap to reset those to the lowest height."));
	setObjectName("EraserButton");
}

Kolf::EraserButton::~EraserButton()
{
}

void Kolf::EraserButton::handleMousePressed(DrawingBoard* board, QMouseEvent* event)
{
	if (event->button() == Qt::LeftButton) 
		m_eraserLastPoint = board->lastPoint();
}


void Kolf::EraserButton::handleMouseMoved(DrawingBoard* board, QMouseEvent* event)
{
	erase(board, event);
}

void Kolf::EraserButton::handleMouseReleased(DrawingBoard* board, QMouseEvent* event)
{
	erase(board, event);
}

void Kolf::EraserButton::erase(DrawingBoard* board, QMouseEvent* event)
{
	board->restoreImage();
	QPainter painter(&board->image());
	painter.setClipPath(board->selection());
	painter.setPen(QPen(Qt::black, m_eraserWidth, Qt::SolidLine, Qt::RoundCap));
	painter.drawLine(m_eraserLastPoint, event->posF());
	board->storeImage();
	m_eraserLastPoint = event->posF();
}

#include "hmcreator-eraserbutton.moc"
