/***************************************************************************
 *   Copyright 2009 Zeng Huan <zh.issac@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "hmcreator-fillconfwidget.h"
#include "hmcreator-shadeconfwidget.h"
#include "hmcreator-toolbutton.h"

#include <QGridLayout>
#include <QGroupBox>
#include <QLabel>
#include <QPushButton>
#include <QToolButton>
#include <KIcon>

Kolf::FillConfWidget::FillConfWidget(ToolButton* button, QWidget* parent)
	: QWidget(parent)
	, m_gradientSpread(QGradient::PadSpread)
	, m_gradientType(Qt::RadialGradientPattern)
	, m_button(button)
{
	setupUI();
	connect(this, SIGNAL(typeChanged(const Qt::BrushStyle&)),
		m_button, SLOT(changeGradientType(const Qt::BrushStyle&)));
}

void Kolf::FillConfWidget::buttonChanged()
{	
	QToolButton* button = dynamic_cast<QToolButton*>(QObject::sender());
	
	if(button && button->isChecked() && button != m_previousButton )
	{
		foreach(QToolButton* item, m_buttonList)
		{
			if(item != button)
				item->setChecked(false);
		}
		if(button->objectName() == QString("RadialButton"))
			emit typeChanged(Qt::RadialGradientPattern);
		if(button->objectName() == QString("LinearButton"))
			emit typeChanged(Qt::LinearGradientPattern);
		if(button->objectName() == QString("ConicalButton"))
			emit typeChanged(Qt::ConicalGradientPattern);
		m_previousButton = button;
	}	
	
	bool flag = true;
	foreach(QToolButton* item, m_buttonList)
		if(item->isChecked())
			flag = false;
	if(flag)
		m_previousButton->setChecked(true);
}

void Kolf::FillConfWidget::setupUI() 
{ 
	QGroupBox* groupBox = new QGroupBox(this);
	QGridLayout* layout = new QGridLayout;
	QLabel* title = new QLabel(QString("Gradient Type"));
	layout->addWidget(title, 0, 0, 1, 4);

	//Radial Gradient Button
	m_radialGradientButton = new QToolButton(this);
	prepareButton(m_radialGradientButton);
	m_radialGradientButton->setIcon(KIcon(""));
	m_radialGradientButton->setToolTip(QString("Radial Gradient"));
	m_radialGradientButton->setObjectName(QString("RadialButton"));
	connect(m_radialGradientButton, SIGNAL(toggled(bool)),
		this, SLOT(buttonChanged()));
	m_previousButton = m_radialGradientButton;
	m_radialGradientButton->setChecked(true);
	layout->addWidget(m_radialGradientButton, 1, 0);

	//Linear Gradient Button
	m_linearGradientButton = new QToolButton(this);
	prepareButton(m_linearGradientButton);
	m_linearGradientButton->setIcon(KIcon(""));
	m_linearGradientButton->setToolTip(QString("Linear Gradient"));
	m_linearGradientButton->setObjectName(QString("LinearButton"));
	connect(m_linearGradientButton, SIGNAL(toggled(bool)),
		this, SLOT(buttonChanged()));
	layout->addWidget(m_linearGradientButton, 1, 1);

	//Conical Gradient Button
	m_conicalGradientButton = new QToolButton(this);
	prepareButton(m_conicalGradientButton);
	m_conicalGradientButton->setIcon(KIcon(""));
	m_conicalGradientButton->setToolTip(QString("Conical Gradient"));
	m_conicalGradientButton->setObjectName(QString("ConicalButton"));
	connect(m_conicalGradientButton, SIGNAL(toggled(bool)),
		this, SLOT(buttonChanged()));
	layout->addWidget(m_conicalGradientButton, 1, 2);

	//Shade Widget
	m_shadeWidget = new ShadeConfWidget(m_button, this);
	layout->addWidget(m_shadeWidget, 2, 0, 2, 4);

	QPushButton* resetButton = new QPushButton("Reset");
	layout->addWidget(resetButton, 4, 0, 1, 3);
	connect(resetButton, SIGNAL(clicked()), m_shadeWidget, SLOT(reset()));
	groupBox->setLayout(layout);
}

void Kolf::FillConfWidget::prepareButton(QToolButton* button)
{	
	button->setCheckable(true);
	button->setChecked(false);
	button->setAutoRaise(true);
	button->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
	m_buttonList << button;	
}
#include "hmcreator-fillconfwidget.moc"
