/***************************************************************************
 *   Copyright 2009 Zeng Huan <zh.issac@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "hmcreator-penbutton.h"

#include <QPainter>
#include <KIcon>
#include <KLocalizedString>

Kolf::PenButton::PenButton(QWidget* parent)
	: ToolButton(parent)
	, m_penColor(Qt::white)
	, m_penWidth(1)
{
	setIcon(KIcon("draw-freehand"));
	setText(i18n("Pen"));
	setToolTip(i18n("Paint on areas of the heightmap to set those to some fixed height."));
	setObjectName("PenButton");
}

Kolf::PenButton::~PenButton()
{
}

void Kolf::PenButton::handleMousePressed(DrawingBoard* board, QMouseEvent* event)
{
	if (event->button() == Qt::LeftButton) 
		m_penLastPoint = board->lastPoint();
}


void Kolf::PenButton::handleMouseMoved(DrawingBoard* board, QMouseEvent* event)
{
	paint(board, event);
}

void Kolf::PenButton::handleMouseReleased(DrawingBoard* board, QMouseEvent* event)
{
	paint(board, event);
}

void Kolf::PenButton::paint(DrawingBoard* board, QMouseEvent* event)
{
	board->restoreImage();
	QPainter painter(&board->image());
	painter.setClipPath(board->selection());
	painter.setPen(QPen(m_penColor, m_penWidth, Qt::SolidLine, Qt::RoundCap));
	painter.drawLine(m_penLastPoint, event->posF());
	board->storeImage();
	m_penLastPoint = event->posF();
}
#include "hmcreator-penbutton.moc"
