/***************************************************************************
 *   Copyright 2008-2009 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef KOLF_BALL_H
#define KOLF_BALL_H

#include "object.h"
class QGraphicsEllipseItem;

namespace Kolf
{
	class SphereGeometry;

	/**
	 * \class Ball
	 * \since 2.0
	 *
	 * The ball.
	 */
	//This object's reference point is its center.
	class Ball : public Kolf::Object
	{
		Q_OBJECT
		//color is not a property because it should not be accessible through the editor
		public:
			Ball(dWorldID worldID, dSpaceID spaceID);
			~Ball();

			bool isInHole() const;
			void setInHole(bool inHole);

			virtual bool isRotateable() const { return false; }
			virtual Kolf::Object::SimulationFlags simulationFlags() const;

			static const int BallRadius;
		Q_SIGNALS:
			void positionChanged(const Kolf::PointF& position);
		protected:
			virtual void objectEvent(int type, const QVariant& data);
			virtual Kolf::Overlay* createOverlay();
		private:
			Kolf::SphereGeometry* m_geometry;

			QGraphicsEllipseItem* m_ballItem;
			bool m_inHole;
	};
}

#endif // KOLF_BALL_H
