/***************************************************************************
 *   Copyright 2009 Zeng Huan <zh.issac@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "geometry-terrain.h"
#include "../3d/texturepool.h"

#include <cmath>

QByteArray Kolf__TerrainGeometry__generateTextureKey()
{
	static int id = 0;
	return "terrain" + QByteArray::number(++id);
}

Kolf::TerrainGeometry::TerrainGeometry(dSpaceID spaceID, Kolf::Object* object)
	: Kolf::Geometry(dCreateTriMesh(spaceID,
			dGeomTriMeshDataCreate(), //dTrimeshDataID
			0, //dTriCallback*
			0, //dTriArrayCallback*
			0) //dTriRayCallback*
		, object)
	, m_dataId(dGeomTriMeshGetTriMeshDataID(Geometry::id()))
	, m_textureKey(Kolf__TerrainGeometry__generateTextureKey())
	, m_sceneSize(1000, 1000, 200)
	, m_xNum(50)
	, m_yNum(50)
{
}

Kolf::TerrainGeometry::~TerrainGeometry()
{
	dGeomTriMeshDataDestroy(m_dataId);
}

const Kolf::HeightmapRenderer& Kolf::TerrainGeometry::heightmapRenderer() const
{
	return m_heightmapRenderer;
}

void Kolf::TerrainGeometry::setHeightmapRenderer(const Kolf::HeightmapRenderer& heightmapRenderer)
{
	m_heightmapRenderer = heightmapRenderer;
	m_heightmapRenderer.computeData(m_xNum, m_yNum, m_sceneSize);
	/* Note: using ode-0.11
	 * ode/common.h:101:typedef float dReal;
	 * ode/common.h:106:typedef double dReal;
	 * ode/common.h:124:typedef dReal dVector3[4];
	 *
	 * ODE_API void dGeomTriMeshDataBuildSingle(dTriMeshDataID g,
	 *                         const void* Vertices, int VertexStride, int VertexCount,
	 *                         const void* Indices, int IndexCount, int TriStride);
	 *
	 * ODE_API void dGeomTriMeshDataBuildSingle1(dTriMeshDataID g,
	 *                         const void* Vertices, int VertexStride, int VertexCount,
	 *                         const void* Indices, int IndexCount, int TriStride,
	 *                         const void* Normals);
	 */
	dGeomTriMeshDataBuildSingle(m_dataId,
		m_heightmapRenderer.odeVertices().data(), sizeof(Vertex_t), m_heightmapRenderer.odeVertices().size(),
		m_heightmapRenderer.odeIndices().data(), m_heightmapRenderer.odeIndices().size(), sizeof(Triangle_t)
	);
	/*
	dGeomTriMeshDataBuildSingle1(m_dataId,
		test_vertices, 4 * sizeof(dReal), (m_xNum + 1) * (m_yNum + 1),
		m_indices.data(), numIndices / 3, 3 * sizeof(int)
		, m_normals.data());
	 */
	dGeomTriMeshSetData(id(), m_dataId);
}

QByteArray Kolf::TerrainGeometry::textureKey() const
{
	return m_textureKey;
}

void Kolf::TerrainGeometry::setTexture(const QImage& texture)
{
	m_texture = texture;
	//2D view
	m_item = QPixmap::fromImage(m_texture).scaled(m_sceneSize.xSpread(), m_sceneSize.ySpread());
	//3D view
	Kolf::TexturePool::instance()->addTexture(m_textureKey, m_texture);
}

void Kolf::TerrainGeometry::createOutlines()
{
	//do not specify active regions (the environment object is not selectable)
	setInteractionOutline(QPainterPath());
	setActivationOutline(QPainterPath());
}
