/***************************************************************************
 *   Copyright 2009 Zeng Huan <zh.issac@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef KOLF_GEOMETRY_TERRAIN_H
#define KOLF_GEOMETRY_TERRAIN_H

#include "geometry.h"
#include "math-pointf.h"
#include "utils-heightmap.h"

#include <ode/collision_trimesh.h>
#include <QPixmap>

namespace Kolf
{
	class HeightmapRenderer;

	/**
	 * \class TerrainGeometry
	 * \since 2.0
	 *
	 * Represents the geometry of a terrain, using ODE's trimesh geometry.
	 */
	class TerrainGeometry : public Kolf::Geometry
	{
		public:
			TerrainGeometry(dSpaceID spaceID, Kolf::Object* object);
			~TerrainGeometry();
			///Returns the current heightmap, or an empty heightmap if no heightmap has been set.
			const Kolf::HeightmapRenderer& heightmapRenderer() const;
			///Creates heightmap and then build up trimesh data.
			void setHeightmapRenderer(const Kolf::HeightmapRenderer& heightmapRenderer);
			///Loads texture, based on image.
			void setTexture(const QImage& texture);
			///Returns the picture which is used for 2d view.
			const QPixmap& backdrop() const {return m_item;}
			QByteArray textureKey() const;
		protected:
			virtual void createOutlines();
		private:
			//direct representations of heightmap and texture
			Kolf::HeightmapRenderer m_heightmapRenderer;
			QImage m_texture;
			//intermediate representations for simulation and rendering
			dTriMeshDataID m_dataId;
			QPixmap m_item;
			QByteArray m_textureKey;
			///m_sceneSize records the size of Kolf's scene, now is x:1000 y:1000 z:200(undetermined).
			Kolf::SizeF m_sceneSize;
			///vertices number in x and y direction, we always set to 50 and 50 to avoid too much computation memory consumption.
			int m_xNum, m_yNum;
	};
}

#endif // KOLF_GEOMETRY_TERRAIN_H
