/***************************************************************************
 *   Copyright 2008-2009 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "object-wall.h"
#include "geometry-wall.h"
#include "objectregistry.h"
#include "overlay-wall.h"

#include <QPen>

REGISTER_KOLF_OBJECT(Kolf::WallObject, ki18n("Wall"), KIcon())

Kolf::WallObject::WallObject(dWorldID worldID, dSpaceID spaceID)
	: Kolf::Object(worldID, spaceID)
	, m_geometry(new Kolf::WallGeometry(spaceID, this))
	, m_line(new QGraphicsLineItem(QLineF(), this))
{
	QPen pen = m_line->pen();
	pen.setWidth(Kolf::WallGeometry::Thickness);
	m_line->setPen(pen);
}

Kolf::WallObject::~WallObject()
{
	delete m_line;
}

Kolf::VectorF Kolf::WallObject::extent() const
{
	return m_geometry->extent();
}

void Kolf::WallObject::setExtent(const Kolf::VectorF& extent)
{
	if (m_geometry->extent() != extent)
	{
		const QPointF extent2D(extent.x(), extent.y());
		m_line->setLine(QLineF(-extent2D, extent2D));
		m_geometry->setExtent(extent);
		propagateUpdate();
	}
}

void Kolf::WallObject::setExtent(const QPointF& point1, const QPointF& point2)
{
	const int courseBoundaryZExtent = 100;
	const QPointF center = (point2 + point1) / 2;
	const QPointF extent = (point2 - point1) / 2;
	setPosition(Kolf::PointF(center.x(), center.y(), 0));
	setExtent(Kolf::PointF(extent.x(), extent.y(), courseBoundaryZExtent));
}

Kolf::Overlay* Kolf::WallObject::createOverlay()
{
	return new Kolf::WallOverlay(this);
}

#include "object-wall.moc"
