/***************************************************************************
 *   Copyright 2009 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "objectregistry.h"
#include "object.h"
#include "scene.h"

//BEGIN Kolf::ObjectRegistry

Kolf::ObjectRegistry* __Kolf_Object_Registry_Instance()
{
	static Kolf::ObjectRegistry registry;
	return &registry;
}

QList<QByteArray> Kolf::ObjectRegistry::knownClasses()
{
	return __Kolf_Object_Registry_Instance()->m_classData.keys();
}

QString Kolf::ObjectRegistry::translatedName(const QByteArray& className)
{
	const Kolf::ObjectRegistry* const instance = __Kolf_Object_Registry_Instance();
	if (instance->m_classData.contains(className))
		return instance->m_classData[className].m_name.toString();
	else
		return QString();
}

QIcon Kolf::ObjectRegistry::icon(const QByteArray& className)
{
	const Kolf::ObjectRegistry* const instance = __Kolf_Object_Registry_Instance();
	if (instance->m_classData.contains(className))
		return instance->m_classData[className].m_icon;
	else
		return QIcon();
}

Kolf::Object* Kolf::ObjectRegistry::createObject(const QByteArray& className, Kolf::Scene* scene)
{
	const Kolf::ObjectRegistry* const instance = __Kolf_Object_Registry_Instance();
	if (instance->m_classData.contains(className))
		return instance->m_classData[className].m_creator(scene->worldID(), scene->spaceID());
	else
		return 0;
}

Kolf::Object* Kolf::ObjectRegistry::createObject(const QByteArray& className, Kolf::Object* object)
{
	const Kolf::ObjectRegistry* const instance = __Kolf_Object_Registry_Instance();
	if (instance->m_classData.contains(className))
		return instance->m_classData[className].m_creator(object->m_worldID, object->m_spaceID);
	else
		return 0;
}

//END Kolf::ObjectRegistry

Kolf::ObjectRegistrar::ObjectRegistrar(const char* className, const KLocalizedString& name, const QIcon& icon, Kolf::ObjectCreator creator)
{
	Kolf::ObjectRegistry* const instance = __Kolf_Object_Registry_Instance();
	instance->m_classData[className].m_name = name;
	instance->m_classData[className].m_icon = icon;
	instance->m_classData[className].m_creator = creator;
}
