/***************************************************************************
 *   Copyright 2008-2009 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "overlay-rect.h"
#include "handle-point.h"
#include "object.h"
#include "math-sizef.h"

#include <QGraphicsRectItem>
#include <QPen>

Kolf::RectOverlay::RectOverlay(Kolf::Object* object)
	: Kolf::Overlay(object)
	, m_topLeftHandle(new Kolf::PointHandle(Kolf::PointHandle::SquareShape, this))
	, m_topRightHandle(new Kolf::PointHandle(Kolf::PointHandle::SquareShape, this))
	, m_bottomLeftHandle(new Kolf::PointHandle(Kolf::PointHandle::SquareShape, this))
	, m_bottomRightHandle(new Kolf::PointHandle(Kolf::PointHandle::SquareShape, this))
	, m_topHandle(new Kolf::PointHandle(Kolf::PointHandle::SquareShape, this))
	, m_leftHandle(new Kolf::PointHandle(Kolf::PointHandle::SquareShape, this))
	, m_rightHandle(new Kolf::PointHandle(Kolf::PointHandle::SquareShape, this))
	, m_bottomHandle(new Kolf::PointHandle(Kolf::PointHandle::SquareShape, this))
{
	addHandle(m_topLeftHandle);
	addHandle(m_topRightHandle);
	addHandle(m_bottomLeftHandle);
	addHandle(m_bottomRightHandle);
	addHandle(m_topHandle);
	addHandle(m_leftHandle);
	addHandle(m_rightHandle);
	addHandle(m_bottomHandle);
	connect(m_topLeftHandle, SIGNAL(moveRequest(const QPointF&)), this, SLOT(moveHandle(const QPointF&)));
	connect(m_topRightHandle, SIGNAL(moveRequest(const QPointF&)), this, SLOT(moveHandle(const QPointF&)));
	connect(m_bottomLeftHandle, SIGNAL(moveRequest(const QPointF&)), this, SLOT(moveHandle(const QPointF&)));
	connect(m_bottomRightHandle, SIGNAL(moveRequest(const QPointF&)), this, SLOT(moveHandle(const QPointF&)));
	connect(m_topHandle, SIGNAL(moveRequest(const QPointF&)), this, SLOT(moveHandle(const QPointF&)));
	connect(m_leftHandle, SIGNAL(moveRequest(const QPointF&)), this, SLOT(moveHandle(const QPointF&)));
	connect(m_rightHandle, SIGNAL(moveRequest(const QPointF&)), this, SLOT(moveHandle(const QPointF&)));
	connect(m_bottomHandle, SIGNAL(moveRequest(const QPointF&)), this, SLOT(moveHandle(const QPointF&)));
}

void Kolf::RectOverlay::update()
{
	Kolf::Overlay::update();
	const Kolf::SizeF size3D = object()->property("Size").value<Kolf::SizeF>();
	const QSizeF size2D(size3D.xSpread(), size3D.ySpread());
	QRectF rect2D(QPointF(), size2D); rect2D.moveCenter(QPointF(0, 0));

	m_topLeftHandle->setPos(rect2D.topLeft());
	m_topRightHandle->setPos(rect2D.topRight());
	m_bottomLeftHandle->setPos(rect2D.bottomLeft());
	m_bottomRightHandle->setPos(rect2D.bottomRight());
	m_topHandle->setPos(QPointF(0.0, rect2D.top()));
	m_bottomHandle->setPos(QPointF(0.0, rect2D.bottom()));
	m_leftHandle->setPos(QPointF(rect2D.left(), 0.0));
	m_rightHandle->setPos(QPointF(rect2D.right(), 0.0));
}

void Kolf::RectOverlay::moveHandle(const QPointF& handleScenePos)
{
	const Kolf::PointHandle* handle = qobject_cast<Kolf::PointHandle*>(sender());
	if (!handle)
		return;
	const QVectorF difference(mapFromScene(handleScenePos) - handle->pos());
	const qreal minSpread = Kolf::Overlay::MinimumObjectDimension;
	//read size
	Kolf::SizeF size3D = object()->property("Size").value<Kolf::SizeF>();
	//changes on left or right side
	if (handle == m_topLeftHandle || handle == m_leftHandle || handle == m_bottomLeftHandle)
		size3D.rxSpread() -= difference.x();
	else if (handle == m_topRightHandle || handle == m_rightHandle || handle == m_bottomRightHandle)
		size3D.rxSpread() += difference.x();
	if (size3D.xSpread() < minSpread)
		size3D.setXSpread(minSpread);
	//changes on left or right side
	if (handle == m_topLeftHandle || handle == m_topHandle || handle == m_topRightHandle)
		size3D.rySpread() -= difference.y();
	else if (handle == m_bottomLeftHandle || handle == m_bottomHandle || handle == m_bottomRightHandle)
		size3D.rySpread() += difference.y();
	if (size3D.ySpread() < minSpread)
		size3D.setYSpread(minSpread);
	//save size
	object()->setProperty("Size", qVariantFromValue(size3D));
}

#include "overlay-rect.moc"
