/***************************************************************************
 *   Copyright 2009 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef KOLF_OVERLAYAREAITEM_H
#define KOLF_OVERLAYAREAITEM_H

#include <QGraphicsPathItem>
#include <QGraphicsSceneMouseEvent>

namespace Kolf
{
	class OverlayAreaItem : public QObject, public QGraphicsPathItem
	{
		Q_OBJECT
		public:
			enum Feature
			{
				Draggable = 1 << 0,
				Clickable = 1 << 1,
				Hoverable = 1 << 2
			};
			Q_DECLARE_FLAGS(Features, Feature)

			OverlayAreaItem(Features features, QGraphicsItem* parent);
		Q_SIGNALS:
			void clicked(int button);
			void dragged(const QPointF& distance);
			void hoverEntered();
			void hoverLeft();
		protected:
			virtual void hoverEnterEvent(QGraphicsSceneHoverEvent* event);
			virtual void hoverLeaveEvent(QGraphicsSceneHoverEvent* event);
			virtual void mousePressEvent(QGraphicsSceneMouseEvent* event);
			virtual void mouseMoveEvent(QGraphicsSceneMouseEvent* event);
		private:
			Features m_features;
	};
}

Q_DECLARE_OPERATORS_FOR_FLAGS(Kolf::OverlayAreaItem::Features)

#endif // KOLF_OVERLAYAREAITEM_H
