/***************************************************************************
 *   Copyright 2009 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef KOLF_UTILS_BYTEARRAYCODEC_H
#define KOLF_UTILS_BYTEARRAYCODEC_H

#include <QByteArray>
#include <QVariant>

namespace Kolf
{
	/**
	 * \class ByteArrayCodec
	 * \since 2.0
	 *
	 * This singleton class allows to use Kolf's 3D geometry structures with KConfig, by serializing these structures into a QByteArray, or deserializing them from a QByteArray.
	 */
	class ByteArrayCodec
	{
		public:
			///Compares QVariants. In constrast to "value1 == value2", this method correctly deserializes encoded Kolf's 3D geometry structures.
			static bool mathAwareCompare(const QVariant& value1, const QVariant& value2);
			///Serializes a given Kolf::PointF, Kolf::SizeF or Kolf::CuboidF structure into a QByteArray.
			static QByteArray encode(const QVariant& value);
			///Deserializes a given Kolf::PointF, Kolf::SizeF or Kolf::CuboidF structure from a QByteArray.
			static QVariant decode(const QByteArray& value, const QVariant& defaultValue);
		private:
			ByteArrayCodec();
			static const Kolf::ByteArrayCodec* instance;
			Q_DISABLE_COPY(ByteArrayCodec)

			int m_pointFTypeID;
			int m_sizeFTypeID;
			int m_cuboidFTypeID;
	};
}

#endif // KOLF_UTILS_BYTEARRAYCODEC_H
