/***************************************************************************
 *   Copyright 2009 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "component-scorecard.h"
#include "scoremodel.h"
#include "transposeproxymodel.h"

#include <QHeaderView>
#include <QTableView>
#include <KLocalizedString>

Kolf::ScoreCardComponent::ScoreCardComponent()
	: Kolf::GameUIDockWidget(i18n("Score card"))
	, m_view(new QTableView)
	, m_tpModel(new Kolf::TransposeProxyModel)
	, m_model(new Kolf::ScoreModel)
{
	setWidget(m_view);
	m_view->setModel(m_model);
	m_tpModel->setSourceModel(m_model);
	//configure dock widget
	setFeatures(DockWidgetMovable | DockWidgetFloatable);
	setObjectName("ScoreCardDock");
	setPreferredArea(Qt::RightDockWidgetArea);
	connect(this, SIGNAL(dockLocationChanged(Qt::DockWidgetArea)), this, SLOT(slotDockLocationChanged(Qt::DockWidgetArea)));
	//configure view
	m_view->horizontalHeader()->setResizeMode(QHeaderView::ResizeToContents);
	m_view->horizontalHeader()->show();
	m_view->verticalHeader()->setResizeMode(QHeaderView::ResizeToContents);
	m_view->verticalHeader()->show();
	m_view->setCornerButtonEnabled(false);
}

Kolf::ScoreCardComponent::~ScoreCardComponent()
{
	delete m_tpModel;
	delete m_model;
}

void Kolf::ScoreCardComponent::setGame(Kolf::Game* game)
{
	m_model->setGame(game);
}

void Kolf::ScoreCardComponent::slotDockLocationChanged(Qt::DockWidgetArea area)
{
	bool usingTpModel = m_view->model() == m_tpModel;
	//set model orientation
	if (area == Qt::LeftDockWidgetArea || area == Qt::RightDockWidgetArea)
	{
		if (usingTpModel)
			m_view->setModel(m_model);
	}
	else
	{
		if (!usingTpModel)
			m_view->setModel(m_tpModel);
	}
}

#include "component-scorecard.moc"
