/***************************************************************************
 *   Copyright 2009 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef KOLF_GAMEUI_DOCKWIDGET_H
#define KOLF_GAMEUI_DOCKWIDGET_H

#include <QDockWidget>

namespace Kolf
{
	class Scene;

	/**
	 * \class GameUIDockWidget
	 * \since 2.0
	 *
	 * This is a thin wrapper around QDockWidget which provides some additional semantics for Kolf::GameUI. Also, it holds a reference to the current Kolf::Scene, and notifies the subclass implementation if the current scene instance changes.
	 */
	class GameUIDockWidget : public QDockWidget
	{
		Q_OBJECT
		public:
			///Create a new dock widget with the given \a title.
			GameUIDockWidget(const QString& title);

			///Returns the area into which the dock widget should be put initially when switching to a new game UI parent.
			Qt::DockWidgetArea preferredArea() const;
			///\see preferredArea()
			void setPreferredArea(Qt::DockWidgetArea area);
		public Q_SLOTS:
			///This slot is called by the game (or the game UI, respectively) when the scene changes.
			void setScene(Kolf::Scene* scene);
		protected:
			///Reimplement this function to react on a changing scene instance. (The default implementation does nothing.) You can safely do any cleanup on the old scene, it is guaranteed that the old scene was not yet deleted when this function is called.
			///\warning It is perfectly possible that one of both parameters is a null pointer.
			virtual void currentSceneChanged(Kolf::Scene* oldScene, Kolf::Scene* newScene);
		private:
			Kolf::Scene* m_scene;
			Qt::DockWidgetArea m_preferredArea;
	};
}

#endif // KOLF_GAMEUI_DOCKWIDGET_H
