/***************************************************************************
 *   Copyright 2009, 2010 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "object-course.h"
#include "object-ball.h"
#include "object-wall.h"
#include "objectfactory.h"
#include "overlay-empty.h"
#include "utils-listcasting.h"

#include <QGraphicsScene>

REGISTER_KOLF_OBJECT(Kolf::CourseObject, {
	classObject->screenName = ki18n("Internal course representation");
	classObject->editorMayCreateInstances = false;
	classObject->groupName = QLatin1String("01-basics");
})

Kolf::CourseObject::CourseObject(b2World* world)
	: Kolf::Object(world)
	, m_hasBoundary(false)
{
}

bool Kolf::CourseObject::hasBoundary() const
{
	return m_hasBoundary;
}

void Kolf::CourseObject::setBoundary(bool hasBoundary)
{
	if (m_hasBoundary == hasBoundary)
		return;
	if ((m_hasBoundary = hasBoundary))
	{
		//create boundary
		Kolf::ObjectFactory* factory = objectFactory();
		for (int i = 0; i < 4; ++i)
		{
			Kolf::Object* wallObject = factory->create("Kolf::WallObject", this);
			addSubObject(wallObject);
			m_wallObjects << qobject_cast<Kolf::WallObject*>(wallObject);
		}
		updateBoundaryPosition(size());
	}
	else
	{
		//delete boundary
		foreach (Kolf::Object* wallObject, m_wallObjects)
		{
			removeSubObject(wallObject);
			delete wallObject;
		}
	}
	propagateUpdate();
}

QSizeF Kolf::CourseObject::size() const
{
	return scene()->sceneRect().size();
}

void Kolf::CourseObject::setSize(const QSizeF& size)
{
	if (this->size() == size)
		return;
	scene()->setSceneRect(QRectF(QPointF(), size));
	if (m_hasBoundary)
		updateBoundaryPosition(size);
	propagateUpdate();
}

void Kolf::CourseObject::updateBoundaryPosition(const QSizeF& size)
{
	if (m_wallObjects.count() != 4)
	{
		qWarning("Error in Kolf::CourseObject: Expected 4 wall objects, found %i wall objects", m_wallObjects.count());
		return;
	}
	QRectF rect(QPointF(), size);
	m_wallObjects[0]->setExtent(rect.topLeft(), rect.topRight());       //top wall
	m_wallObjects[1]->setExtent(rect.bottomLeft(), rect.bottomRight()); //bottom wall
	m_wallObjects[2]->setExtent(rect.topLeft(), rect.bottomLeft());     //left wall
	m_wallObjects[3]->setExtent(rect.topRight(), rect.bottomRight());   //right wall
}

Kolf::BallObject* Kolf::CourseObject::createBallObject()
{
	Kolf::Object* ball = objectFactory()->create("Kolf::BallObject");
	ball->setPos(pos());
	return qobject_cast<Kolf::BallObject*>(ball);
}

Kolf::Overlay* Kolf::CourseObject::createOverlay()
{
	return new Kolf::EmptyOverlay(this); //TODO: an overlay that connects to a Kolf::BallObject to monitor its position
}

#include "object-course.moc"
