/***************************************************************************
 *   Copyright 2008, 2009, 2010 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "putter-simple.h"

#include <QPen>

Kolf::SimplePutter::SimplePutter(QGraphicsScene* scene)
	: Kolf::Putter(scene)
	, m_lineItem(new QGraphicsLineItem(graphicsItem()))
	, m_direction(0)
	, m_activeView(0)
{
	m_lineItem->setPen(Qt::DashLine);
	graphicsItem()->setAcceptedMouseButtons(Qt::LeftButton | Qt::RightButton);
}

void Kolf::SimplePutter::handleMousePress(Qt::MouseButton button, const QPointF& cursorPosition)
{
	Q_UNUSED(cursorPosition) //We rely on our update cache.
	switch ((int) button)
	{
		case Qt::LeftButton:
			//read velocity from line vector, and do putting
			setVelocity(m_lineItem->line().p2());
			break;
		case Qt::RightButton:
			//change direction of velocity vector
			m_direction = (m_direction + 1) % 4;
			updateAppearance();
			break;
	}
}

void Kolf::SimplePutter::update(QWidget* activeView, const QPointF& cursorPosition)
{
	m_activeView = activeView;
	m_cursorPosition = cursorPosition;
	updateAppearance();
}

void rotateBy90Degrees(QPointF& p)
{
	qreal x = -p.x();
	p.rx() = p.y();
	p.ry() = x;
}

void Kolf::SimplePutter::updateAppearance()
{
	if (!m_activeView)
		m_lineItem->hide();
	else
	{
		m_lineItem->show();
		//calculate line
		QPointF lineEnd = m_cursorPosition;
		for (int i = 0; i < m_direction; ++i)
			rotateBy90Degrees(lineEnd);
		//update line
		m_lineItem->setLine(QLineF(QPointF(), lineEnd)); //NOTE: The fact that the first point is (0,0) is used by handleMousePress()!
	}
}
