/***************************************************************************
 *   Copyright 2010 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "scene-contactlistener.h"

#include <KGlobal>
#include <Box2D/Dynamics/Contacts/b2Contact.h>

//BEGIN Kolf::ContactListener

void Kolf::ContactListener::BeginContact(b2Contact* contact)
{
	b2Fixture* fixture1 = contact->GetFixtureA();
	b2Fixture* fixture2 = contact->GetFixtureB();
	//convert b2Contact -> Kolf::ContactData
	Kolf::ContactData data;
	data.object1 = reinterpret_cast<Kolf::Object*>(fixture1->GetBody()->GetUserData());
	data.object2 = reinterpret_cast<Kolf::Object*>(fixture2->GetBody()->GetUserData());
	data.shape1 = reinterpret_cast<Kolf::Shape*>(fixture1->GetUserData());
	data.shape2 = reinterpret_cast<Kolf::Shape*>(fixture2->GetUserData());
	//save contact in list of encountered contacts
	m_encounteredContacts << data;
}

void Kolf::ContactListener::clear()
{
	m_encounteredContacts.clear();
}

QList<Kolf::ContactData> Kolf::ContactListener::encounteredContacts() const
{
	return m_encounteredContacts;
}

//END Kolf::ContactListener
//BEGIN Kolf::ContactCallback and friends

struct ContactCallbackRegistry
{
	QList<Kolf::ContactCallback> callbackList;
};

K_GLOBAL_STATIC(ContactCallbackRegistry, contactCallbackRegistry)

QList<Kolf::ContactCallback> Kolf::knownContactCallbacks()
{
	return contactCallbackRegistry->callbackList;
}

Kolf::ContactCallbackRegistrar::ContactCallbackRegistrar(Kolf::ContactCallback callback)
{
	if (!contactCallbackRegistry->callbackList.contains(callback))
		contactCallbackRegistry->callbackList << callback;
}

//END Kolf::ContactCallback and friends
