/***************************************************************************
 *   Copyright 2008, 2009, 2010 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "shape.h"
#include "object.h"
#include "overlay.h"

#include <Box2D/Dynamics/b2Fixture.h>

const qreal Kolf::Shape::ActivationOutlinePadding = 0.1;

Kolf::Shape::Shape()
	: m_traits(Kolf::Shape::ParticipatesInPhysicalSimulation)
	, m_body(0)
	, m_fixtureDef(new b2FixtureDef)
	, m_fixture(0)
	, m_shape(0)
	, m_object(0)
{
	m_fixtureDef->density = 1;
	m_fixtureDef->restitution = 0.8;
	m_fixtureDef->friction = 0;
	m_fixtureDef->userData = this; //NOTE: this is used by Kolf::ContactListener
}

Kolf::Shape::~Shape()
{
	delete m_fixtureDef;
	updateFixture(0); //clear fixture and shape
}

bool Kolf::Shape::attach(Kolf::Object* object, b2Body* body)
{
	if (m_body)
		return false;
	m_body = body;
	m_object = object;
	updateFixture(createShape());
	return true;
}

QPainterPath Kolf::Shape::activationOutline() const
{
	return m_activationOutline;
}

QPainterPath Kolf::Shape::interactionOutline() const
{
	return m_interactionOutline;
}

void Kolf::Shape::setActivationOutline(const QPainterPath& outline)
{
	m_activationOutline = outline;
}

void Kolf::Shape::setInteractionOutline(const QPainterPath& outline)
{
	m_interactionOutline = outline;
}

Kolf::Shape::Traits Kolf::Shape::traits() const
{
	return m_traits;
}

void Kolf::Shape::setTraits(Kolf::Shape::Traits traits)
{
	if (m_traits == traits)
		return;
	m_traits = traits;
	updateFixture(createShape());
}

void Kolf::Shape::updateFixture(b2Shape* newShape)
{
	if (!m_body)
		return;
	//destroy old fixture
	if (m_fixture)
		m_body->DestroyFixture(m_fixture);
	delete m_shape;
	//create new fixture
	if (m_traits & Kolf::Shape::CollisionDetectionFlag)
	{
		m_shape = newShape;
		if (m_shape)
		{
			b2FixtureDef fixtureDef = *m_fixtureDef;
			fixtureDef.shape = m_shape;
			fixtureDef.isSensor = !(m_traits & Kolf::Shape::PhysicalSimulationFlag);
			m_fixture = m_body->CreateFixture(&fixtureDef);
		}
	}
	else
		delete newShape; //TODO: inefficient
}

void Kolf::Shape::update()
{
	updateFixture(createShape());
	createOutlines();
	//propagate update to overlays
	if (m_object)
	{
		Kolf::Overlay* overlay = m_object->overlay(false);
		if (overlay)
			overlay->update();
	}
}
