/***************************************************************************
 *   Copyright 2008, 2009, 2010 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef KOLF_COURSE_H
#define KOLF_COURSE_H

#include <QObject>

namespace Kolf
{
	class Course;
	class ManagedObjectGroup;
	class Putter;
	class Scene;

	/**
	 * \class Hole
	 * \since 2.0
	 *
	 * This class represents a hole. Instances of this class can only be obtained from Kolf::Course.
	 */
	class Hole
	{
		protected:
			friend class Course; //manages instances of this class
			Hole(int parScore);
			~Hole();
		public:
			int parScore() const;
			Kolf::Scene* scene() const;
			Kolf::ManagedObjectGroup* managedObjectGroup() const;
			Kolf::Putter* putter() const;

			void setParScore(int parScore);
		private:
			int m_parScore;
			Kolf::Scene* m_scene;
			Kolf::ManagedObjectGroup* m_managedObjectGroup;
			Kolf::Putter* m_putter;
	};

	/**
	 * \class Course
	 * \since 2.0
	 *
	 * This class represents a course, a set of holes. It can be loaded from and saved to standard locations.
	 */
	class Course : public QObject
	{
		Q_OBJECT
		public:
			///Gives the file names of all available courses.
			static QStringList availableCourses();
			///Returns the title of the course with the given file name.
			static QString courseTitle(const QString& courseName);
			///Creates a new course object from the course file with the given \a name.
			Course(const QString& courseName);
			~Course();

			///Returns the name of this course, or an empty string if the course file was invalid.
			QString title() const;
			///Returns the Kolf representation of all holes in this course, or an empty list if the course file was invalid.
			QList<Kolf::Hole*> holes() const;
		public Q_SLOTS:
			///Saves the changes that have been made to this course.
			void save() const;
		private:
			QString m_courseName, m_title;
			QList<Kolf::Hole*> m_holes;
	};
}

#endif // KOLF_COURSE_H
