/***************************************************************************
 *   Copyright 2010 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef KOLF_INTERFACE_H
#define KOLF_INTERFACE_H

class QDockWidget;
class QGraphicsScene;
#include <QMap>
#include <QObject>
class KAction;
class KXmlGuiWindow;

namespace Kolf
{
	enum Action
	{
		//Game menu
		NewAction,
		SaveAction,
		QuitAction,
		//Course menu
		PreviousHoleAction,
		NextHoleAction,
		ChooseHoleAction,
		//Hole menu
		UndoAction,
		RedoAction
	};

	class View;

	/**
	 * \class Interface
	 * \since 2.0
	 *
	 * This class provides a coherent interface to the main window. It manages
	 * the common actions and the window title.
	 */
	class Interface : public QObject
	{
		Q_OBJECT
		public:
			Interface(KXmlGuiWindow* window);

			///Forwards dock widgets to the main window. The Game is responsible for cleaning up its dock widgets when it is destroyed.
			void addDockWidget(Qt::DockWidgetArea area, QDockWidget* widget);
			///\note The hole navigation actions will ignore this function. Their state is controlled internally by the hole count and current hole. \sa setCurrentHole(), setHoleCount()
			void setActionEnabled(Kolf::Action action, bool enabled);
		public Q_SLOTS:
			void setCurrentHole(int hole);
			void setHoleCount(int holes);
			void setScene(QGraphicsScene* scene);
			void setWindowTitle(const QString& windowTitle);
			void setClean(bool clean);
			void setRedoEnabled(bool redoEnabled);
			void setUndoEnabled(bool undoEnabled);
		Q_SIGNALS:
			void holeSelected(int hole);
			void redoRequest();
			void undoRequest();
			void saveRequest();
		private Q_SLOTS:
			void actionTriggeredMapper();
			void actionTriggeredMapper(int index);
		private:
			void setupHoleActions();

			KXmlGuiWindow* m_window;
			Kolf::View* m_view;
			QMap<Kolf::Action, KAction*> m_actions;
			int m_currentHole, m_holeCount;
			bool m_clean; QString m_windowTitle;
	};
}

#endif // KOLF_INTERFACE_H
