#ifndef _IMAGEVIEW_H_
#define _IMAGEVIEW_H_

#include <QGraphicsView>
#include <QGraphicsItem>
#include <QMouseEvent>
#include <KDebug>
#include <QString>
#include <QList>

#define BORDER_SIZE 10
#define TEXT_HEIGHT 20
#define ITEM_TYPE_KEY 1001
#define ITEM_TYPE_NONE 0
#define ITEM_TYPE_NAME 1
#define ITEM_TYPE_IMAGE 2
#define ITEM_TYPE_SELECTION 3
#define ITEM_TYPE_PARENT 4


class KGGraphicsParentItem: public QGraphicsItem
{
private:
    int width;
    int height;
    
public:
    KGGraphicsParentItem(int argWidth, int argHeight, QGraphicsItem *parent = 0);
    QRectF boundingRect() const;
    void resize(int newWidth, int newHeight);
    void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget);
};

struct imageObject 
{
    QGraphicsItem *image;
    QGraphicsItem *name;
    KGGraphicsParentItem *parent;
};

class KGGraphicsNameItem: public QGraphicsItem
{
private:
    QString name;
    int width;
    
public:
    KGGraphicsNameItem(int argWidth, QString &argName,QGraphicsItem *parent = 0);
    QRectF boundingRect() const;
    void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget);
};

class KGGraphicsSelectionItem: public QGraphicsItem
{
private:
    int width;
    int height;
    
public:
    KGGraphicsSelectionItem(int argWidth, int argHeight, QGraphicsItem *parent = 0);
    QRectF boundingRect() const;
    virtual ~KGGraphicsSelectionItem();
    void resize(int newWidth, int newHeight);
    void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget);
};



/** 
    @author Keith Neuse<KNeuse@Gmail.com>
    This class uses the QGraphicsView control for displaying the various objects where the user
    can make their select

    TODO: this entire class needs to be purged and QListView needs to be used in its place.
**/

class KGImageView: public QGraphicsView 
{
    Q_OBJECT
protected:
    //vars
    int maxImageWidth;
    int maxImageHeight;
    bool itemSelected;
    void resizeAllText(void);
    KGGraphicsSelectionItem *selectionBox;
    QGraphicsScene scene;
    QList <imageObject> imageObjectList;
public:
    
    /** constructor **/
    KGImageView(QWidget * parent=0, const char* name=0);
    /** deconstructor **/
    virtual ~KGImageView();

    /**
	adds an object to the imageView control
	@PARAM newItem = the item to add
	@PARAM name = the caption for the item
    **/
    void addItem(QGraphicsItem *newItem, QString name);

//events

    /**
	handles the resize event for this control
    **/
    void resizeEvent(QResizeEvent *event);
    
    /**
	handles mouse events mainly for selecting objects
    **/
    void mousePressEvent(QMouseEvent *event);

signals:

    /**
	This signal is emitted when a new object is selected
    **/
    void selectionChanged(QGraphicsItem *item);
};

#endif
