#ifndef _KGTOOLDRAWER_H_
#define _KGTOOLDRAWER_H_

#include <KPushButton>
#include <QWidget>
#include <QPixmap>
#include <QList>
#include <KUrl>
#include <KFileDialog>

#include <QXmlStreamWriter>
#include <QDomDocument>
#include <QDomElement>
#include <QVBoxLayout>

#include "KGImageView.h"
#include "KGLevelEditor.h"
#include "KGContainer.h"
#include "KGObject.h"
#include "KGTile.h"

/** 
    @author Keith Neuse<KNeuse@Gmail.com>
    This class is responsible for handling the individual panels
    that are within the QToolBox
**/

class KGToolDrawer: public QWidget
{
Q_OBJECT
private:
    //vars
    KPushButton *btnAdd;
    KGImageView *imageView;
    KGContainer *myContainer;
    KGObject *selectedObject;
    QList<KGObject *> objectList;
    QString m_name;
    int layerID;

public:
    /** constructor **/
    KGToolDrawer(int argLayerID, QWidget *parent = 0, QString = QString());

    /**
	returns the selection object for the panel
    **/
    KGObject *getSelectedObject();

    /**
	saves all objects for the current panel
	//NOTE this function will be removed later
    **/
    void saveObjects(QDomDocument &writer);

    /**
	get the name of the panel
	@return name of the widget
    **/
    QString name(void);
    
    /**
	returns the layer ID for the current object
    **/
    int getLayerID();

    /**
	Add a new object to the selection widget
	@PARAM newTile = the tile object to add
    **/
    void addObject(KGTile* newObject);

    /**
	unselects all objects within the imageview control
	TODO: implimented this function
    **/
    void selectNone(void);

    /**
	assigns a container to the toolbox which contains all the objects for this widget
	KGContiner = container object
    **/
    void addDefaultObjects(void);

private slots:
    /** 
	This is called when the imageView's seletion is changed
	@PARAM item = the new selected object
    **/
    void processSelectionChanged(QGraphicsItem *item);

    /** 
	Displays a dialog box for the user to select a image which
	it converts to a tile object
    **/
    void addTileFromFile(void);
};

#endif
