/*
 * developer.h
 *
 * Copyright (C) 2007 Keith Neuse <KNeuse@Gmail.com>
 */ 
#ifndef _DEVELOPER_H_
#define _DEVELOPER_H_

#include <KMainWindow>
#include <KPushButton>
#include <QApplication>
#include <KStandardDirs>
#include <KConfig>
#include <QDockWidget>
#include <QToolBox>
#include <KTabWidget>
#include <QLabel>
#include <QGraphicsView>
#include <QMainWindow>
#include <KAction>
#include <KXmlGuiWindow>
#include <KActionCollection>
#include <KStandardAction>
#include <QMessageBox>
#include <KToggleAction>
//#include <KStdAction>
#include <KGuiItem>
#include <KLocale>

#include "KGLevelEditor.h"
#include "KGTabInterface.h"
#include "KGSelectionInterface.h"
#include "KGTabInterface.h"
#include "KGToolBox.h"
#include "KGEraser.h"
#include "KGContainer.h"
#include "KGNewGameWizard.h"
#include "KGGameData.h"
#include "KGProject.h"

/** 
    @author Keith Neuse<KNeuse@Gmail.com>
    This object is the main MDI window
**/
class DEVELOPER: public KXmlGuiWindow
{
    Q_OBJECT
public:
    /**
	constructor
    **/
    DEVELOPER(QWidget *parent=0, Qt::WindowFlags flags=0 );
    
    /**
	deconstructor
    **/
    virtual ~DEVELOPER();
    
protected:
    KToggleAction *eraserAction;
    KToggleAction *tileLayerAction;
    KToggleAction *objectLayerAction;
    KGContainer container;
    KGGameData *gameData;
    QDockWidget *toolBoxDock;
    QDockWidget *projectDock;
    KGNewGameWizard *newGameWizard;
    KTabWidget *tabWidget;
    KGToolBox *toolBox;
    KGProject *project;
    int levelNumber;
    KGEraser *eraserObject;
    QList<KGLevelEditor *> levelList;

    /**
	loads any special tiles like the eraser tile
    **/
    void loadTiles(void);
    /**
	creates the layer actions for switching between layers
    **/
    void initLayerToolbar(void);
    /**
	create the other actions such as loading/saving levels
    **/
    void initMainActions(void);

    KGObject *getSelectedObject();

    void setupGame(KGGameData *gameData);
public slots:
    /**
	Startes the wizard to create a new game
    **/
    void newGame(void);
protected slots:
    void openGame(void);
    void changeLayerID(int layerID);
    /**
	when a level is renamed through the treeview, this functions handles renaming the tab
    **/
    void renameTab(QString oldReference, QString newReference, QString newName);
    /**
	When a level object is selected within a tree view this functions handles selecting the correct tab
	corresponding to the level
    **/
    void selectTab(QString reference);
    /**
	After the wizard is completed, this function sets up the treeview control along with a new level
    **/
    void createNewGame(void);
    /**
	loads a level from a file
    **/
    void openLevel(void);
    /**
	save a level to a file
    **/
    void saveLevel(void);
    /**
	saves the current level. Note: not implemented yet
    **/
    void saveLevelAs(void);
    /**
	this is called when the user switches to tile layer
    **/
    void selectTileLayer(bool state);
    /**
	this is called when the user switches to object layer
    **/
    void selectObjectLayer(bool state);
    /**
	this is called when the user toggles erasor mode
    **/
    void eraserModeChanged(bool mode);
    void setEraserMode(bool mode);
    /**
	adds a new level to the current project
    **/
    void newLevel();
    /**
	This function closes the current tab
    **/
    void closeLevel();
};

#endif
