#include "kollagame.h"

#include <KDebug>

Kollagame::Kollagame(QWidget *parent) :
KXmlGuiWindow(parent)
{
    graphicsEngine = new KGGraphicsEngine(this);
    setCentralWidget(graphicsEngine);

    KStandardAction::quit(qApp, SLOT(quit()), actionCollection());

    QAction *action = NULL;

    //new level
    action = new KAction(KIcon("document-open"), i18n("&Open Level"), this);
    actionCollection()->addAction("open_level", action);
    connect(action, SIGNAL(triggered(bool)), this, SLOT(openLevel()));

    QString resourceLocation = KGlobal::dirs()->findResource("data", "kollagame/kollagame/kollagame.rc");
    setupGUI(ToolBar | Create, resourceLocation);
}

void Kollagame::openLevel()
{
    KUrl filename = KFileDialog::getOpenFileName(KUrl(),"*",this,"Open a level");
    if(filename == NULL)
    {
	kDebug() << "not loadding" << endl;;
	return;
    }
    
    QFile openLevel(filename.path());
    QDomDocument doc;

    openLevel.open(QIODevice::ReadWrite);
    doc.setContent(&openLevel);
    openLevel.close();
    QDomElement element = doc.documentElement();
    QDomNodeList elementList = element.childNodes();
    
    int maxWidth = 0;
    int maxHeight = 0;

    kDebug() << "Loading..." << endl;
    for(unsigned int loop = 0; loop < elementList.length(); loop++)
    {
	QDomElement objectElement = elementList.item(loop).toElement();
	KGObject *object = container.findObject(objectElement.attribute("reference",QString()));
	if(!object)
	{
	    kDebug() << "failed to load object" << endl;
	    continue;
	}
	QDomElement locationElement = objectElement.firstChildElement("location");
	QString x = locationElement.attribute("x",QString());
	int xSize = x.toInt();
	if(xSize > maxWidth)
	    maxWidth = xSize;
	QString y = locationElement.attribute("y",QString());
	int ySize = y.toInt();
	if(ySize > maxHeight)
	    maxHeight = ySize;
	
	graphicsEngine->addObject(object,QPoint(xSize,ySize));
    }
    graphicsEngine->setSize(maxWidth,maxHeight);
}

Kollagame::~Kollagame()
    {}

void Kollagame::resizeEvent(QResizeEvent *event)
{
     //graphicsEngine->resize(event->size());
}

#include "kollagame.moc"
