#ifndef _KGOBJECT_H_
#define _KGOBJECT_H_

#include <QGraphicsItem>
#include <QDomDocument>
#include <QDomElement>

class KGObject: public QGraphicsItem
{
protected:
    QString varReference;
    QString varName;
public:
    KGObject(QGraphicsItem *parent = 0);

    /**
	TODO: the KGLevelEditor needs to be updated to call this function
	This function is called by KGLevelEditor when the user attempts to edit the item
	objects that inheret this class need to implement a dialog to input data in order
	to alter the settings of the object
    **/
    virtual void editObject(void) = 0;
    
    /**
	The object creates its element for the xml file, this function is called through
	the KGContianer class while saving the tile
	@return QDomElement new element for the object
    **/
    virtual QDomElement saveObject(QDomDocument &doc) = 0;

    /**
	This function is responsible for loading all the various data for the object
	from the XML file
    **/
    virtual void loadObject(QDomElement &element) = 0;

    /**
	this function duplicates the object, this is primarly used through the KGLevelEditor
	object to duplicate the object for placing it on the level
    **/
    virtual KGObject *clone() = 0;
    
    /**
	this is called when the object is going to merge with another object,
	This allows for objects to be created with drag and drop actions
    **/
//    virtual KGObject *merge(KGObject *otherObject) = 0;

    /**
	this is used by KGContainer in order to locate the item, this is needed because
	objects can have the same name while being placed in different categories
	@param argReference = the object to merge with
    **/
    void setReference(QString argReference);

    /**
	get the reference string
	@return the reference string
    **/
    QString reference();

    /**
	set the name for the object
	@PARAM argName = the new name
    **/
    void setName(QString argName);

    /**
	get the name for the object
	@return the name of the object
    **/
    QString name();
};

#endif
