#ifndef _KGTILE_H_
#define _KGTILE_H_

#include <QGraphicsItem>
#include <KStandardDirs>
#include <QPainter>
#include <QSharedData>

#define TILE_SIZE 50

#include "KGObject.h"
 
/** 
    @author Keith Neuse<KNeuse@Gmail.com>
    This class creates tile objects which can be displayed in the level editor 
**/
class KGTile : public QSharedData, public KGObject
{
public:

    KGTile(QPixmap &getImage, QString name = QString(), QGraphicsItem *parent = 0);
    /** copy constructor
	@param argImage = the iamge to use for creating the tile
	@param name = the name for the tile
    **/
    KGTile(QString picLocation, QString name = QString(), QGraphicsItem *parent = 0);

    /** copy constructor
	@param argImage = the tile to duplicate
	@param name = the name for the tile
    **/
    KGTile(KGTile *tileObject, QString name = QString(), QGraphicsItem *parent = 0);

    /** copy constructor for creating an empty tile **/
    KGTile();

    /** deconstructor **/
    virtual ~KGTile();

    /** 
	This informs the level editor the dimensions of the tiles 
	@return a QRectF object with the dimensions
    **/
    QRectF boundingRect() const;

    /**
	changes the current image for the tile
	@param argImage = the new image
    **/
    void setImage(QPixmap &argImage);

    QString getPicLocation(void);

    /**
	get the current image
	@return the QPixmap object with the image
    **/
    QPixmap &getImage(void);

    void editObject();
    QDomElement saveObject(QDomDocument &doc);
    void loadObject(QDomElement &element );
    KGObject *clone();
protected:
    QString picLocation;
    QPixmap image;

    void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget);
};

#endif
