/***************************************************************************
 *   Copyright (C) 2005 by Albert Astals Cid <tsdgeos@terra.es>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef BOARD_H 
#define BOARD_H 

#include <QPair>
#include <QPoint>
#include <QStringList>

#include <kombination_export.h>
#include "square.h"
#include "tileposition.h"

class dictionary;
class rack;
class letter;

enum FinalizeAction
{
	ACCEPT = 0,
	DELETE
};

typedef QList<QPair<QString, tilePositionList> > wordsAndTilePositions;

class KOMBINATION_EXPORT board
{	
	public:
		board();
		~board();
		bool checkValidCross( Orientation orientation, int x, int y ) const;
		wordsAndTilePositions checkValidTilesPositions(int &score, int &error) const;
		tilePositionList getWord(int x, int y, Orientation align) const;

		//void addAndLockTiles(const tilePositionList &tiles);
		void addCrossPoints(const uint score, int x, int y);
		void updateCrossLetters(int x, int y, Orientation orientation);

		//taken from boardwidget
		void setTileStringList(const QList< QPair<QString, int> > &tileStrings);

		//hack
		void setDictionary(dictionary *dict);

		/*!
		 * Saves tiles used in move by any player. If word is not correct then they can be deleted. 
		 * removes tiles from m_playedTiles and adds new ones, needs confirmation with 
		 * finalizePlayedTiles(ACCEPT). It won't add tiles that overlap with tiles that
		 * are on board and locked.
		 */
		void setPlayedTiles(const tilePositionList &tiles);

		/*!
		 * Returns tiles used in move by any player.
		 */
		const tilePositionList& getPlayedTiles();

		/*!
		 * clear m_playedTiles or add them pernamently to board, counts cross points and anchors.
		 */
		void finalizePlayedTiles(const FinalizeAction act);

		void dumpBoard();
		QList<QList<tilePosition*> *> * getPossibleMoves(QList<tile*> &tiles);
		int pointsForMove(QList<tilePosition*> &tiles);

		//AI
		bool isLocked(int x, int y) { return m_squares[x][y].isLocked(); }

		/*!
		 * Deletes tiles, resets anchors, locks etc. Used before next game.
		 */
		void reset();
	protected:
		square m_squares[15][15];
		int m_numFixedTiles;
		tilePositionList m_playedTiles;
		QList< QPair<QString, int> > m_tilesStringList;
		dictionary *m_dictionary;
	private:
		int findMaxLeftSize(int x, int y, Orientation orientation, bool &leftSideOnBoard, int maxSize);
		void leftPart(QList<tilePosition> &partialWord, rack* playerRack, int nodeIndex, letter* letters, int x, int y, Orientation orientation, QList<QList<tilePosition*> *> *wordList, int maxSize);
		void extendRight(QList<tilePosition> &partialWord, rack* playerRack, int nodeIndex, letter* letters, int x, int y, Orientation orientation, QList<QList<tilePosition*> *> *wordList, bool tileOnAnchor);
};

#endif
