/***************************************************************************
 *   Copyright (C) 2005-2007 by Albert Astals Cid <aacid@kde.org>          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef GAME_H
#define GAME_H

#include <sonnet/speller.h>

#include "board.h"
#include "replacementshandler.h"
#include "tilebox.h"
#include "tileposition.h"

class aiPlayer;
class player;
class spectator;
class dictionary;

class game : public QObject
{
Q_OBJECT
	public:
		game();
		game(int id, const QString &name, int maxPlayers, const QString &comment, dictionary *dict, bool allowSpectators);
		~game();
		
		spectator *addPlayer(spectator *s, bool joinAsSpectator);
		void addAi(const QString &name, const QString &languageCode);
		
		int id() const;
		QString name() const;
		int numberOfPlayers() const;
		int numberOfSpectators() const;
		int maxNumberOfPlayers() const;
		bool hasStarted() const;
		bool allowsSpectators() const;
		QString comment() const;
		QString language() const;
		
		QMap<QString, QString> toMap(const QString &tag) const;

	private slots:
		void checkCanStartGame(player *p);
		void gotTiles(player *p, const tilePositionList &tiles);
		void checkVotingResult();
		void incorrectWordTimeout();
		void passTurn(player *p);
		void exchangeTiles(player *p, const QList<tile*> &tiles);
	
	private:
		void nextTurn();
		void gameEnded(player *p);
		
		void addPlayerHelper(spectator *s);
	
		QList<player *> m_players;
		QList<spectator *> m_spectators;
		QList<aiPlayer *> m_aiPlayers;
	
		int m_id;
		QString m_name;
		int m_maxPlayers;
		QString m_comment;
	
		tileBox m_tileBox;
		int m_currentPlayer;

		dictionary *m_dict;

		replacementsHandler m_rh;
		QStringList m_exclusiveTileList;

		board m_board;
		bool m_incorrectWordAskingTimeouted;

		bool m_allowSpectators;
	
		int m_scoreLastWord;
	
		bool m_started;
};

#endif
