/****************************************************************************
 *   Copyright (C) 2005 by Albert Astals Cid <tsdgeos@terra.es>             *
 * Copyright (C) 2007-2008 by Michal Sciubidlo <michal.sciubidlo@gmail.com> *
 *                                                                          *
 *   This program is free software; you can redistribute it and/or modify   *
 *   it under the terms of the GNU General Public License as published by   *
 *   the Free Software Foundation; either version 2 of the License, or      *
 *   (at your option) any later version.                                    *
 ****************************************************************************/
#ifndef NEWKOMBINATION_H
#define NEWKOMBINATION_H

#include <kxmlguiwindow.h>
#include "log.h"

class KombinationScene;
class KombinationView;
class QGraphicsView;
class QPushButton;
class chatWidget;
class gameListWidget;
class multiWidget;
class client;
class server;
class tileWidget;
class dictionary;
class boardWidget;
class tileWidget;

class Kombination : public KXmlGuiWindow
{
Q_OBJECT

public:
	Kombination();

	~Kombination();

public slots:
	void newGame();
	void acceptedIntoGame(const QString &languageCode);
	void clientError(const QString &text);
	void serverError(const QString &text);
	void showErrorBox(const QString &text, const QString &caption);
	void serverStarted();

	void startGame();
	void buttonPressed();
	void nextPlayer(const QString &, bool);
	void gameEnded();
	void serverDown();

	void ignorePlayers();
	void showTilesScores();
	void passTurn();
	void viewLog();
	void confirmReject(int, const QString &);

	void addScoreCurrentPlayer(int score);

	void enteredAMultiServer(const QStringList &playerList);
	void createServerGame();
	void joinServerGame();

private:
	void createClient(const QString &server, int port, const QString &nick, bool joinAsSpectator);
	void closeConnection();
	void disableServer();

	KombinationView * view;
	KombinationScene * scene;

	boardWidget *m_board;
	tileWidget *m_tileWidget;
	tileWidget *m_exchangeWidget;

	chatWidget *m_chatWidget;
	multiWidget *m_multiWidget;
	client *m_client;
	server *m_server;
	QPushButton *m_readyButton;
	QPushButton *m_passButton;
	gameListWidget *m_gameListWidget;

	int m_port;
	QString m_nick;
	bool m_ready;

	QAction *m_ignorePlayersAction;
	QAction *m_tilesScoresAction;
	QAction *m_viewLogAction;

	log m_scoresLog;

};

#endif
