/***************************************************************************
 *   Copyright (C) 2005 by Albert Astals Cid <tsdgeos@terra.es>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "log.h"

#include <klocale.h>

logEntry::logEntry(const QStringList &words, int score) :
	m_words(words), m_score(score)
{
}

QString logEntry::representation() const
{
	// TODO would be nice to have that in two lines but seems we have to reimplement the
	// delegate that is doing the drawing and seems two difficult for now :D
	return m_words.join(", ") + ' ' + i18np("1 Point","%1 Points",m_score);
}


log::log()
{
}

log::~log()
{
	clean();
}

void log::start(const QStringList &players)
{
	clean();
	foreach(const QString &nick, players)
		m_scores.append(QPair< QString, QList<logEntry*> >(nick, QList<logEntry*>()));
}

void log::clean()
{
	QList< QPair< QString, QList<logEntry*> > >::const_iterator it, itEnd;
	it = m_scores.constBegin();
	itEnd = m_scores.constEnd();
	for ( ; it != itEnd; ++it)
		qDeleteAll((*it).second);
	m_scores.clear();
}

void log::addScore(const QString &nick, const QStringList &words, int score)
{
	QList< QPair< QString, QList<logEntry*> > >::iterator it, itEnd;
	it = m_scores.begin();
	itEnd = m_scores.end();
	for ( ; it != itEnd; ++it)
	{
		if ((*it).first == nick)
		{
			(*it).second.append(new logEntry(words, score));
			break;
		}
	}
}

int log::rowCount(const QModelIndex &/*index*/) const
{
	int maxRows = 0;
	QList< QPair< QString, QList<logEntry*> > >::const_iterator it, itEnd;
	it = m_scores.begin();
	itEnd = m_scores.end();
	for ( ; it != itEnd; ++it)
		maxRows = qMax(maxRows, (*it).second.count());
	return maxRows;
}

int log::columnCount(const QModelIndex &/*index*/) const
{
	return m_scores.size() + 1;
}

QVariant log::data(const QModelIndex &index, int role) const
{
	if (!index.isValid()) return QVariant();
	
	int column, row;
	column = index.column();
	row = index.row();
	
	if (role != Qt::DisplayRole) return QVariant();
	
	if (column == 0) return i18n("Turn %1",row + 1);

	const QPair<QString, QList<logEntry*> > &p = m_scores.at(column - 1);
	if (p.second.count() > row)
	{
		const logEntry *entry = p.second.at(row);
		return entry->representation();
	}
	else return QVariant();
}

QVariant log::headerData(int section, Qt::Orientation /*orientation*/, int role) const
{
	if (role != Qt::DisplayRole) return QVariant();
	
	if (section == 0) return QVariant();
	return m_scores.at(section - 1).first;
}
