/***************************************************************************
 *   Copyright (C) 2007 by Albert Astals Cid <aacid@kde.org>               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "messagehandler.h"

#include <QDomDocument>
#include <QTcpSocket>

#include <kdebug.h>

void messageHandler::setSocket(QTcpSocket *socket)
{
	m_stream.setDevice(socket);

	connect(socket, SIGNAL(readyRead()), this, SLOT(readFromSocket()));
}

void messageHandler::sendMessage(const message &m)
{
	QString attributes;
	QMap<QString, QString>::const_iterator dataIt, dataItEnd;
	dataIt = m.data.begin();
	dataItEnd = m.data.end();
	for ( ; dataIt != dataItEnd; ++dataIt )
	{
		attributes += QString("%1=\"%2\" ").arg(dataIt.key()).arg(quote(dataIt.value()));
	}
	QString mString = QString("<%1 %2/>").arg(m.type).arg(attributes);
	kDebug() << "Sending message " << mString << endl;
	m_stream << mString << endl;
}

void messageHandler::readFromSocket()
{
	QString aux = m_stream.readLine();
	while( !aux.isNull() )
	{
		kDebug() << "Receiving message " << aux << endl;
		QDomDocument doc;
		doc.setContent(aux);
		message m;
		m.type = doc.documentElement().tagName();
		const QDomNamedNodeMap &attributesMap = doc.documentElement().attributes();
		for (int i = 0; i < attributesMap.count(); ++i)
		{
			QDomAttr attr = attributesMap.item(i).toAttr();
			if (!attr.isNull()) m.data.insert(attr.name(), attr.value());
		}
		emit newMessage(m);
		if (m_stream.atEnd()) aux.clear();
		else aux = m_stream.readLine();
	}
}

QString messageHandler::quote(const QString &string) const
{
	QString result = string;
	result.replace('\"', "&quot;");
	return result;
}

#include "messagehandler.moc"
