/***************************************************************************
 *   Copyright (C) 2005-2007 by Albert Astals Cid <aacid@kde.org>          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef SERVER_H
#define SERVER_H

#include <QAbstractTableModel>
#include <QList>
#include <QMap>

#include <kombination_export.h>

class game;
class spectator;
class dictionary;
class QTcpServer;

class KOMBINATION_EXPORT server : public QAbstractTableModel
{
Q_OBJECT
	public:
		enum eKindOfServer{ MULTISERVER, SINGLESERVER };
		
		server(int maxPlayers, eKindOfServer type);
		~server();
		
		void start(int port);
		
		int rowCount(const QModelIndex &index) const;
		int columnCount(const QModelIndex &index) const;
		QVariant data(const QModelIndex &index, int role) const;
		QVariant headerData(int section, Qt::Orientation orientation, int role) const;
	
	public slots:
		void createGame(spectator *creator, const QString &name, int maxPlayers, const QString &comment, const QString &languageCode, bool allowSpectators, int aiCount = 0);
	
	signals:
		void connected();
		void error(const QString &error);
	
	private slots:
		void slotReadyAccept();
		void slotSocketError(int e);

		void wantToJoin(spectator *s);
		void addPlayer(spectator *p);
		void addPlayerToGame(spectator *s, int gameId, bool joinAsSpectator);
		void sendMessage(spectator *p, const QString &message);
		void removePlayer(spectator *s);

	private:
		game *findGameWithName(const QString &name) const;
		
		QTcpServer *m_serverSocket;
		int m_gameId;
		QList<game *> m_games;
		QList<spectator *> m_connectedClients;
		int m_maxPlayers;
		eKindOfServer m_type;
		QMap<QString, dictionary*> m_dictionaries;
};

#endif
