/***************************************************************************
 *   Copyright (C) 2005-2007 by Albert Astals Cid <aacid@kde.org>          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef SPECTATOR_H
#define SPECTATOR_H

#include <QList>
#include <QPair>
#include <QAbstractSocket>

#include "tileposition.h"

class game;
class message;
class messageHandler;
class tile;
class QTcpSocket;

/**
 * This is the representation of a spectator on the server side 
 */ 
class spectator : public QObject
{
Q_OBJECT
	
	friend class player;

	public:
		spectator(QTcpSocket *socket);
		spectator(spectator *s);
		spectator();
		~spectator();
		
		void unsetSocket();
		void deleteSocket();
	
		bool isSpectator();

		void askForNick();
		void tooManyPlayers();
		
		const QString &nick() const;
		bool wasNickInUse() const;
		
		void setCurrentGame(game *g);
		game *currentGame() const;
		
		void acceptedInServer(const QStringList &players, QList<game *> games);
		void addGame(game *g);
		void canNotCreateGame(const QString &reason);

		void acceptedInGame(const QList< QPair<QString, int> > &players, const QString &language);
		void nickInUse();
		void doesntAllowSpectators();
		void sendMessage(const QString &nick, const QString &message);
		void startGame();
		void sendNewTiles(const QList<tile *> &tiles);
		void addPlayer(const QString &nick, bool isPlayer);
		void addPlayer(const QString &nick);
		void otherPlayerEnteredAGame(const QString &nick, int gameId);
		void nextTurn(const QString &nick);
		void sendTiles(const tilePositionList &tiles);
		void correctWord(int score);
		void unknownWords();
		void gameEnded(const QString &finisher, const QString &winner);
		void removePlayer(const QString &nick);
		void rejectVotedTiles(const QString &nick);
		void usesExclusiveTiles(const QString &nick);
		void sendStatus(const QString &nick, int which);
		void passedTurn(const QString &nick);
		void exchangedTiles(const QString &nick, int nTiles);

	signals:
		void wantToJoin(spectator *s);
		void joinGame(spectator *s, int gameId, bool joinAsSpectator);
		void gotNick(spectator *s);
		void gotMessage(spectator *s, const QString &message);
		void createServerGame(spectator *s, const QString &name, int maxPlayers, const QString &comment, const QString &languageCode, bool allowSpectators);
		void remotelyDisconnected(spectator *s);
	
	protected slots:
		virtual void processNewMessage(const message &m);
		
		virtual void slotSocketError(QAbstractSocket::SocketError error);

	protected:
		void handleMessage(const message &m);
		
		QString m_nick;
		messageHandler *m_messageHandler;
	
		enum Phase {WAITING_FOR_SPECTATOR, WAITING_FOR_NICK, NICK_ALREADY_IN_USE, WAITING_TO_START_GAME, PLAYING_GAME, TRADING_PHASE};
		Phase m_phase;

	private:
		game *m_game;
		QTcpSocket *m_socket;
		tilePositionList m_privTiles;
		
		/*! spectator or not - useful only for SINGLESERVER */
		bool m_isSpectator;
};

#endif
