/***************************************************************************
 *   Copyright (C) 2005 by Albert Astals Cid <tsdgeos@terra.es>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef SQUARE_H
#define SQUARE_H 
#include <kombination_export.h>
#include <QBitArray>

class tile;
#define MAX_LETTERS_NUM 63

enum Orientation
{
	HORIZONTAL = 0,
	VERTICAL = 1
};

class KOMBINATION_EXPORT square
{
	public:
		square();
		~square();	
		
		enum Kind
		{
			NORMAL,
			CENTER,
			DOUBLE_WORD,
			TRIPLE_WORD,
			DOUBLE_LETTER,
			TRIPLE_LETTER
		};

		enum State
		{
			EMPTY,
			ANCHOR,
			TILE
		};

		void setTile(tile *t);
		tile* ourTile();
		const tile* ourTile() const;

		void setLocked(bool lock);
		bool isLocked() const;

		void setKind(Kind k);
		Kind kind() const;

		int horizontalPoints() const;
		int verticalPoints() const;
		void addHorizontalPoints(int points);
		void addVerticalPoints(int points);

		void turnOnCrossLetter(int index, Orientation orientation);
		void turnOffCrossLetter(int index, Orientation orientation);
		bool crossLetterState(int index, Orientation orientation);
		QBitArray crossLettersState(Orientation orientation) { return possibleLetters[orientation]; }

		//void setState(State s) { m_state = s; }
		State state() const;
		void setToAnchor();
		bool isAnchor() const;
		void setToTile();
		void reset();
	private:
		Kind m_kind;
		tile *m_tile;
		bool m_locked;
		int m_nearPoints[2];
		State m_state;
		QBitArray possibleLetters[2];
};

#endif
