/***************************************************************************
 *   Copyright (C) 2005-2007 by Albert Astals Cid <aacid@kde.org>          *
 *   Copyright (C) 2005 by Pino Toscano      <toscano.pino@tiscali.it>     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef TILE_H
#define TILE_H

#include <QMap>
#include <QString>

#include <kdebug.h>
#include <kombination_export.h>

class KOMBINATION_EXPORT tile
{
	public:
		enum Kind
		{
			NORMAL = 0,
			HIGHLIGHT
		};

		tile();
		tile(const QString &s);
//		~tile();

		void setString(const QString &s);
		QString string(bool spaceIfEmpty = false) const;

		void setBlank(bool b);
		bool isBlank() const;

		void setScore(const uint score);
		uint score() const;

		void setKind(const tile::Kind k);
		tile::Kind kind() const;

		bool operator==(const tile &t) const;
		
		QMap<QString,QString> toMap(const QString &tag) const;
		void fromMap(const QMap<QString,QString> &data, const QString &tag);

	private:
		void init(const QString &s);

		QString m_string;
		uint m_score;
		Kind m_kind;
		bool m_isBlank;
		bool m_isBlankSet;
};

class KOMBINATION_EXPORT rack
{
	public:
		rack(QList<tile*> &tiles);
		bool contains(const QString &letter);
		tile* removeLetter(const QString &letter);
		void putLetter(tile* newTile);
		bool started() { return m_tiles.count() != maxCount; }
	private:
		QList<tile*> m_tiles;
		int maxCount;
};

template<template<typename U> class K>
QMap<QString,QString> toMap(const K<tile *> &list)
{
	QMap<QString,QString> result;
	//TODO: passing tile list with null ones
	int tCount = 0;
	for (int i = 0; i < list.count(); ++i)
	{
		if (list[i] != 0) tCount++;
	}

	result.insert("tiles", QString::number(tCount));
	for (int i = 0; i < list.count(); ++i)
	{
		if (list[i] != 0)
			result.unite( list[i]->toMap(QString("tile%1").arg(i)) );
	}
	return result;
}

template<template<typename U> class K>
void fromMap(const QMap<QString,QString> &data, K<tile *> &result)
{
	int tiles = data["tiles"].toInt();
	for (int i = 0; i < tiles; ++i)
	{
		tile *t = new tile();
		t->fromMap(data, QString("tile%1").arg(i));
		result.append( t );
	}
}

#endif
