/***************************************************************************
 *   Copyright (C) 2005 by Albert Astals Cid <tsdgeos@terra.es>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "tilebox.h"

#include <stdlib.h>

#include <krandom.h>
#include <kdebug.h>
tileBox::tileBox()
{
}

void tileBox::newGame(const QList<tile> availableTiles)
{
	m_availableTiles = availableTiles;
	shuffle();
}

tile *tileBox::getTile()
{
	int r = (int)((double)m_availableTiles.count() * KRandom::random() / (RAND_MAX + 1.0));
	tile *t = new tile(m_availableTiles[r]);
	m_availableTiles.removeAt(r);
	return t;
}

QList<tile*> tileBox::getTiles(int num)
{
	QList<tile*> tiles;
	if (num > m_availableTiles.count()) num = m_availableTiles.count();
	for (int i = 0; i < num; i++)
		tiles << getTile();
	return tiles;
}

void tileBox::addTiles(const QList<tile*> &tiles)
{
	foreach(tile *t, tiles)
		m_availableTiles.append(*t);
}

bool tileBox::isEmpty()
{
	return m_availableTiles.empty();
}

void tileBox::shuffle()
{
	for (int times = 0; times < 3; times++)
	{
		for (int i = 0; i < m_availableTiles.count()/2; i++)
		{
			int r = (int)((double)m_availableTiles.count() * KRandom::random() / (RAND_MAX + 1.0));
			tile t = m_availableTiles[r];
			m_availableTiles.removeAt(r);
			m_availableTiles.insert(0, t);
		}
	}
}

void tileBox::removeTile(const tile* const tile)
{
	for (int i = 0; i < m_availableTiles.count(); i++)
	{
		if ((m_availableTiles[i].string() == tile->string()) || (m_availableTiles[i].isBlank() == tile->isBlank() == true))
		{
			m_availableTiles.removeAt(i);
			break;
		}
	}
}

void tileBox::removeTiles(const QList<tile*> &tiles)
{
	foreach (tile *t, tiles)
		removeTile(t);
}
