/***************************************************************************
 *   Copyright (C) 2005 by Albert Astals Cid <tsdgeos@terra.es>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "tilesscoresdialog.h"
#include "tile.h"
#include "renderer.h"

#include <QLabel>
#include <QPainter>
#include <QScrollArea>
#include <QPixmap>

class dummyTile : public QWidget
{
	public:
		dummyTile(QWidget *parent, tile *t, const QPixmap &pixmap) : QWidget(parent), m_tile(t)
		{
			setFixedSize(30, 30);
			pix = pixmap;
		}

		~dummyTile()
		{
			delete m_tile;
		}

	protected:
		void paintEvent(QPaintEvent *)
		{
			QPainter p(this);
			p.drawPixmap(0, 0, 32, 32, pix);
			p.drawText(0, 0, 32, 32, Qt::AlignCenter, m_tile->string());
		}
	
	private:
		tile *m_tile;
		QPixmap pix;
};

tilesScoresDialog::tilesScoresDialog(QWidget *parent, const QList< QPair<QString, int> > &tileStringList) : QDialog(parent)
{
	setupUi(this);
	QVBoxLayout *vbl = new QVBoxLayout(mainWidget);
	QScrollArea *sa = new QScrollArea(mainWidget);
	vbl->addWidget(sa);

	QWidget *container = new QWidget(mainWidget);
	sa->setWidget(container);
	sa->setWidgetResizable(true);
	
	QGridLayout *gl = new QGridLayout(container);
	QPair<QString, int> p;
	int row = 0;
	for(int i = 0; i < tileStringList.size(); i++)
	{
		tile *t = new tile(tileStringList[i].first);
		dummyTile *dt = new dummyTile(container, t, renderer::self()->tileBackground());
		QLabel *l = new QLabel(container);
		l->setText(i18np("%1 point", "%1 points", p.second));
		gl->addWidget(dt, row, 0);
		gl->addWidget(l, row, 1);
		++row;
	}
	--row;
	container->setMinimumSize(0, 30 * row + gl->spacing() * (row + 1) + gl->margin() * 2);
}

