/**************************************************************************
*   Copyright (C) 2005 by Lukas Tinkl                                     *
*   lukas@kde.org                                                         *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.             *
***************************************************************************/

#ifndef _GAME_H_
#define _GAME_H_

#include "gameManager.h"

#include <qdom.h>

class QString;

/**
 * Class holding info about an individual type of game (server).
 * It is not meant to be instantied by itself, use GameManager to get
 * relevant info about available/supported games.
 * @short Game class
 */
class Game
{
    friend class GameManager;
public:
    /**
     * CTOR
     */
    Game() { };

    /**
     * Construct the game from QDomElement @p e
     */
    Game( const QDomElement & e );

    /**
     * @return unique ID of the game
     */
    QString id() const;

    /**
     * @return user visible game name
     */
    QString name() const;

    /**
     * @return command used to launch the game client
     */
    QString exec() const;

    /**
     * @return the type string for executing the game client (game itself)
     * This decides what command line options to pass to the binary
     */
    QString execClientType() const;

    /**
     * @return CLI option used to query the server
     */
    QString serverOption() const;

    /**
     * @return CLI option used to query the master server
     */
    QString masterServerOption() const;

    /**
     * @return the master server address
     */
    QString masterServer() const;

    /**
     * @return game application pixmap (icon)
     */
    QPixmap gameIcon() const;

    /**
     * @return the base game (ie the parent game)
     */
    QString baseGame() const;

    /**
     * Compare the name of this game
     */
    bool operator< ( const Game & rhs );

protected:
    /**
     * DOM element holding the game definition
     */
    QDomElement m_dom;

    /**
     * Helper function to retrieve a specific DOM node called @p name
     */
    QString getNode( const QString & name ) const;
};

#endif
