/**************************************************************************
*   Copyright (C) 2005 by Lukas Tinkl                                     *
*   lukas@kde.org                                                         *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.             *
***************************************************************************/

#ifndef _PLAYER_H_
#define _PLAYER_H_

#include <klistview.h>

class QString;

/**
 * Class combining data storage and a listview item
 * representing a player
 * @short Player class
 */
class Player: public KListViewItem
{
public:
    Player( QListView * parent ):
        KListViewItem( parent ) { }

    Player( QListViewItem * parent ):
        KListViewItem( parent ) { }

    Player( QListView *parent, QString name ):
        KListViewItem( parent, name ), m_name( name ) { }

    Player( QListViewItem *parent, QString name ):
        KListViewItem( parent, name ), m_name( name ) { }

    /**
     * @return the player's name
     * (stripped of font markup)
     */
    QString name() const;

    /**
     * Set the player's name
     * (accepts HTML <font> markup)
     */
    void setName( const QString & name )
        { m_name = name; }

    /**
     * @return the number of player's frags (kills)
     */
    int frags() const
        { return m_frags; }

    /**
     * Set the number of player's frags (kills)
     */
    void setFrags( int frags )
        { m_frags = frags; }

    /**
     * @return the player's connection ping
     */
    int ping() const
        { return m_ping; }

    /**
     * Set the player's connection ping
     */
    void setPing( int ping )
        { m_ping = ping; }

protected:
    virtual QString text( int col ) const;
    virtual int compare( QListViewItem * i, int col, bool ascending ) const;
    virtual void paintCell( QPainter * p, const QColorGroup & cg, int column, int width, int align );
    virtual int width( const QFontMetrics & fm, const QListView * lv, int c ) const;

private:
    QString m_name;
    int m_frags;
    int m_ping;
};

#endif
