/**************************************************************************
*   Copyright (C) 2005 by Lukas Tinkl                                     *
*   lukas@kde.org                                                         *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.             *
***************************************************************************/

#include "qstatIface.h"
#include "settings.h"

#include <qstring.h>
#include <qstringlist.h>

#include <kprocess.h>
#include <kdebug.h>
#include <kstandarddirs.h>

QStatIface::~QStatIface()
{
}

pid_t QStatIface::getServerList( const QString & gameID )
{
    return callQStat( QStringList() << m_manager->masterServerOption( gameID )
                      << m_manager->masterServer( gameID ) << "-R" );
}

pid_t QStatIface::getServerInfo( const QString & server, const QString & gameID )
{
    return callQStat( QStringList() << m_manager->serverOption( gameID )
                      << server << "-R" << "-P" );
}

pid_t QStatIface::callQStat( const QStringList & args )
{
    m_buffer = "";

    KProcess * proc = new KProcess( this );
    *proc << "qstat";
    *proc << "-cfg" << locate( "data", "kqf/data/qstat.cfg" );

    *proc << args;

    *proc << "-xml" << "-utf8" << "-htmlnames";

    // settings
    if ( Settings::resolveHostnames() )
        *proc << "-H";
    if ( !Settings::displayFullServers() )
        *proc << "-nf";
    if ( !Settings::displayEmptyServers() )
        *proc << "-ne";

    kdDebug() << "Calling qstat with args: " << proc->args()  << endl;

    connect( proc, SIGNAL( processExited( KProcess * ) ),
             this, SLOT( slotProcessExited( KProcess * ) ) );

    connect( proc, SIGNAL( receivedStdout( KProcess *, char *, int ) ),
             this, SLOT( slotRecvdStdout( KProcess *, char *, int ) ) );

    proc->start( KProcess::NotifyOnExit, KProcess::Stdout );

    kdDebug() << "Started proc: " << proc->pid() << endl;

    return proc->pid();
}

void QStatIface::slotRecvdStdout( KProcess *, char * buf, int len )
{
    m_buffer += QString::fromUtf8( buf, len );
}

void QStatIface::slotProcessExited( KProcess * proc )
{
    if ( proc->normalExit() && !m_buffer.isEmpty() && !proc->isRunning() )
    {
        QDomDocument dom;
        dom.setContent( m_buffer );

        emit result( proc->pid(), dom.documentElement() );

        delete proc;
    }
}



#include "qstatIface.moc"
