/****************************************************************
Copyright 2008  Matt Williams matt@milliams.com
Copyright 2008  Parker Coates parker.coates@gmail.com

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) version 3, or any
later version accepted by the membership of KDE e.V. (or its
successor approved by the membership of KDE e.V.), which shall
act as a proxy defined in Section 6 of version 3 of the license.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library.  If not, see <http://www.gnu.org/licenses/>.
****************************************************************/

#ifndef KSCOREBOARDDIALOG_H
#define KSCOREBOARDDIALOG_H

#include <libkdegames_export.h>
#include <kscoreboardbase.h>

#include <KDialog>

/**
 * \class KScoreBoardDialog kscoreboarddialog.h <KScoreBoardDialog>
 *
 * \brief A KDialog based highscore table
 *
 * \since KDE 4.2
 * \author Matt Williams <matt@milliams.com>
 */
class KDEGAMES_EXPORT KScoreBoardDialog : public KDialog, public KScoreBoardBase
{
    Q_OBJECT

    public:
        /**
         * This class will use the information from
         */
        KScoreBoardDialog();
        virtual ~KScoreBoardDialog();

        void setDisplayOptions(const KScoreBoardBase::DisplayOptions & options);
        void switchToGroup(const QString& groupKey);

        /**
         * Calling this function will display the scoreboard reflecting the
         * current status of the singleton KScoreManager::self().
         *
         * If KScoreManager has the AskName option set (the default) then the
         * dialog will ask for the user's name for the most recently added
         * score.
         *
         * \param scoreDisplay whether or not the last added score should be
         *                     highlighted and whether the comment should be shown
         */
    public Q_SLOTS:
        void show();
        void exec();

    protected Q_SLOTS:
        virtual void accept();
        virtual void reject();

    private:
        class KScoreBoardDialogPrivate;
        KScoreBoardDialogPrivate* const d;

        Q_PRIVATE_SLOT(d, void newScore(const QString& groupKey, int rank))

        Q_DISABLE_COPY(KScoreBoardDialog)
};

#endif //KSCOREBOARDDIALOG_H
