/*
Copyright (c) 2006, 2007, Nicolas Roffet, <nicolas-kde@roffet.com>

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU Library General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
*/

#include "ksimilimainwindow.h"



#include <kactioncollection.h>
#include <kexthighscore.h>
#include <klocale.h>
#include <kmenubar.h>
#include <kstandardgameaction.h>
#include <ktoggleaction.h>
#include <ktoolbar.h>


#include "ksimilimainwindowcentralwidget.h"



KSimiliMainWindow::KSimiliMainWindow(QWidget *parent) : KXmlGuiWindow(parent)
{
	setWindowTitle(i18n("KSimili"));


	m_centralWidget = new KSimiliMainWindowCentralWidget(this);
	setCentralWidget(m_centralWidget);


	// Menu Game
	m_actionGameNew = KStandardGameAction::gameNew(this, SLOT(newGame()), actionCollection());
	m_actionGameNew->setToolTip(i18n("Start a new game"));
	m_actionGameNew->setWhatsThis(i18n("Start a new game.<br />If a game is running, it will be canceled."));

	m_actionHighscores = KStandardGameAction::highscores(this, SLOT(showHighscores()), actionCollection());

	m_actionQuit = KStandardGameAction::quit(this, SLOT(close()), actionCollection());

	m_actionPause = KStandardGameAction::pause(this, SLOT(togglePause()), actionCollection());
	m_actionPause->setEnabled(false);
	m_actionPause->setChecked(false);
	m_actionPause->setToolTip(i18n("Pause the game"));
	m_actionPause->setWhatsThis(i18n("Pause the game and hide the board. Note: this <i>costs</i> one second time of the game."));


	// Menu "Move"
	m_actionHint = KStandardGameAction::hint(this, SLOT(hint()), actionCollection());
	m_actionHint->setToolTip(i18n("Highlight a card that is part of a ksimili"));
	m_actionHint->setWhatsThis(i18n("If you cannot find any ksimili, you can get a hint.<br />This hightligts <b>one</b> card of one of the possible combinations. You have to find the 2 other cards by yourself then!<br />If you still cannot find any combination, you can let display a solution with \"<b>solve</b>\"."));

	//TODO
	//m_actionDemo = KStandardGameAction::demo(this, SLOT(demo()), actionCollection());

	m_actionSolve = KStandardGameAction::solve(this, SLOT(solve()), actionCollection());
	m_actionSolve->setToolTip(i18n("Show a possible combination"));
	m_actionSolve->setWhatsThis(i18n("This shows a solution: it hightligts <b>one</b> possible combinations. You can also discover all possible combinations one after each other.<br />Note: If you cannot find any combination, you can also just get a small <b>hint</b> and try to find a solution by yourself."));


	// Menu "Settings"
	m_actionDisplayNumberOfCombinations = new KToggleAction( i18n("&Display total of possible combinations on the board"), actionCollection());
	m_actionDisplayNumberOfCombinations->setShortcut(i18n("Ctrl+D"));
	m_actionDisplayNumberOfCombinations->setCheckable(true);
	m_actionDisplayNumberOfCombinations->setChecked(true);
	m_actionDisplayNumberOfCombinations->setEnabled(true);
	connect(m_actionDisplayNumberOfCombinations, SIGNAL( triggered(bool) ), m_centralWidget, SLOT( activateDisplayPossibleCombinations(bool) ) );


	setupGUI();

	newGame();
}



//
// Slots
//

void KSimiliMainWindow::newGame()
{
	m_actionPause->setEnabled(true);
	m_actionPause->setChecked(false);
	m_centralWidget->newGame(SIZE_BOARD, TIME_TO_PLAY);
}


void KSimiliMainWindow::pause(bool pause)
{
	m_actionPause->setChecked(pause);
	m_centralWidget->pauseGame(pause);
}


void KSimiliMainWindow::togglePause()
{
	pause(m_actionPause->isChecked());
}


void KSimiliMainWindow::gameOver(const int speed)
{
	m_actionPause->setEnabled(false);
	m_actionPause->setChecked(false);
	
	KExtHighscore::ScoreType wonOrLost;
	if (speed>0)
		wonOrLost = KExtHighscore::Won;
	else
		wonOrLost = KExtHighscore::Lost;
	KExtHighscore::Score score(wonOrLost);
	score.setScore((uint) speed);
	//score.setData("total_time", TODO);
	KExtHighscore::submitScore(score, this);
}


void KSimiliMainWindow::showHighscores()
{
	//TODO
	KExtHighscore::show(this);
}


void KSimiliMainWindow::hint()
{
	m_centralWidget->hint();
}


void KSimiliMainWindow::demo()
{
	// TODO
}


void KSimiliMainWindow::solve()
{
	m_centralWidget->solve();
}

#include "ksimilimainwindow.moc"
