/* This file is part of Ktank.
   Created by sacha schutz and olivier gueudelot

   KTank is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation, version 2.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA
*/

#include "ktmaineditor.h"
 #include <QDockWidget>
 #include <KApplication>
#include <KStatusBar>
#include <KToggleAction>
#include <KFileDialog>
#include <KMessageBox>


KTMainEditor::KTMainEditor( QWidget * parent ): KXmlGuiWindow(parent)
{
view        = new KTMapView;
scene       = new KTMapScene(0,0);
itemProperty= new KTItemProperty(this);
itemsList   = new KTItemList(this,scene);



view->setScene(scene);
setCentralWidget(view);

itemsList->setScene(scene);


QDockWidget *dockProp = new QDockWidget(i18n("item property"), this);
dockProp->setAllowedAreas(Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea);
dockProp->setWidget(itemProperty);
addDockWidget(Qt::LeftDockWidgetArea, dockProp); 

QDockWidget *dockTex = new QDockWidget(i18n("texture"), this);
dockTex->setAllowedAreas(Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea);
dockTex->setWidget(itemsList);
addDockWidget(Qt::RightDockWidgetArea, dockTex); 


setupActions();





}
//================================================================
void KTMainEditor::setupActions()
{
KAction* addGrid = new KAction(this);
addGrid->setText(i18n("addGrid"));
addGrid->setIcon(KIcon("go-up"));
addGrid->setShortcut(Qt::CTRL + Qt::Key_W);

actionCollection()->addAction("addGrid", addGrid);

KAction* remGrid = new KAction(this);
remGrid->setText(i18n("remGrid"));
remGrid->setIcon(KIcon("go-down"));
remGrid->setShortcut(Qt::CTRL + Qt::Key_Z);

actionCollection()->addAction("remGrid", remGrid);

KAction* remove = new KAction(this);
remove->setText(i18n("remove"));
remove->setIcon(KIcon("edit-delete"));
actionCollection()->addAction("remove", remove);



KToggleAction *showGrid = new KToggleAction(KIcon("insert-table"),"showGrid", this) ;
actionCollection()->addAction("showGrid",showGrid);	
showGrid->setChecked (true);


connect(addGrid, SIGNAL(triggered(bool)),scene, SLOT(uperGrid()));
connect(remGrid, SIGNAL(triggered(bool)),scene, SLOT(downGrid()));
connect(remove, SIGNAL(triggered(bool)),scene, SLOT(removeItemSelected()));


connect(showGrid, SIGNAL(toggled (bool)), scene, SLOT(showGrid(bool)));


KStandardAction::quit(kapp, SLOT(quit()),actionCollection());

KStandardAction::zoomIn(view,SLOT(zoomIn()),actionCollection()); 
KStandardAction::zoomOut(view,SLOT(zoomOut()),actionCollection()); 
KStandardAction::copy  	(scene,SLOT(copyItems()),actionCollection()); 	
KStandardAction::paste  	(scene,SLOT(pasteItems()),actionCollection()); 	

KStandardAction::cut 	(scene,SLOT(AFAIRE()),actionCollection()); 

	
KStandardAction::open  (this,SLOT(openMap()),actionCollection()); 	
KStandardAction::openNew (this,SLOT(openMap()),actionCollection()); 	
KStandardAction::save (this,SLOT(saveMap()),actionCollection()); 	
KStandardAction::saveAs (this,SLOT(saveMap()),actionCollection()); 	



connect(scene, SIGNAL(selected(KTBlockItem *)), itemProperty, SLOT(setupInfo(KTBlockItem *)));

connect(scene, SIGNAL(refresh()), itemsList, SLOT(update()));





setupGUI();


}
//==================================================================================
void KTMainEditor::saveMap()
{

KUrl path= KFileDialog ::getSaveFileName (KUrl(),"*.ktm",0, "caption");


if ( QFileInfo(path.path()).exists() )
   {



     int ret = KMessageBox::warningContinueCancel(this,
                                          i18n( "The file \"%1\" already exists. Do you wish to overwrite it?" ,
                                            path.path()), i18n( "Overwrite File?" ), KStandardGuiItem::overwrite() );
    if ( ret != KMessageBox::Continue )
     {
       return ;
     }
  }

KTMapFile file;
file.createFile(path,scene);

statusBar()->showMessage (path.fileName());


}
//==================================================================================
void KTMainEditor::openMap()
{
int ret;
if ( scene->items().size() >0)
{
 ret =KMessageBox::warningYesNo(this, i18n( "There are some items on the scene. Create new map will erase all. Are you sure you want create new map ?"), i18n( "Overwrite File?" ));


}
if (ret == KMessageBox::No) return;
scene->clear();

KUrl path= KFileDialog ::getOpenFileName (KUrl(),"*.ktm",0, "caption");


KTMapFile file;
file.openFile(path,scene);

file.readDoc(scene);


itemsList->update();

statusBar()->showMessage (path.fileName());

}
//==================================================================================
void KTMainEditor::openMap(const QString &inputFileName)
{

KUrl url ( inputFileName);


QFile f (url.path());

if ( !f.exists())
{
qDebug()<<"pas de fichier: "<<url.path() ;
return;
}


KTMapFile file;
file.openFile(url,scene);

file.readDoc(scene);


itemsList->update();

statusBar()->showMessage (f.fileName());


}




