/***************************************************************************
 *   Copyright (C) 2003-2007 by Albert Astals Cid                          *
 *   tsdgeos@terra.es                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "boardwidget.h"

#include <kapplication.h>
#include <kdebug.h>
#include <klocale.h>
#include <kmessagebox.h>

#include <qlayout.h>

#include "kcasewidget.h"
#include "settings.h"

boardWidget::boardWidget(QWidget *parent) : board(parent, false)
{
	frame = new QFrame(parent);
	QGridLayout *layout = new QGridLayout(frame);

	for (int i = 0; i < 3; i++)
	{
		for (int j = 0; j < 3; j++)
		{
			cases[i][j] = new kcaseWidget(frame, i, j);
			connect((kcaseWidget*)cases[i][j], SIGNAL(caseClicked(int, int)), this, SLOT(boardClicked(int, int)));
			layout->addWidget(static_cast<kcaseWidget*>(cases[i][j]), j, i);
		}
	}
}

boardWidget::~boardWidget()
{
}

void boardWidget::newGame()
{
	board::newGame();
	changeStatusBar();
}

void boardWidget::endGame()
{
	board::endGame();
	changeStatusBar();
}

void boardWidget::changeStatusBar()
{
	if (currentPlayer == kcase::PLAYER1)
	{
		if (player1 == ktttSettings::EnumPlayer1::Human && player2 == ktttSettings::EnumPlayer2::Human) emit statusBarMessage(i18n("It is player 1's turn."));
		else if (player1 == ktttSettings::EnumPlayer1::Human && player2 == ktttSettings::EnumPlayer1::Computer) emit statusBarMessage(i18n("It is your turn."));
		else emit statusBarMessage(i18n("The computer is thinking, please wait."));
	}
	else if (currentPlayer == kcase::PLAYER2)
	{
		if (player1 == ktttSettings::EnumPlayer1::Human && player2 == ktttSettings::EnumPlayer1::Human) emit statusBarMessage(i18n("It is player 2's turn."));
		else if (player2 == ktttSettings::EnumPlayer1::Human && player1 == ktttSettings::EnumPlayer1::Computer) emit statusBarMessage(i18n("It is your turn."));
		else emit statusBarMessage(i18n("The computer is thinking, please wait."));
	}
	else emit statusBarMessage(i18n("Press Game -> New to start a new game."));
}

QFrame* boardWidget::getFrame()
{
	return frame;
}

bool boardWidget::processClassic(int x, int y)
{
	if (cases[x][y] -> isFree())
	{
		place(x, y);
		return true;
	}
	else return false;
}

void boardWidget::processPartial(int x, int y)
{
	if (currentPlayer == kcase::PLAYER1 && player1PiecesLeft > 0)
	{
		processTotal(x, y);
	}
	// player 2 still has pieces to play
	else if (currentPlayer == kcase::PLAYER2 && player2PiecesLeft > 0)
	{
		processTotal(x, y);
	}
	// currentPlayer selects a piece
	else if (currentPlayer == cases[x][y] -> getOcupation() && !selected)
	{
		processTotal(x, y);
	}
	// currentPlayer unselects the selected piece
	else if (currentPlayer == cases[x][y] -> getOcupation() && selected
			&& selected_x == x && selected_y == y)
	{
		processTotal(x, y);
	}
	// currentPlayer tries to move the selected piece
	else if (cases[x][y] -> isFree() && selected)
	{
		if(canMoveFrom(selected_x, selected_y, x, y))
		{
			processTotal(x, y);
		}
	}
}

void boardWidget::processTotal(int x, int y)
{
	// player 1 still has pieces to play
	if (currentPlayer == kcase::PLAYER1 && player1PiecesLeft > 0)
	{
		if (processClassic(x, y)) player1PiecesLeft--;
	}
	// player 2 still has pieces to play
	else if (currentPlayer == kcase::PLAYER2 && player2PiecesLeft > 0)
	{
		if (processClassic(x, y)) player2PiecesLeft--;
	}
	// currentPlayer selects a piece
	else if (currentPlayer == cases[x][y] -> getOcupation() && !selected)
	{
		select(x, y);
	}
	// currentPlayer unselects the selected piece
	else if (currentPlayer == cases[x][y] -> getOcupation() && selected
			&& selected_x == x && selected_y == y)
	{
		unselect();
	}
	// currentPlayer moves the selected piece
	else if (cases[x][y] -> isFree() && selected)
	{
		placeSelected(x, y);
	}
}

void boardWidget::nextTurn()
{
	board::nextTurn();
	changeStatusBar();
}

void boardWidget::readSettings()
{
	int newMode;
	int newPlayer1, newPlayer2;
	newMode = ktttSettings::game_mode();
	newPlayer1 = ktttSettings::player1();
	newPlayer2 = ktttSettings::player2();

	for (int i = 0; i < 3; i++)
	{
		for (int j = 0; j < 3; j++)
		{
			static_cast<kcaseWidget*>(cases[i][j]) -> updateColors();
		}
	}

	if (playing() && (newMode != currentMode || player1 != newPlayer1 || player2 != newPlayer2))
	{
		if (KMessageBox::questionYesNo(frame, "You have changed some settings that can not be applied to the current game. Do you want to apply them now (restarting the game)?") == KMessageBox::Yes)
			newGame();
	}
	else if (!playing())
	{
		currentMode = newMode;
		player1 = newPlayer1;
		player2 = newPlayer2;
		if (player1 == ktttSettings::EnumPlayer1::Computer)
		{
				setPlaying(true);
				moveComputer();
		}
	}
}

void boardWidget::boardClicked(int x, int y)
{
	// check if there is a game in play
	if (currentPlayer != kcase::NOBODY)
	{
		// check if is the turn of a human
		if (currentPlayer == kcase::PLAYER1 && player1 == ktttSettings::EnumPlayer1::Human ||
			currentPlayer == kcase::PLAYER2 && player2 == ktttSettings::EnumPlayer2::Human)
		{
			if (currentMode == ktttSettings::EnumGame_mode::Classic) processClassic(x, y);
			else if (currentMode == ktttSettings::EnumGame_mode::Partial) processPartial(x, y);
			else processTotal(x, y);
		}
	}
}

#include "boardwidget.moc"
