//Magazynier, a warehouse assistant game.
//Copyright (C) 2008, 2009 Tadeusz Andrzej Kadlubowski yess@hell.org.pl
//
//This program is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation; either version 2 of the License, or
//(at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License along
//with this program; if not, write to the Free Software Foundation, Inc.,
//51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

#include "levelPack.h"

#include <KConfig>
#include <KConfigGroup>
#include <KDebug>
#include <KGlobal>
#include <KStandardDirs>
#include <QFile>
#include <QRegExp>
#include <QTextStream>

#include "levelModel.h"

LevelPack::LevelPack(QString path) //{{{
{
	
	QRegExp r("/(.+)/1.txt");
	r.setMinimal(true);
	if (r.indexIn(path) != -1) {
		m_name = r.cap(1);
	} else {
		kDebug() << "Don't know how to create LevelPack from path" << path;
		return;
	}
	kDebug() << "Found LevelPack" << m_name;
	
	QStringList levels = KGlobal::dirs()->findAllResources("data",
			QString("magazynier/") + m_name + QString("/*.txt"),
			KStandardDirs::NoDuplicates|KStandardDirs::Recursive);
			
	kDebug() << levels.size() << "level(s) found.";
	m_levelCount = levels.size();

	KConfig config;
	KConfigGroup levelPacksGroup = config.group("LastSolved");
	QString lastSolved = levelPacksGroup.readEntry(m_name);
	if (not lastSolved.isNull()) {
		bool b = false;
		m_firstUnsolved = lastSolved.toInt(&b) + 1;		
		if (not b) {
			kDebug() << lastSolved << "as a last solved level number?";
			m_firstUnsolved = 1;
		}
	} else {
		m_firstUnsolved = 1;
	}
	m_currentLevel = m_firstUnsolved;
} //}}}

QString LevelPack::getName() //{{{
{
	return m_name;
} //}}}

int LevelPack::getLevelCount() //{{{
{
	return m_levelCount;
} //}}}

int LevelPack::getCurrentLevelNumber() //{{{
{
	return m_currentLevel;
} //}}}

int LevelPack::getSolvedLevelCount() //{{{
{
	return m_firstUnsolved - 1;
} //}}}

bool LevelPack::hasPrevLevels(void) //{{{
{
	kDebug() << "hasPrevLevels: m_currentLevel = " << m_currentLevel;
	return m_currentLevel > 1;
} //}}}

bool LevelPack::hasNextAvailableLevels(void) //{{{
{
	kDebug() << "hasNextAvailableLevels: m_currentLevel = " << m_currentLevel;
	return m_currentLevel < m_levelCount and m_currentLevel < m_firstUnsolved;
} //}}}

void LevelPack::loadPrevLevel(void) //{{{
{
	if (!hasPrevLevels()) {
		return;
	}

	loadLevelByNumber(m_currentLevel - 1);

	return;
} //}}}

void LevelPack::loadNextLevel(void) //{{{
{
	Q_ASSERT(hasNextAvailableLevels());

	loadLevelByNumber(m_currentLevel + 1);

	return;
} //}}}

void LevelPack::reloadCurrentLevel(void) //{{{
{
	loadLevelByNumber(m_currentLevel);
}
//}}}

void LevelPack::loadFirstUnsolvedLevel(void) //{{{
{
	Q_ASSERT(m_firstUnsolved > 0 && m_firstUnsolved <= m_levelCount);
	
	loadLevelByNumber(m_firstUnsolved);

	return;
} //}}}

void LevelPack::thisLevelPackSelected(void) //{{{
{
	kDebug() << "Selecting levelPack" << getName();
	
	KConfig config;
	KConfigGroup generalConfig = config.group("General");
	generalConfig.writeEntry("LastLevelPack", m_name);
	
	emit changeLevelPack(this);
} //}}}

void LevelPack::markCurrentLevelAsSolved(void) //{{{
{
	if (m_currentLevel < m_firstUnsolved) {
		return;
	}

	KConfig config;
	KConfigGroup levelPacksGroup = config.group("LastSolved");
	levelPacksGroup.writeEntry(m_name, m_currentLevel);
	
	kDebug() << "Level" << m_currentLevel << "is solved.";

	++m_firstUnsolved;
} //}}}

/*void LevelPack::markCurrentLevelAsLastPlayed(void)
{
}*/

void LevelPack::loadLevelByNumber(int n) //{{{
{
	Q_ASSERT(n > 0 && n <= m_levelCount);
	
	m_currentLevel = n;

	QString levelFileName = KStandardDirs::locate("data",
			QString("magazynier/%1/%2.txt").arg(m_name).arg(m_currentLevel));
	kDebug() << "loading level:" << levelFileName;
	
	QString levelData;
	QFile levelFile(levelFileName);
	if (levelFile.open(QFile::ReadOnly)) {
		QTextStream in(&levelFile);
		levelData = in.readAll();
		levelFile.close();
	} else {
		emit levelNotFound();
		return;
	}
	
	LevelModel *newLevelModel = new LevelModel();
	newLevelModel->initLevelData(levelData);

	emit levelLoaded(newLevelModel);
} //}}}

// vim: foldmethod=marker
