//Magazynier, a warehouse assistant game.
//Copyright (C) 2008, 2009 Tadeusz Andrzej Kadlubowski yess@hell.org.pl
//
//This program is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation; either version 2 of the License, or
//(at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License along
//with this program; if not, write to the Free Software Foundation, Inc.,
//51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

#include "renderer.h"

#include <math.h>

#include <KDebug>
#include <KGameRenderer>
#include <QPixmap>

#include "levelModel.h"
#include "settings.h"

Renderer::Renderer(QObject *parent) : QObject(parent) //{{{
{
	m_itemSize = 0;
	m_levelHeight = m_levelWidth = m_viewWidth = m_viewHeight = 0;
	m_gameRenderer = new KGameRenderer(Settings::self()->theme());
	loadTheme();
} //}}}

Renderer *Renderer::m_instance = NULL;
Renderer *Renderer::instance(void) //{{{
{
	if (m_instance == NULL) {
		m_instance = new Renderer(NULL);
	}
	return m_instance;
} //}}}

void Renderer::loadTheme(void) //{{{
{
	kDebug() << "loading graphics theme from" << Settings::self()->theme();
	m_gameRenderer->setTheme(Settings::self()->theme());
	return;
} //}}}

int Renderer::getItemSize(void) //{{{
{
	Q_ASSERT(m_itemSize > 0);
	return m_itemSize;
} //}}}

void Renderer::setItemSize() //{{{
{
	m_itemSize = (floor(m_viewHeight/m_levelHeight) < floor(m_viewWidth/m_levelWidth)) ?
					floor(m_viewHeight/m_levelHeight) : 
					floor(m_viewWidth/m_levelWidth);
	kDebug() << "Renderer will render" << m_itemSize << "x" << m_itemSize << "pixmaps";
} //}}}


QPixmap Renderer::getBackground() //{{{
{
	return m_gameRenderer->spritePixmap("Background", QSize(m_viewWidth, m_viewHeight));
} //}}}

void Renderer::setViewSize(int height, int width) //{{{
{
	m_viewHeight = height;
	m_viewWidth = width;
	kDebug() << "View size:" << height << "x" << width;
	sizeChanged();
	return;
} //}}}

void Renderer::setLevelSize(int height, int width) //{{{
{
	m_levelHeight = height;
	m_levelWidth = width;
	kDebug() << "Level size:" << height << "x" << width;
	sizeChanged();
	return;
} //}}}

void Renderer::sizeChanged() //{{{
{
	if (m_levelWidth > 0 && m_levelHeight > 0 && m_viewWidth > 0 && m_viewHeight > 0) {
		setItemSize();
	}
} //}}}

// vim: foldmethod=marker
