/***************************************************************************
 *   Copyright (C) 2008 Copyright 2008  Tom Vollerthun <vollerthun@gmx.de>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "ds/package.h"

#include <KLocale>

#include "ds/data.h"
#include "ds/description.h"

Package::Package()
{
    m_games = new QList<Data *>();

    m_name = i18n("Unknown package");
    m_description = i18n("No description available.");
    m_default_description = i18n("No description available.");

    m_filename = "";
}

void Package::insert(int i, Data * data)
{
    m_games->insert(i, data);
}

void Package::append(Data * data)
{
    m_games->append(data);
}

void Package::removeAt(int i)
{
    m_games->removeAt(i);
}

void Package::removeAll()
{
    for (int i = 0; i < m_games->size(); ++i) {
        m_games->removeLast();
    }
}

int Package::size()
{
    return m_games->size();
}

Data * Package::at(int i)
{
    Data * retval = m_games->at(i);

    /*Make the description contain some sane defaults*/
    if (retval->description() == NULL) {
        Description * emptyDescription = new Description();
        retval->setDescription(emptyDescription);
    }

    Description * d = retval->description();
    if (d->description() == NULL || d->description().isEmpty())
    {
        d->setDescription(m_default_description);
    }

    if (d->name() == NULL || d->name().isEmpty())
    {
        d->setName(i18n("Unknown game"));
    }

    if (d->hint() == NULL || d->hint().isEmpty())
    {
        d->setHint(i18n("No hint"));
    }

    return retval;
}

void Package::setName(const QString & name)
{
    m_name = name;
}

void Package::setDescription(const QString & description)
{
    m_description = description;
}

void Package::setDefaultDescription(const QString & defaultDescription)
{
    m_default_description = defaultDescription;
}


QString Package::name()
{
    return m_name;
}

QString Package::description()
{
    return m_description;
}

void Package::setFilename(const QString& name) {
    m_filename = name;
}

QString Package::filename() {
    return m_filename;
}


