/* ObjectItem.h

    Parsek - free client for playing 4X space strategy games
    Copyright (C) 2006-2007  Jure Repinc <jlp@holodeck1.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

/** @file
* This file defines the class ObjectItem which is used in ObjectsModel
* @author Jure Repinc <jlp@holodeck1.com>
*/

#ifndef OBJECTITEM_H
#define OBJECTITEM_H

#include <tpproto/object.h>

#include <QHash>
#include <QList>
#include <QVariant>

using namespace TPProto;

/** Class for representing individual space object in space objects model */
class ObjectItem
{
public:
    enum ObjectType { Unknown = -1, Universe, Galaxy, StarSystem, Planet, Fleet };
    ObjectItem(ObjectItem *parent = 0);
    ~ObjectItem();
    void appendChild(ObjectItem *child);
    ObjectItem *child(int row);
    int childrenCount() const;
    int propertiesCount() const;
    QVariant property(const QString &label) const;
    int row() const;
    ObjectItem *parent();

    quint32 id() const;
    void setId(quint32 id);
    quint32 parentid() const;
    void setParentId(quint32 id);
    QList<ObjectParameterGroup*> parameterslist() const;
    void setParameters(std::list<ObjectParameterGroup*> parameters);
    ObjectType type() const;
    void setType(ObjectType type);
    QString typestring() const;
    QString name() const;
    void setName(const QString &name);
    quint64 modTime() const;
    void setModTime(quint64 time);
    void setDescription(const QString &desc);
    QString description() const;
    QList<ObjectItem*> children;
private:
    ObjectItem *m_parent;
    quint32 m_id;
    quint32 m_parentid;
    QList<ObjectParameterGroup*> m_parameters;
    ObjectType m_type;
    QString m_name;
    QString m_description;
    quint64 m_modTime;
};

#endif // ObjectItem_H
