/* ObjectModel.h

    Parsek - free client for playing 4X space strategy games
    Copyright (C) 2006-2007  Jure Repinc <jlp@holodeck1.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

/** @file
* This file defines the class ObjectModel which is used to store a Universe
* @author Jure Repinc <jlp@holodeck1.com>
*/

#ifndef OBJECTMODEL_H
#define OBJECTMODEL_H

#include <QAbstractItemModel>
#include <QPixmap>
#include <boost/bind.hpp>
#include <tpproto/objectcache.h>
#include <tpproto/gamelayer.h>
#include <tpproto/object.h>

#include "objectitem.h"

using namespace TPProto;

/* Class for storing ObjectsItems and representing a Universe */

class ObjectModel : public QAbstractItemModel
{
    Q_OBJECT
public:
    ObjectModel(QObject *parent = 0);
    virtual ~ObjectModel();
    void refresh();
    void setGame(TPProto::GameLayer *game);
    void getUniverse();
    void receiveUniverse(boost::shared_ptr<Object> object);
    void getObject(unsigned int objectid, ObjectItem *parent);
    void receiveObject(boost::shared_ptr<Object> object, ObjectItem *parent);
    QModelIndex index(int row, int column, const QModelIndex &parent = QModelIndex() ) const;
    QModelIndex parent(const QModelIndex &index) const;
    int rowCount(const QModelIndex &parent = QModelIndex() ) const;
    int columnCount (const QModelIndex &parent = QModelIndex() ) const;
    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;
    QVariant headerData(int section, Qt::Orientation orientation,int role = Qt::DisplayRole) const;
    Qt::ItemFlags flags(const QModelIndex &index) const;
    ObjectItem* m_ObjectItemFromIndex (const QModelIndex &index) const;
private:

    ObjectItem* m_root;
    ObjectItem* m_universe;
    TPProto::GameLayer* m_game;
};

#endif // OBJECTMODEL_H
