#include "tile.h"
#include <QPainter>
#include <QSvgRenderer>
#include <QApplication>


Tile::Tile(int x, int y, QWidget *parent) : QFrame(parent)
{
  row=y;
  column=x;

  setMinimumSize(50,50);
  highlight=false;

  renderer= new QSvgRenderer(QString("default.svg"),this);
 }


void
Tile::mouseReleaseEvent(QMouseEvent* /*event*/)
{
  emit(clicked(value));
}

void
Tile::setValue(QChar letter)
{
  value=letter;
  repaint();
}

QChar
Tile::getValue()
{
  return value;
}

int
Tile::getRow()
{
  return row;
}

int
Tile::getColumn()
{
  return column;
}

QList<Tile *>
Tile::getNeighbours()
{
  return neighbours;
}

void
Tile::addNeighbour(Tile* neighbour)
{
  neighbours.append(neighbour);
}

void Tile::setHighlight(bool on)
{
  highlight=on;
}




QList<int>
Tile::listNeighbourIDs()
{
  int index,x,y;
  QList<int> neighbours;
  for (y=-1; y<=1; y++)
  {
	for (x=-1; x<=1; x++)
	{
	  index=boardIndex(column+x,row+y);
	  if (index >= 0)
	  {
		 neighbours.append(index);
	  }
	}
  }
  return neighbours;
}

void
Tile::paintEvent(QPaintEvent* /*event*/)
{
  QPainter painter(this);

  if(highlight)
  {
	renderer->render(&painter,"player_1");
  }
//	painter.fillRect(rect(),Qt::red);

  renderer->render(&painter,"lighting");


  painter.setPen(QApplication::palette().color(QPalette::Text));
  painter.setFont(QFont("Arial", height()/2));
  painter.drawText(rect(), Qt::AlignCenter, QString(value));

  paintedWith=highlight;
}

bool
Tile::needsRepaint()
{
  return highlight != paintedWith;
}

//TODO this uses a fixed board size
int
Tile::boardIndex(int x,int y)
{
  if (x<0 || x>=4)
	return -1;
  if (y<0 || y>=4)
	return -1;
  if(x==column && y == row)
	return -1;
  return y*4 + x;
}
