#include "wordkubes.h"
#include "wordkubes.h"
#include <KStandardGameAction>
#include <QLabel>
#include <QScrollArea>


void WordKubes::setupActions()
{
  //TODO set up the menus using this thingy
  //KStandardGameAction::gameNew(this, SLOT(newGame()), actionCollection);
  //KStandardGameAction::highscores(this,  SLOT(showHighscores()),  actionCollection);
  //KStandardGameAction::quit(this, SLOT(close()), actionCollection);
}

//TODO rig up a timer connected to the label
//TODO use the KGame start() and end() methods
WordKubes::WordKubes(QWidget *parent) : KMainWindow(parent)
{
  QMainWindow *widget = new QMainWindow( this );
  ui.setupUi(widget);
  setCentralWidget( ui.centralwidget );
  

  actionCollection = new QActionGroup( this );
  setupActions();

  ui.gameGrid->setLayout(&board);
  board.setSpacing(1);
  setupBoard();

  speller = new Sonnet::Speller("en");

  connect(ui.submitButton,SIGNAL(pressed()),SLOT(submitPressed()));
  connect(ui.lineEdit,SIGNAL(returnPressed()),SLOT(submitPressed()));
  connect(ui.lineEdit,SIGNAL(textEdited(QString)),SLOT(checkOnBoard(QString)));

  //181 = 3 minutes +1 because we take away a second when we update the clock
  connect(&gameTimer,SIGNAL(timeout()),this,SLOT(updateClock()));

  startGame();
}

//a new game is started
void
WordKubes::startGame()
{
  gameActive=true;
  populateBoard();

  gameClock=181;
  updateClock();
  gameTimer.start(1000);
 
  ui.lineEdit->setEnabled(true);
  ui.submitButton->setEnabled(true);

  ui.lineEdit->setFocus();
  ui.lineEdit->setText("");

  ui.wordList->clear();

  totalScore=0;
  updateScore();
}

void
WordKubes::endGame()
{
	wordsPlayed.clear();
	clearHighlight();
	gameActive=false;
	ui.timeRemaining->display("0");
	gameTimer.stop();
	ui.lineEdit->setEnabled(false);
	ui.submitButton->setEnabled(false);
	startGame();
}


WordKubes::~WordKubes()
{
  delete(speller);
}


void
WordKubes::updateClock()
{
  gameClock--;
  if(gameClock==0)
  {
	endGame();
  }
  else
  {
	ui.timeRemaining->display(gameClock);
  }
}





//TODO read a file with this in from the right language
//TODO this will only work for 16 sqaures
//this updates dice to have all the dice in it
//these get popped off so call every new game
void
WordKubes::loadDice()
{
  srand((unsigned)time(0));
  diceList=QString("EDNOSW,AACIOT,ACELRS,EHINPS,EEFHIY,ELPSTU,ACDEMP,GILRUW,EGKLUY,AHMORS,ABILTY,ADENVZ,BFIORX,DKNOTU,ABJMOQ,EGINTV").split(',');
}


QChar
WordKubes::getDiceLetter()
{
  //pick a dice that's left
  QString dice=diceList.takeAt( rand() % (diceList.size()) );
  //pick a letter from it
  return dice.at(rand() % 6).toLower();
}




bool
WordKubes::checkOnBoard(QString word)
{
  bool valid;
  clearHighlight();
  valid= checkOnBoard(tiles,word);
  //repaint everything that has changed
  for (int i=0;i<tiles.size();i++)
  {
	  if(tiles.at(i)->needsRepaint()) 
		tiles.at(i)->repaint();
  }
  return valid;
}

void
WordKubes::appendChar(QChar value)
{
  ui.lineEdit->setText(ui.lineEdit->text()+value);
  checkOnBoard(ui.lineEdit->text());
}

void
WordKubes::updateScore()
{
  ui.totalScore->setText("<b>Score:</b> " + QString::number(totalScore));
}

//TODO check doeesn't go on same square twice
bool
WordKubes::checkOnBoard(QList<Tile *> possibleTiles, QString remaining)
{
  if (remaining.size()==0)
  {
	return true;
  }

  QChar front = remaining.at(0);
  Tile* tile;
  remaining = remaining.right(remaining.size()-1);

  for (int i=0;i<possibleTiles.size();i++)
  {
	tile=possibleTiles.at(i);
	//if tile matches, and word is completable from this position
	if(tile->getValue()==front  && checkOnBoard(tile->getNeighbours(),remaining))
	{
	  tile->setHighlight(true);
	  return true;
	}
  }
  return false;
}

void
WordKubes::clearHighlight()
{
  for (int i=0;i<tiles.size();i++)
  {
	  tiles.at(i)->setHighlight(false);
  }
}


//TODO display this next to the word in question i.e "cow (1 point)" "asdf (not a valid word)" with some nice colours and such
void
WordKubes::submitPressed()
{
  QString text;
  QString error;

  text=ui.lineEdit->text();
  
  int score=getScore(text);

  //check word validity
  if(!score)
  {
	error=i18n("Word too Small");
  }
  else if(! checkOnBoard( text ))
  {
	error= i18n("Not On Board");
	score=0;
  }
  else if(speller->isMisspelled(text))
  {
	error=i18n("Only Real Words Allowed");
	score=0;
  }
  else if(wordsPlayed.contains(text))
  {
	error=i18n("Word already played");
	score=0;
  }

  if(! error.isNull())
	error=" (" + error + ")";

  wordsPlayed.append(text);
  totalScore+=score;

  //QLabel* newText=new QLabel(text + " " + QString::number(score) +  error);
  ui.wordList->insertItem(0,text + ' ' + QString::number(score) +  error);
  ui.lineEdit->clear();
  updateScore();
}

int
WordKubes::getScore(QString word)
{
  int size=word.size();
  //special bonus..
  if(word=="wordkubes")
	return 50;
  if(size<3)
	return 0;
  if(size<5)
	return 1;
  if(size<6)
	return 2;
  if(size<7)
	return 3;
  if(size<8)
	return 5;
  return 11;
}

void
WordKubes::populateBoard()
{
  loadDice();
  for (int i=0;i<tiles.size();i++)
  {
	  tiles.at(i)->setValue(getDiceLetter());
  }
}

void
WordKubes::setupBoard()
{
  int x,y;
  for (y=0; y<4; y++)
  {
	for (x=0; x<4; x++)
	{
	  Tile* label = new Tile(x,y);
	  label->resize(50,50);
	  connect(label,SIGNAL(clicked(QChar)),SLOT(appendChar(QChar)));
	  board.addWidget(label,y,x,Qt::AlignCenter);
	  tiles.append(label);
	}
  }
  populateNeighbours();
}

void WordKubes::populateNeighbours()
{
  QList<int> neighbours;
  int id;
  for (int i=0;i<tiles.size();i++)
  {
	  neighbours = tiles.at(i)->listNeighbourIDs();
	  while (! neighbours.isEmpty())
	  {
		id=neighbours.takeFirst();
		tiles.at(i)->addNeighbour(tiles.at(id));
	  }
  }
}

// #include "wordkubes.moc"
